      real*8 function cgaunt (j3, j1, m1, j2, m2)
c
c.....cgaunt - compute Gaunt coefficient for complex spherical harmonics
c
c.....The routine uses the Gaunt expansion for 3j symbols, returning:
c        j1 m1
c       c                = <j1,m1|j2,m2|j3,(m1-m2)>
c        j2 m2 j3 (m1-m2)
c     where |ji,mi> are (complex) spherical harmonics.
c
c     The routine assumes it is never called with wrong arguments; so,
c     the indices must fulfill:
c
c       |j1-j2| <= j3 <= (j1+j2),         (j1+j2+j3)/2 is integer,
c       |m1| <= j1,      |m2| <= j2,      |m1-m2| <= j3
c
c
      include           'implicit.inc'
      include           'fact.inc'
      include           'consts.inc'
                  
      integer           j3, j1, m1, j2, m2
c
      integer           m3, j, k, kmin, kmax, ik
      real*8            lncoef1, coef2, lndumy
c
c.....Local variables
c
      m3 = m1 - m2
      j = (j1+j2+j3) / 2
c
c.....Gaunt algorithm
c
      lncoef1 = facl(j1+j2-j3)+facl(j1-j2+j3)+facl(j2+j3-j1)+facl(j)
     &-(facl(j-j1)+facl(j-j2)+facl(j-j3)+facl(j+j+1))
     &+0.5d0*(facl(j1+m1)+facl(j2+m2)+facl(j3+m3)+facl(j1-m1)
     &+facl(j2-m2)+facl(j3-m3))
      kmin = max(0,j2-j3+m1,j1-j3+m2)
      kmax = min(j1+j2-j3,j1+m1,j2+m2)
      if (mod(kmin+m2+j3+j,2).eq.0) then
        ik = 1
      else
        ik = -1
      endif
      coef2 = 0d0
      do k = kmin, kmax
        lndumy = facl(k)+facl(j3-j2-m1+k)+facl(j3-j1-m2+k)
     &+facl(j1+j2-j3-k)+facl(j1+m1-k)+facl(j2+m2-k)
        coef2 = coef2 + ik *exp(lncoef1 - lndumy)
        ik = -ik
      enddo
      cgaunt = sqrt((j1+j1+1)*(j2+j2+1)*(j3+j3+1)/(4d0*pi))*coef2
c
c.....return
c
      return
      end function
