c
c-----------------------------------------------------------------------
c
      subroutine coulint (nai, nfi, rs, w, ivstar)
c
c.....input parameters:
c
c     nai ..... active center label
c     nfi ..... frozen center label
c     rs ...... distance between the two centers
c     ivstar ..
c
c.....output parameters:
c
c     w(lpq,lab) ..... integral between lpq and lab densities of the
c                      active and frozen centers, respectively
c
      include           'implicit.inc'
      include           'global.inc'
      include           'coed.inc'
      include           'iondens.inc'
      include           'comb.inc'
      include           'cont.inc' 
      include           'consts.inc'
      include           'dgaunt.inc'
      include           'fact.inc'
      include           'rsh.inc'
      include           'tfunc.inc'
      include           'symmetry.inc' 
      include           'star.inc'
c
c.....toosmall is a tested value for the difference between two orbital
c     exponents to consider them equal; toobig is a number such that the
c     hyperbolic functions will overflow, so exp(-alpha) is multiplied
c     in nu integrals, and divided in mu ones; conv is the relative
c     convergence required in nu integrals when they are evaluated as
c     infinite series
c
      real*8            toosmall, toobig, conv
      parameter ( toosmall = 1d-8 )
      parameter ( toobig = 500d0 )
      parameter ( conv = 1d-16 )
c
      integer            nai, nfi, ivstar
      integer            lp, lq, la, lb
      real*8             rs, w(mdens,mdens)
      real*8            rexp(4*namax+1), alfaexp(4*namax + maxl + 1)
      real*8            hnu(0:4*namax+maxl), hmu(0:4*namax+maxl+1)
      real*8            hmua(0:2*namax+maxmp)
      real*8            hmuf(0:2*namax+maxmp), xr(0:maxl+1)
      real*8            zzpq, zab, xxpq, xab, dpq, dab
      real*8            xinv, xexp, zabr, zpqr, alfa, beta
      real*8            sh, ch, binv, u1, u2, term
      real*8            chi1, chi2, chi3, chi4, binteg, cinteg
      real*8            dl, alaf, cfac, xfac, afac, afmen, afmas
      integer           nda, ndf,nlpq, nlab, igaunt, icoef
      integer           l1, l2, l3, m1, m2, ma, mb, mp, mq
      integer           labmas, labmenos, lpqmas, lpqmenos
      integer           l12mas, l12menos, isig
      integer           nmua, nmuf, nh, nm, ism, ismr
      integer           i, n, ir, i2r, is, it, iu, imu, inu
      integer           la0, lb0, lp0, lq0, l10, l20, l30
      integer           n0, ilim, istart, icomb, ifunc
      integer           nl1, nl2, im1, im2, im3, lm1, lm2
c
      logical           odd
c
c.....Number of densities of each ion
c
      ndf = ndens(nfi)
      nda = ndens(nai)
c
c.....Initialize the W matrix
c
      do lpq = 1, nda
        do lab = 1, ndf
          w(lpq,lab) = zero
        enddo
      enddo 
c
c.....Calculation of powers of the distance
c
      rexp(1) = rs
      do i = 2, 4*namax+1
        rexp(i) = rexp(i-1) * rs ! rexp(i)=rs**i
      enddo
c
c.....Loop over active ion densities
c
      do lpq = 1, nda
        nlpq = npq(lpq,nai) 
        zzpq = zpq(lpq,nai)
        zpqr = zzpq * rs 
        xxpq = xpq(lpq,nai)
        lp = denlp(lpq,nai)
        lq = denlq(lpq,nai)
        lp0 = il0(lp)
        lq0 = il0(lq)  
        lpqmas = lp + lq
        lpqmenos = abs(lp-lq)
c
c.....Mu integrals for the active ion
c
        nmua = nlpq + lpqmas
        xinv = one / zpqr
        xexp = exp(-zpqr) * xinv
        hmua(0) = xexp
        do n = 1, nmua
          hmua(n) = n*hmua(n-1)*xinv + xexp
        enddo

c
c.....Loop over frozen ion densities
c
        do lab = 1, ndf
          nlab = npq(lab,nfi) 
          zab = zpq(lab,nfi)
          zabr = zab * rs
          xab = xpq(lab,nfi)  
          la = denlp(lab,nfi)
          lb = denlq(lab,nfi)
          la0 = il0(la)
          lb0 = il0(lb)  
          labmas = la + lb
          labmenos = abs(la-lb)
c
c.....Mu integrals for the frozen ion
c
          nmuf = nlab + labmas
          xinv = one / zabr
          xexp = exp(-zabr) * xinv
          hmuf(0) = xexp
          do n = 1, nmuf
            hmuf(n) = n*hmuf(n-1)*xinv + xexp
          enddo
c
c.....Auxiliar variables for the mu and nu integrals
c
          alfa = (zpqr + zabr) * half
          beta = (zpqr - zabr) * half
          nh = nmuf + nmua
c
c.....Compute (2\alpha)^{-i} 
c
          alfaexp(1) = half / alfa
          do i = 2, nh+1
            alfaexp(i) = alfaexp(i-1) * alfaexp(1)
          enddo
c
c.....Compute basic integrals mu and nu
c
c.....(i) test special case beta < toosmall ---> zpq=zab
c
          if (abs(beta).lt.toosmall) then
            do is = 0, nh-1, 2
              hnu(is) = one / (is+1)
              hnu(is+1) = zero
            enddo
            if (mod(nh,2).eq.0) hnu(nh) = one / (nh+1)
c
c.....in this case, alfa=zpqr=zabr, so hmu=hmua=hmuf
c
            nh = nh + 1
            do n = 0, nmuf
              hmu(n) = hmuf(n)
            enddo
            do n = nmuf+1, nh
              hmu(n) = n*hmu(n-1)*xinv + xexp
            enddo
c
c.....(ii) beta is not zero --->  zpq .ne. zab
c
          else
c
c.....test special case beta > toobig, to include exp(-alfa) in nu
c     integrals
c
            xinv = one / alfa
            if (abs(beta).gt.toobig) then
              xexp = xinv
              sh = (exp(beta-alfa) - exp(-beta-alfa)) * half
              ch = (exp(beta-alfa) + exp(-beta-alfa)) * half
            else
              xexp = exp(-alfa) * xinv
              sh = (exp(beta) - exp(-beta)) * half
              ch = (exp(beta) + exp(-beta)) * half
            endif
c
c.....calculation of mu integrals
c
            hmu(0) = xexp
            do n = 1, nh
              hmu(n) = n*hmu(n-1)*xinv + xexp
            enddo
c
c.....classify nu integrals for up or down recursion
c
            ilim = min(int(abs(beta))-1,nh)
            odd = mod(ilim+2,2).eq.1
c
c.....run up recursion from 0 to ilim
c
            if (ilim.ge.0) then
              binv = one / beta
              hnu(0) = sh * binv
              do is = 1, ilim-1, 2
                hnu(is) = (ch - is*hnu(is-1)) * binv
                hnu(is+1) = (sh - (is+1)*hnu(is)) * binv
              enddo
              if (odd) hnu(ilim) = (ch - ilim*hnu(ilim-1)) * binv
            endif
c
c.....run down recursion from nh to ilim+1
c
            if (ilim.ne.nh) then
c
c.....sum the top element until convergence
c
              it = nh + 1
              term = one / it
              u1 = term
              u2 = zero
              do while (abs(term).gt.conv*abs(u1))
                it = it + 1
                term = term * beta / it
                u2 = u2 + term
                it = it + 1
                term = term * beta / it
                u1 = u1 + term
              enddo
c
c.....now follow the down recursion
c
              if (mod(nh,2).eq.1) then
                hnu(nh) = u1*sh - u2*ch
                istart = nh - 1
              else
                hnu(nh) = u1*ch - u2*sh
                if (nh.ne.ilim+1)
     &            hnu(nh-1) = (sh - beta*hnu(nh)) / nh
                istart = nh - 2
              endif
              do is = istart, ilim+2, -2
                hnu(is) = (ch - beta*hnu(is+1)) / (is+1)
                hnu(is-1) = (sh - beta*hnu(is)) / is
              enddo
              if (odd .and. nh.ne.ilim+1) hnu(ilim+1) =
     &          (ch - beta*hnu(ilim+2)) / (ilim+2)
            endif
          endif
c
c.....Loop over different angular momentum couplings, adding
c     to the coulomb integrals
c
          do l1 = lpqmenos, lpqmas, 2
            nl1 = nlpq - l1 - 1
            l10 = il0(l1)
            do l2 = labmenos, labmas, 2
              nl2 = nlab - l2 - 1
              l20 = il0(l2)
              l12mas = l1 + l2
              l12menos = abs(l1-l2)
              icoef = indcoefe(l1,nl1,l2,nl2)
c
c.....(A) precompute Xr integrals
c
              nm = nl1 + nl2
              do ir = 0, l12mas+1
                i2r = ir + ir
                xr(ir) = zero
                imu = nm + i2r 
                do inu = 0, nm+i2r
                  xr(ir) = xr(ir) + coefe(inu,ir,icoef) * hnu(inu)
     &                              * hmu(imu)
                  imu = imu - 1
                enddo
              enddo
c
c.....(B) carry on the l3 sum
c
              ism = (l1+l2+l12menos)/2
              do l3 = l12menos, l12mas, 2
                l30 = il0(l3)
                ism = ism + 1
c
c.....(i) compute the Alaf integral (the R^{Nmax+2} is a common factor)
c
                alaf = zero
                ismr = ism 
                do ir = 0, ism
                  alaf = alaf + coefd(ismr,l3) * xr(ir)
                  ismr = ismr - 1
                enddo
c
c.....(ii) compute A^{l1,l2,l3} coefficient, saving the first and last
c     coefficients for the C and B integrals
c
                afac = zero
                do m1 = -l1, l1
                  lm1 = l10 + m1
                  xfac = zero
                  do m2 = -l2, l2
                    lm2 = l20 + m2
                    ifunc = itfunc(lm1,lm2,l3)
                    if (ivstar.eq.3) then
                      xfac = xfac + udstar(lab,lm2)*tfunc(ifunc)
                    else
                      isig = 1
                      if (ivstar.eq.2 .and. mod(l2,2).eq.1) isig = -1 
                      xfac = xfac + isig*udpq(lab,lm2,nfi)*tfunc(ifunc)
                    endif
                  enddo
                  afac = afac + udpq(lpq,lm1,nai)*xfac
                enddo 
                if (l3.eq.l12menos) afmen = afac
                if (l3.eq.l12mas) afmas = afac
                w(lpq,lab) = w(lpq,lab) - afac * alaf
              enddo ! l3
c
c.....(C) compute the B integrals; R^{n1+n2+l1+l2+2} is a common factor
c
              nm = nlpq + nlab + l12mas
c
c.....(i) \zeta_1 = \zeta_2
c
              if (abs(beta).lt.toosmall) then
c
c.....(a) \chi^IIIsr
c
                chi3 = - ( hmu(nm+1) 
     &                    / ((nm+1)*comb(((nm-1)*(nm+2))/2+nlab+l2+1)) )
c
c.....(b) \chi^IV
c
                chi4 = zero
                do iu = 0, nm
                  chi4 = chi4 + coefc(nlpq+l1,nlab+l2,iu) *
     &                          hmu(nm-iu) * hnu(iu)
                enddo
                binteg = cdelta(l1,l2) * (chi3 + chi3 + chi4) 
c
c.....(ii) \zeta_1 \ne \zeta_2
c
              else
c
c.....(a) \chi^I
c
                chi1 = zero
                icomb = ((nlpq+l1-1)*(nlpq+l1+2))/2 + 1
                do iu = 0, nlpq+l1
                  chi1 = chi1 + comb(icomb+iu) * fact(nm-iu) *
     &                          hmua(iu) * alfaexp(nm+1-iu)
                enddo
c
c.....(b) \chi^II
c
                chi2 = zero
                icomb = ((nlab+l2-1)*(nlab+l2+2))/2 + 1
                do iu = 0, nlab+l2
                  chi2 = chi2 + comb(icomb+iu) * fact(nm-iu) *
     &                          hmuf(iu) * alfaexp(nm+1-iu)
                enddo
c
c.....(c) \chi^IV
c
                chi4 = zero
                do iu = 0, nm
                  chi4 = chi4 + coefc(nlpq+l1,nlab+l2,iu) *
     &                          hmu(nm-iu) * hnu(iu)
                enddo
c
c.....(d) add together all contributions
c
                binteg = - cdelta(l1,l2) * (chi1+chi1+chi2+chi2+chi4)
              endif
c
c.....(D) compute C integral; R^{n1+n2+abs(l1-l2)+2} is a common factor
c
              nm = nlpq + nlab - l12menos - 1
c
c.....(i) l1 > l2
c
              if (l1.gt.l2) then
c
c.....(a) chi^I
c
                chi1 = zero
                icomb = ((nl1-1)*(nl1+2))/2 + 1
                do iu = 0, nl1
                  chi1 = chi1 + comb(icomb+iu) * fact(nm-iu) *
     &                          hmua(iu) * alfaexp(nm+1-iu)
                enddo
c
c.....(b) \chi^IV
c
                chi4 = zero
                do iu = 0, nm
                  chi4 = chi4 + coefc(nl1,nlab+l2,iu) *
     &                          hmu(nm-iu) * hnu(iu)
                enddo
c
c.....(c) add together all contributions
c
                cinteg = cdelta(l2,l12menos) * (chi1+chi1+chi4)
c
c.....(ii) l1 < l2
c
              else if (l1.lt.l2) then
c
c.....(a) \chi^II
c
                chi2 = zero
                icomb = ((nl2-1)*(nl2+2))/2 + 1
                do iu = 0, nl2
                  chi2 = chi2 + comb(icomb+iu) * fact(nm-iu) *
     &                          hmuf(iu) * alfaexp(nm+1-iu)
                enddo
c
c.....(b) \chi^IV
c
                chi4 = zero
                do iu = 0, nm
                  chi4 = chi4 + coefc(nlpq+l1,nl2,iu) *
     &                          hmu(nm-iu) * hnu(iu)
                enddo
c
c.....(c) add together all contributions
c
                cinteg = cdelta(l1,l12menos) * (chi2+chi2+chi4)
c
c.....(iii) l1 = l2
c
              else
c
c.....(a) chi^I
c
                chi1 = zero
                icomb = ((nl1-1)*(nl1+2))/2 + 1
                do iu = 0, nl1
                  chi1 = chi1 + comb(icomb+iu) * fact(nm-iu) *
     &                          hmua(iu) * alfaexp(nm+1-iu)
                enddo
c
c.....(b) \chi^II
c
                chi2 = zero
                icomb = ((nl2-1)*(nl2+2))/2 + 1
                do iu = 0, nl2
                  chi2 = chi2 + comb(icomb+iu) * fact(nm-iu) *
     &                          hmuf(iu) * alfaexp(nm+1-iu)
                enddo
c
c.....(c) \chi^IV
c
                chi4 = zero
                do iu = 0, nm
                  chi4 = chi4 + (coefc(nl1,nlab+l2,iu) +
     &                           coefc(nlpq+l1,nl2,iu)) *
     &                          hmu(nm-iu) * hnu(iu)
                enddo
c
c.....(d) add together all contributions
c
                cinteg = cdelta(l1,0) * (chi1+chi1+chi2+chi2+chi4)
              endif
c
c.....(E) Add B and C integrals inside W matrix 
c
              w(lpq,lab) = w(lpq,lab) + binteg * afmas + cinteg * afmen
            enddo ! l2
          enddo ! l1
c
c.....Finally, we multiply the W matrix by the constant factor
c
          isig = 1
          if (mod(lp+lq,2).eq.1) isig = -1
          cfac = isig * 8d0 * pisquare * xxpq * xab 
     &           * rexp(nlpq+nlab+1)
          w(lpq,lab) = cfac * w(lpq,lab) 
c
        enddo ! lab
      enddo ! lpq
      return
      end
