c
c-----------------------------------------------------------------------
c
      subroutine create (nfirst, nlast)
c
c.....create - use the symmetry information to build up the unit cell
c              from the minimum number of non-equivalent atoms.
c
c ncel   ... number of atoms in the crystallographic cell
c mcel   ... max number of atoms in the unit cell
c ilocal ... number of local symmetry operations for each
c            inequivalent atom (new)
c neqv   ... number of Seitz operators {R|t}, where R are only proper
c            rotations
c mrot   ... number of Seitz operators {R|t}, where R are both proper
c            and improper rotations / number of different positions in
c            the unit cell (without the action of centering vectors)
c            for each inequivalent atom
c rotm(3,4,24) ... Seitz operators (matrix representation), where R are
c                  only proper rotations
c xxneq(mnt,3) ... Wyckoff positions for the inequivalent atoms
c vec(mcel,3)  ... Different positions (without the action of centering
c                  vectors) in the unit cell for each inequivalent atom
c indaux(mcel) ... Pointer that indicates the rotation (proper or improper) that
c                  generates the current position from the Wyckoff position 
c                  of the representative atom
c                  indaux < 0 -> improper rotations
c                  indaux > 0 -> proper rotations
c                  (new)
c indcel(mcel) ... idem indaux, but this is the final pointer for the
c                  different atoms in the unit cell (new)
c rotloc(3,3,24,mnt) ... Local symmetry operations for the
c                        inequivalent atoms (new)
c nlocal(mnt)  ... number of local symmetry operations for the
c                  inequivalent atoms (new)
c ncent  ... 0 -> non centrosymmetric lattices
c            1 -> centrosymmetric lattices
c ncv    ... number of centering vectors
c
      include           'implicit.inc'
      include           'global.inc'
      include           'consts.inc'
      include           'neqatm.inc'
      include           'error.inc'
      include           'celatm.inc'
      include           'spgcom.inc'
      include           'symloc.inc'
c
      dimension         vec(mcel,3), indaux(mcel) 
      logical           val
c
      ncel = 0 
      do i = nfirst, nlast
         ilocal = 0  
         mrot = 0
         do j = 1, neqv 
            mrot = mrot + 1
            if (mrot .gt. mcel)
     &         call error ('create', 'mrot > mcel!', faterr)
c........STEP I: Action of Seitz operators on the Wyckoff positions
c                for the inequivalent atoms
            val = .true.
            do l = 1, 3
               vec(mrot,l) = zero
c..............Ia: Proper rotation 
               do m = 1, 3
                  vec(mrot,l) = vec(mrot,l) + rotm(l,m,j) * xxneq(i,m)
               enddo
c..............Ib: Traslation, where 0<=rotm(l,4,j)<=1 (if it exist)
               vec(mrot,l) = vec(mrot,l) + rotm(l,4,j)
               val = val .and. abs(rotm(l,4,j)).lt.EPSFRAC
c..............Ic: Move created points to the zeroth cell
               if (abs(vec(mrot,l)-nint(vec(mrot,l))).le.EPSFRAC) then
                  vec(mrot,l) = zero
               else if (vec(mrot,l).lt.zero) then
                  vec(mrot,l) = vec(mrot,l)+1
               else if (vec(mrot,l).ge.one) then
                  vec(mrot,l) = vec(mrot,l)-1
               endif
            enddo
            indaux(mrot) = j ! proper rotation 
c........STEP II: Identification of the local symmetry operation
c                 for the inequivalent atoms
c...........IIa: Proper rotations
            if (abs(vec(mrot,1)-xxneq(i,1)).lt.EPSFRAC .and.
     &          abs(vec(mrot,2)-xxneq(i,2)).lt.EPSFRAC .and.
     &          abs(vec(mrot,3)-xxneq(i,3)).lt.EPSFRAC .and. val) then
*            if (abs(vec(mrot,1)-xxneq(i,1)).lt.EPSFRAC .and.
*     &          abs(vec(mrot,2)-xxneq(i,2)).lt.EPSFRAC .and.
*     &          abs(vec(mrot,3)-xxneq(i,3)).lt.EPSFRAC) then
              ilocal = ilocal + 1
              do l = 1, 3
                do m = 1, 3
                  rotloc(l,m,ilocal,i) = rotm(l,m,j)
                enddo
              enddo
            endif 
c........STEP III: Test for centrosymmetric lattices
            if (ncent.eq.1) then
               mrot = mrot + 1
               if (mrot .gt. mcel)
     &            call error ('create', 'mrot > mcel!', faterr)
               do l = 1, 3
                   aux = -vec(mrot-1,l)
                   if (abs(aux-nint(aux)).le.EPSFRAC) then
                      vec(mrot,l) = zero
                   else if (aux.lt.zero) then
                      vec(mrot,l) = aux + 1
                   else
                      vec(mrot,l) = aux
                   endif
               enddo
               indaux(mrot) = -j ! improper rotation
c..............IIb: Improper rotations
               if (abs(vec(mrot,1)-xxneq(i,1)).lt.EPSFRAC .and.
     &             abs(vec(mrot,2)-xxneq(i,2)).lt.EPSFRAC .and.
     &             abs(vec(mrot,3)-xxneq(i,3)).lt.EPSFRAC .and. 
     &             val) then
*               if (abs(vec(mrot,1)-xxneq(i,1)).lt.EPSFRAC .and.
*     &             abs(vec(mrot,2)-xxneq(i,2)).lt.EPSFRAC .and.
*     &             abs(vec(mrot,3)-xxneq(i,3)).lt.EPSFRAC) then
                 ilocal = ilocal + 1
                 do l = 1, 3
                   do m = 1, 3
                     rotloc(l,m,ilocal,i) = - rotm(l,m,j)
                   enddo
                 enddo
               endif
            endif
         enddo ! j = 1, ..., neqv
         nlocal(i) = ilocal
c........STEP IV: Destroy identical atoms
         call uniq (vec, indaux, 1, mrot) ! modifications
c........STEP V: Create centered positions 
         natact = ncel
         do k = 1, mrot
            do j = 1, ncv
               ncel = ncel + 1
               if (ncel .gt. mcel)
     &            call error ('create', 'ncel > mcel!', faterr)
               indcel(ncel) = indaux(k) 
               do l = 1, 3
                  aux = vec(k,l) + cen(l,j)
                  if (abs(aux-nint(aux)).le.EPSFRAC) then
                     xxcel(ncel,l) = zero
                  else if (aux.lt.zero) then
                     xxcel(ncel,l) = aux + one
                  else if (aux.ge.one) then
                     xxcel(ncel,l) = aux - one
                  else
                     xxcel(ncel,l) = aux
                  endif
               enddo
            enddo
         enddo
c........STEP VI: Destroy identical atoms
         call uniq (xxcel, indcel, natact+1, ncel)
c........STEP VII: Assign type
         do m = natact+1, ncel
            typcel(m) = i
            qcel(m) = qneq(i)
         enddo
c........STEP VIII: Multiplicity of the position
         mult(i) = ncel - natact
      enddo ! i = nfirst, ..., nlast
      return
      end
