c
c.....dgaunt.inc - Gaunt coefficients between real spherical harmonics
c     data structures
c
c     Non-zero Gaunt coefficients d_{l1m1,l2m2}^{l3m3} will be stored
c     on a correlative vector, accessed via m3num, m3val and ig0; for
c     example, accessing all non-zero coefficients for given l1m1, l2m2
c     and l3 will be done as:
c
c     i = ig0(lm1,lm2,l3)
c     do im3 = 1, m3num(m1,m2,l3)
c       d_{l1m1,l2m2}^{l3m3} = dgaunt(i+im3)
c       m3 = m3val(m1,m2,im3)
c     enddo
c
c     lm1 and lm2 are the indices for real spherical harmonics S_{l,m}
c     stored as a supervector, given by (l*(l+1)+m)
c
c     m3num(m1,m2,l3) . number of non-zero coefficients with m1 and m2
c                       for l3
c     m3val(m1,m2,im3)  m3 value for im3'th non-zero coefficient with
c                       given m1 and m2
c     ig0() ........... index to the last element of the dgaunt vector
c                       before d_{lm1,lm2}^{l3m3} elements
c     dgaunt() ........ vector with non-zero Gaunt coefficients
c     maxgaunt ........ number of non-zero Gaunt coefficients; this is
c                       a tested value which covers Coulomb integrals:
c                       * up to and including four f orbitals = 6408
c                       * up to and including four g orbitals = 22661
c                       * up to and including four l=5 orbitals = 62152
c                       * up to and including four l=6 orbitals = 143927
c
      integer           maxgaunt
      parameter         ( maxgaunt = 6408 )
      integer           m3num(-maxmp:maxmp,-maxmp:maxmp,0:maxl)
      integer           m3val(-maxmp:maxmp,-maxmp:maxmp,2)
      integer           ig0(0:maxmp*(maxmp+2),0:maxmp*(maxmp+2),0:maxl)
      real*8            dgaunt(maxgaunt)
      common /dgaunti/  m3num, m3val, ig0
      common /dgauntr/  dgaunt
