c
c-----------------------------------------------------------------------
c
      subroutine edef (line, lp, luw, nimpre)
c
c.....edef - computes the deformation energy of an ion.
c
c.....Input parameters:
c     line ..... 'EDEF' order line.
c     lp ....... parsing position on this line.
c     luw ...... output logical unit.
c     nimpre ... print level of this run.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'consts.inc'
      include           'cont.inc'
      include           'corr.inc'
      include           'error.inc'
      include           'files.inc' 
      include           'intg.inc'
      include           'neqatm.inc'
      include           'scf.inc'
      include           'types.inc'
      include           'vef.inc'
c
      character*(ilen)  line
      integer           lp, luw, nimpre
c
      logical           setword
      real*8            dtvac(mdens), dosvac(mdens)
c
c.....Begin the run:
c
      call prtimer (2, ipid, 'edef', luw)
      if (nimpre.ge.0) write (luw,600)
c
c.....Parse the input line
c
      if (.not. setword (fion(1), line, lp)) call error ('edef',
     &   'first ionic filename missing', faterr)
      if (.not. setword (fulm(1), line, lp)) call error ('edef',
     &   'first ionic filename missing', faterr)
      if (.not. setword (fcoef(1), line, lp)) call error ('edef',
     &   'first ionic filename missing', faterr)
      if (.not. setword (fion(2), line, lp)) call error ('edef',
     &   'second ionic filename missing', faterr)
      if (.not. setword (fulm(2), line, lp)) call error ('edef',
     &   'first ionic filename missing', faterr)
      if (.not. setword (fcoef(2), line, lp)) call error ('edef',
     &   'first ionic filename missing', faterr)
c
c.....Read in vacuo description, and save density matrices
c
      call readion (fion(2), 2, -4, luw)
      call readulm (fulm(2), 2, -4, luw)
      call readcoef (fcoef(2), 2, -4, luw) 
      call densi (2)
      do lpq = 1, ndens(2)
        dtvac(lpq) = dt(lpq)
        dosvac(lpq) = dos(lpq)
      enddo
c
c.....Compute the monocentric matrices for first ion
c
      call readion (fion(1), 1, -4, luw)
      call readulm (fulm(1), 1, -4, luw)
      call readcoef (fcoef(1), 1, -4, luw)
      call istupq ("edef.int", 1, typcorr, nwint, luw, nimpre)
      call densi (1)
c
c.....Compute the deformation energy (see energ.f)
c
      e2e = zero
      enuc = zero
      ekin(1) = zero
      k = 0
      do i = 1, ndens(1)
         enuc = enuc + u(i) * (dt(i)-dtvac(i))
         ekin(1) = ekin(1) + t(i) * (dt(i)-dtvac(i))
         do j = 1, i-1
            k = k + 1
            e2e = e2e + (p(k)*(dt(i)*dt(j)-dtvac(i)*dtvac(j))
     &                 - q(k)*(dos(i)*dos(j)-dosvac(i)*dosvac(j)))
         enddo
         k = k + 1
         e2e = e2e + half * (p(k)*(dt(i)*dt(i)-dtvac(i)*dtvac(i))
     &                     - q(k)*(dos(i)*dos(i)-dosvac(i)*dosvac(i)))
      enddo
      enet = ekin(1) + enuc + e2e
c
c.....compute the DFT contributions if requested
c
c      if (typcorr.eq.typWig .or. typcorr.eq.typGom .or.
c     &    typcorr.eq.typLYP .or. typcorr.eq.typBR) then
c        do lpq = 1, ndens(2)
c          rhomat(lpq,1) = dt(lpq)
c          rhomat(lpq,2) = dtvac(lpq)
c        enddo
c        call correl (1, luw, nimpre)
c        call correl (2, luw, nimpre)
c        difcorr = ecorr(1) - ecorr(2)
c        e2e = e2e + difcorr
c        enet = enet + difcorr
c      endif
c
c.....Print out the results
c
      if (nimpre.ge.0) then
         write (luw,610) namneq(1), namneq(2)
         write (luw,620) ekin(1), enuc, e2e, enet
      else
         write (luw,630) enet
      endif
c
c.....End of calculation:
c
      if (nimpre.ge.0) write (luw,700)
      call prtimer (4, ipid, 'edef', -1)
      return
c
c.....Formats
c
 600  format (//1x, '*** EDEF - deformation energy calculation'/)
 610  format (/1x, 'Deformation of [', a, '] with respect to [', a,
     & '] (in vacuo) '/)
 620  format (/1x, 'Kinetic deformation energy            = ', 1pe17.9,
     &        /1x, 'Nuclear attraction deformation energy = ', 1pe17.9,
     &        /1x, 'Bielectronic deformation energy = ', 1pe17.9,
     &        /1x, 'Total deformation energy              = ', 1pe17.9/)
 630  format (1x, 'Edef = ', 1pe17.9)
 700  format (/1x, '*** EDEF: End of run'/)
      end
