c
c-----------------------------------------------------------------------
c
      subroutine energ (ni)
c
c.....energ - computes the effective and additive energy of the ion.
c
c     The different contributions to the energy are:
c
c     a) Monoelectronics:
c        - kinetic energy :          T() DT()
c        - nuclear attraction:       U() DT()
c        - ion-lattice interaction:  V() DT()
c
c     b) Bielectronics:   DT+() P() DT() - DOS+() Q() DOS()
c
c     c) Nucleus-lattice interaction:   EnucRed = Vred(0) ZN
c        This contribution is computed in IntRed.
c
c     The additive energy contains only one half of the ion-lattice
c     interaction contributions.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'cont.inc'
      include           'consts.inc'
      include           'intg.inc'
      include           'lint.inc'
      include           'scf.inc'
c
      eint(ni) = zero
      ekin(ni) = zero
      emp      = zero
      ensr     = zero
      eesr     = zero
      eex      = zero
      epl      = zero
c
c.....bielectronic energy:
c
      e2e = zero
      k = 0
      do i = 1, ndens(ni)
         do j = 1, i-1
            k = k + 1
            e2e = e2e + (dt(i)*p(k)*dt(j) - dos(i)*q(k)*dos(j))
         enddo
         k = k + 1
         e2e = e2e + half * (dt(i)*p(k)*dt(i) - dos(i)*q(k)*dos(i))
      enddo
c
c.....monoelectronic energy (includes interaction with the lattice):
c
      enuc = zero
      do i = 1, ndens(ni)
         ekin(ni) = ekin(ni) + dt(i) * t(i)
         enuc     = enuc     + dt(i) * u(i)
         eint(ni) = eint(ni) + dt(i) * v(i)
         emp      = emp      + dt(i) * vmp(i)
         ensr     = ensr     + dt(i) * vnsr(i)
         eesr     = eesr     + dt(i) * vesr(i)
         eex      = eex      + dt(i) * vex(i)
         epl      = epl      + dt(i) * pla(i)
      enddo
      enet     = ekin(ni) + enuc + e2e
      eint(ni) = eint(ni) + enllr + enlsr
      epot(ni) = enuc + e2e + eint(ni)
      eeff(ni) = enet + eint(ni)
      eadd(ni) = enet + half * eint(ni)
      ecorr(ni) = zero
c
      end
