c
c-----------------------------------------------------------------------
c
      subroutine energsym (ni,luw)
c
c.....energsym - computes the effective and additive energy of the ion
c             associated with each irrep.
c
c     The different contributions to the energy are:
c
c     a) Monoelectronics:
c        - kinetic energy :          T() DT()
c        - nuclear attraction:       U() DT()
c        - ion-lattice interaction:  V() DT()
c
c     b) Bielectronics:   DT+() P() DT() - DOS+() Q() DOS()
c
c     c) Nucleus-lattice interaction:   EnucRed = Vred(0) ZN
c        This contribution is computed in IntRed.
c
c     The additive energy contains only one half of the ion-lattice
c     interaction contributions.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'cont.inc'
      include           'consts.inc'
      include           'intg.inc'
      include           'lint.inc'
      include           'scf.inc'
      include           'symmetry.inc'
c
      real*8    ekinsym, enucsym, eintsym
      real*8    empsym, ensrsym, eesrsym, eexsym, eplsym
      real*8    e2esym(mirrep)             
c
c.....Monoelectronic energy (includes interaction with the lattice):
c
      write (luw,50) 
      do irrep = 1, nirrep(ni)
        ekinsym = zero
        enucsym = zero
        eintsym = zero
        empsym  = zero
        ensrsym = zero
        eesrsym = zero
        eexsym  = zero
        eplsym  = zero
        lpq = nadens(irrep,ni)
        do i = 1, ndensym(irrep,ni)
          lpq = lpq + 1
          ekinsym = ekinsym + dt(lpq) * t(lpq)
          enucsym = enucsym + dt(lpq) * u(lpq)
          eintsym = eintsym + dt(lpq) * v(lpq)
          empsym  = empsym  + dt(lpq) * vmp(lpq)
          ensrsym = ensrsym + dt(lpq) * vnsr(lpq)
          eesrsym = eesrsym + dt(lpq) * vesr(lpq)
          eexsym  = eexsym  + dt(lpq) * vex(lpq)
          eplsym  = eplsym  + dt(lpq) * pla(lpq)
        enddo
        write (luw,100) label(irrep,ni), ekinsym, enucsym, eintsym, 
     &                  empsym, ensrsym, eesrsym, eexsym, eplsym
      enddo
c
c.....Bielectronic energy:
c
      write (luw,60) (label(irrep,ni), irrep = 1, nirrep(ni))
      do irrep1 = 1, nirrep(ni)
        do irrep2 = 1, irrep1
          e2esym(irrep2) = zero
          if (irrep1.ne.irrep2) then
            lpq = nadens(irrep1,ni)
            do i = 1, ndensym(irrep1,ni)
              lpq = lpq + 1
              lab = nadens(irrep2,ni)
              do j = 1, ndensym(irrep2,ni)
                lab = lab + 1
                k = (lpq*(lpq-1)/2) + lab
                e2esym(irrep2) = e2esym(irrep2) +
     &               (dt(lpq)*p(k)*dt(lab) - dos(lpq)*q(k)*dos(lab))
              enddo
            enddo
          else
            lpq = nadens(irrep1,ni)
            do i = 1, ndensym(irrep1,ni)
              lpq = lpq + 1
              lab = nadens(irrep1,ni)
              do j = 1, i-1
                lab = lab + 1
                k = (lpq*(lpq-1)/2) + lab
                e2esym(irrep2) = e2esym(irrep2) +
     &               (dt(lpq)*p(k)*dt(lab) - dos(lpq)*q(k)*dos(lab))
              enddo
              k = (lpq*(lpq+1)/2)
              e2esym(irrep2) = e2esym(irrep2) + half *
     &             (dt(lpq)*p(k)*dt(lpq) - dos(lpq)*q(k)*dos(lpq))
            enddo
          endif
        enddo
        write (luw,200) label(irrep1,ni),
     &                  (e2esym(irrep2), irrep2 = 1, irrep1)
      enddo
      return
c
c.....Formats:
c
 50   format (/1x,'*** Partition of 1-e effective energy terms into',
     &        /1x,'    symmetry contributions',
     &        /1x,'---Sym--------ekin--------enuc--------eint',
     &            '--------emp--------ensr--------eesr--------eex',
     &            '--------epl--------')
 100  format (1x,a6,1x,e21.14,1x,e21.14,1x,e21.14,1x,e21.14,
     &        1x,e21.14,1x,e21.14,1x,e21.14,1x,e21.14)
 60   format (/1x,'*** Partition of 2-e net energy terms into',
     &        /1x,'    symmetry contributions',
     &        /1x,'sym1/sym2',1x,10a6)
 200  format (1x,a6,1x,10e21.14)
c
      end
