c
c-----------------------------------------------------------------------
c
      subroutine envclus (ni, nimpre, luw)
c
c.....envclus - build succesive shells around an ion in a cluster. 
c
c.....parameters:
c     ni ...... ion around which the environment should be built.
c     nimpre .. print level.
c     luw ..... output logical unit.
c
      include           'implicit.inc'
      include           'global.inc'
c
      include           'celatm.inc'
      include           'consts.inc'
      include           'cont.inc'
      include           'converge.inc'
      include           'environ.inc'
      include           'error.inc'
      include           'impurity.inc'
      include           'neqatm.inc'
      include           'red.inc'
      include           'star.inc'
      include           'symloc.inc'
c
      integer           ni, luw, nimpre
c
      real*8            rn(mnbuff), raux(mnbuff,3),
     &                  vec(48,3)
      integer           indaux(mnbuff), iord(mnbuff), typenv(mred),
     &                  indenv(maxeq,mred), nprim(48)
      logical           mrat, done, final
c
c.....on line function: distance between two lattice points
c
      dis(x,y,z) = sqrt( x * ( x*g(1) + two*y*g(2) + two*z*g(4) ) +
     &                   y * ( y*g(3) + two*z*g(5) ) + z*z*g(6) )
c
c.....loop over types of ions
c
      itot = 0 ! counter to obtain the crystallographic coordinates
      nenv = 0 ! counter for the number of shells
      do nf = 1, nneq ! nneq = inequivalent ions of the cell unit
c
c.....build all shells of type nf
c
        n = 0 ! counter for the total number of 'nf' ions
        do i = 1+itot, itot+mult(nf)  
c          i = counter of the crystallographic coordinates 
c              of the 'nf' ions in the cluster
c         rr = interionic distance between the ions 'ni' and 'nf'
          rr = dis (ximp(i)-xneq(ni),yimp(i)-yneq(ni),zimp(i)-zneq(ni))
          if (rr .gt. epsdis) then
            if (n.gt.mnbuff) call error ('envclus', 
     &          'too many atoms in the buffer list. '//
     &          'try to increase mnbuff', faterr)
            n = n +1
c           raux = interionic distance vector
            raux(n,1) = ximp(i) - xneq(ni)
            raux(n,2) = yimp(i) - yneq(ni)
            raux(n,3) = zimp(i) - zneq(ni)
            rn(n) = rr
            iord(n) = n
            indaux(n) = indcel(i) ! new
          endif
        enddo
        itot = itot + mult(nf)
        nenvni(nf) = 0
        if (n.gt.0) then
c
c.....order the neighbors by distance:
c
          call sortrx (n, rn, iord)
c
c.....classify the ions at the same distance:
c
          i = 1 ! i = 1,...,n
          ieq = 1 ! counter for the ions 'nf' belonging to the same shell
          rr = rn(iord(i))
          xenv(ieq,nenv+1) = raux(iord(i),1)
          yenv(ieq,nenv+1) = raux(iord(i),2)
          zenv(ieq,nenv+1) = raux(iord(i),3)
          indenv(ieq,nenv+1) = indaux(iord(i)) ! new
          nenvold = nenv
          do while (i.le.n)
            mrat = i.lt.n
            if (mrat) mrat = abs(rn(iord(i+1))-rr).le.epsdis
            if (mrat) then ! 'i' e 'i+1' are the same shell
              i = i + 1
              ieq = ieq + 1
              if (ieq.gt.maxeq) call error ('envclus', 'too many atoms
     & in the same shell. try to increase maxeq', faterr)
c             (x,y,z)env = interionic distance vector for the 'ieq'
              xenv(ieq,nenv+1) = raux(iord(i),1)
              yenv(ieq,nenv+1) = raux(iord(i),2)
              zenv(ieq,nenv+1) = raux(iord(i),3)
              indenv(ieq,nenv+1) = indaux(iord(i)) ! new
            else ! 'i+1' belong to a new shell
              nenv = nenv + 1
              if (nenv .ge. mred) call error ('envclus',
     &         'too many shells. Increase the parameter mred',faterr)
              renv(nenv) = rr ! interionic distance for the 'nenv' shell
              eqenv(nenv) = ieq ! number of 'nf' ions in the 'nenv' shell
              typenv(nenv) = nf ! use in printing of information
              i = i + 1
              if (i.le.n) then
                ieq = 1 ! it initializes the 'ieq' counter for a new shell
                rr = rn(iord(i))
                xenv(ieq,nenv+1) = raux(iord(i),1)
                yenv(ieq,nenv+1) = raux(iord(i),2)
                zenv(ieq,nenv+1) = raux(iord(i),3)
                indenv(ieq,nenv+1) = indaux(iord(i)) ! new
              endif
            endif
          enddo
          nenvni(nf) = nenv - nenvold ! number of shells formed by 'nf' ions 
c
c.........NEW: OBTAIN THE STARS (ORBITS) THAT FORM THE LAYERS
c
c.........External loop: shells formed by 'nf'--atoms   
          do i = 1, nenvni(nf)
            ish = nenvold + i ! counter for the shell  
            done = .false.      
            Nshell = eqenv(ish) ! initial 'nf'--atoms in the 'ish'--shell      
            istar = 0 ! counter for the stars in the 'ish'--shell
            do while (.not.done)
               istar = istar + 1
               if (istar.gt.mstar) call error ('envclus', 'too many
     & stars in a shell. Increase the parameter mstar',faterr)    
c.........STEP I: Selection of the representative in the 'istar'--star 
               ieq = 1
               final = .false.
               do while (.not.final .and. ieq.le.Nshell)
                 if (abs(indenv(ieq,ish)).eq.1) then
c         indenv < -1 => R is a improper rotation
c         indenv = -1 => R = I (inversion) 
c   NOTE: indenv =  1 => R = E (identity)
c         indenv >  1 => R is a proper rotation       
                   xxstar(1,istar,ish) = xenv(ieq,ish)
                   xxstar(2,istar,ish) = yenv(ieq,ish)
                   xxstar(3,istar,ish) = zenv(ieq,ish)
                   indstar(istar,ish) = indenv(ieq,ish)
                   final = .true.
                 endif
                 ieq = ieq + 1
               enddo
               if (.not.final) then
                 xxstar(1,istar,ish) = xenv(1,ish)
                 xxstar(2,istar,ish) = yenv(1,ish)
                 xxstar(3,istar,ish) = zenv(1,ish)
                 indstar(istar,ish) = indenv(1,ish)
               endif
c.........STEP II: Obtaining the 'nf'--atoms in the 'istar'--star  
c..............STEP IIa: Rotation 
*               vec(1,1) = xxstar(1,istar,ish)
*               vec(1,2) = xxstar(2,istar,ish)
*               vec(1,3) = xxstar(3,istar,ish) 
*               do j = 2, nlocal(1)
               do j = 1, nlocal(ni)
*c              NOTE: j=1 => identity operator 
                 do l = 1, 3
                   vec(j,l) = zero
                   do m = 1, 3
                     vec(j,l) = vec(j,l) +
     &                          rotloc(l,m,j,ni)*xxstar(m,istar,ish)
                   enddo
                 enddo
               enddo
c..............STEP IIb: Obtaining the different vectors
               nprim(1) = 1
               num = 1
               do j = 2, nlocal(ni)
                 final = .false.
                 k = 1
                 do while (.not.final .and. k.le.(j-1))
                   if (abs(vec(k,1)-vec(j,1)).lt.epsfrac .and.
     &                 abs(vec(k,2)-vec(j,2)).lt.epsfrac .and.
     &                 abs(vec(k,3)-vec(j,3)).lt.epsfrac) then
                     final = .true.
                   endif
                   k = k + 1
                 enddo
                 if (.not.final) then
                   num = num + 1
                   nprim(num) = j
                 endif
               enddo   
c..............STEP IIc: Ordination of the different vectors
               do j = 1, num
                 do k = 1, 3
                   vec(j,k) = vec(nprim(j),k)
                 enddo
               enddo
               nelst(istar,ish) = num
c.......STEP III: Elimination of the 'nf'--atoms of the 'ish'--shell
c                 that belong to the 'istar'--star
               num = 0
               do ieq = 1, Nshell
                 j = 1
                 final = .false.
                 do while (.not.final .and. j.le.nelst(istar,ish))
                   if (abs(xenv(ieq,ish)-vec(j,1)).lt.epsfrac .and.
     &                 abs(yenv(ieq,ish)-vec(j,2)).lt.epsfrac .and.
     &                 abs(zenv(ieq,ish)-vec(j,3)).lt.epsfrac) then
                     final = .true.
                   endif
                   j = j + 1
                 enddo
                 if (.not.final) then
                   num = num + 1
                   xenv(num,ish)   = xenv(ieq,ish)
                   yenv(num,ish)   = yenv(ieq,ish)
                   zenv(num,ish)   = zenv(ieq,ish)
                   indenv(num,ish) = indenv(ieq,ish)
                 endif
               enddo
               if ((Nshell-num).ne.nelst(istar,ish)) then
                 call error ('envclus', 'num .ne. nelst', warning)
                 nelst(istar,ish) = Nshell - num
               endif
               Nshell = num
c.........STEP IV: If we have deleted all the atoms in the 'ish'--shell => end    
               done = Nshell.eq.0
            enddo    
            nstar(ish) = istar      
          enddo ! i = 1,...,nevni(nf)         
        endif
      enddo ! nf
c
c.....create an ordered index vector
c
      ish = 0
      do nf = 1, nneq
        do i = 1, nenvni(nf)
          ish = ish + 1
          iord(ish) = ish
        enddo
      enddo
c
c.....sort all shells by distances, and print the environment
c
      if (nimpre.ge.-2) then
        call sortrx (nenv, renv, iord)
        titenv(ni) = namneq(ni)(1:leng(namneq(ni)))//':'//
     &               titrun(1:leng(titrun))
        write (luw,600) ni, namneq(ni), xneq(ni), yneq(ni), zneq(ni)
        write (luw,610) titenv(ni)(1:leng(titenv(ni)))
        write (luw,620) nenv,
     &    (eqenv(iord(k)), namneq(typenv(iord(k))),
     &     typenv(iord(k)), renv(iord(k)), k = 1, nenv)
      endif
c
c.....formats
c
 600  format (/1x, '*** ENVIRONMENT DESCRIPTION :'/
     & 1x, 'Active ion number: ', i2, '  Name: ', a6, '  Position: ',
     & 3f15.7)
 610  format (1x, 'Title: ', a)
 620  format (1x, 'Number of shells in environment: ', i4/
     & 1x, 'List of shells --> (number-of-ions name (type) distance)'/
     & ( 2(1x,'(',i3,1x,a6,' (',i2,')',f10.6,')',2x) ) )
c
c.....end
c
      return
      end
