c
c-----------------------------------------------------------------------
c
      subroutine envimp (ni, nimpre, luw)
c
c.....envimp - build succesive shells around an impurity ion in a
c     crystal.
c
c     CONVENTION: the first shells are formed by the impurity cluster
c     neighbors, irrespective of the distance. Once the cluster
c     neighbors end, the rest of the shells correspond to the host
c     crystal, and they are ordered by distance to the target ion.
c
c.....parameters:
c     ni ...... ion around which the environment should be built.
c     nimpre .. print level.
c     luw ..... logical unit for writing.
c
      include           'implicit.inc'
      include           'global.inc'
c
      include           'celatm.inc'
      include           'consts.inc'
      include           'converge.inc'
      include           'environ.inc'
      include           'error.inc'
      include           'hole.inc'
      include           'impurity.inc'
      include           'modatm.inc'
      include           'neqatm.inc'
      include           'red.inc'
c
      integer           ni, nimpre, luw
c
      real*8            rn(mnbuff), raux(mnbuff,3), neqenv(mred)
      integer           iord(mnbuff), typenv(mred)
      logical           mrat, ishole
c
c.....on line function: distance between two lattice points
c
      dis(x,y,z) = sqrt( x * ( x*g(1) + two*y*g(2) + two*z*g(4) ) +
     &                   y * ( y*g(3) + two*z*g(5) ) + z*z*g(6) )
c
c.....loop over types of ions
c
      itot = 0 ! counter to obtain the crystallographic coordinates
      nenv = 0 ! counter for the number of shells
      do nf = 1, nimneq ! nimneq = inequivalent imp. cluster ions
c
c.....build all shells of type nf
c
        n = 0 ! counter for the total number of 'nf' ions
        do i = 1+itot, mult(nf)+itot  
c counter of the crystallographic coordinates of the 'nf' ions
c in the cluster
c         rr = interionic distance between the ions 'ni' and 'nf'
          rr = dis (ximp(i)-xneq(ni),yimp(i)-yneq(ni),zimp(i)-zneq(ni))
          if (rr .gt. epsdis) then
            if (n.gt.mnbuff) call error ('envimp',
     &                            'Increase mnbuff!', faterr)
            n = n +1
c           raux = interionic distance vector for the
c                  'n' ion type 'nf'
            raux(n,1) = ximp(i) - xneq(ni)
            raux(n,2) = yimp(i) - yneq(ni)
            raux(n,3) = zimp(i) - zneq(ni)
            rn(n) = rr
            iord(n) = n
          endif
        enddo
        itot = itot + mult(nf)
        nenvni(nf) = 0
        if (n.gt.0) then
c
c.....sort the neighbors by distance:
c
          call sortrx (n, rn, iord)
c
c.....classify the ions at the same distance:
c
          i = 1 ! i = 1,...,n
          ieq = 1 ! counter for the ions 'nf' belonging to the same shell
          rr = rn(iord(i))
          xenv(ieq,nenv+1) = raux(iord(i),1)
          yenv(ieq,nenv+1) = raux(iord(i),2)
          zenv(ieq,nenv+1) = raux(iord(i),3)
          nenvold = nenv
          do while (i.le.n)
            mrat = i.lt.n
            if (mrat) mrat = abs(rn(iord(i+1))-rr).le.epsdis
            if (mrat) then ! 'i' and 'i+1' are the same shell
              i = i + 1
              ieq = ieq + 1
              if (ieq.gt.maxeq) call error ('envimp', 'too many atoms
     & in the same shell. try to increase maxeq', faterr)
c             (x,y,z)env = interionic distance vector for the 'ieq'
c                          ion in the 'nenv+1' shell
              xenv(ieq,nenv+1) = raux(iord(i),1)
              yenv(ieq,nenv+1) = raux(iord(i),2)
              zenv(ieq,nenv+1) = raux(iord(i),3)
            else ! 'i+1' belong to a new shell
              nenv = nenv + 1
              if (nenv .ge. mred) then
                call error ('envimp',
     &         'too many shells. Increase the parameter mred',faterr)
              endif
              renv(nenv) = rr ! interionic distance for the 'nenv' shell
              eqenv(nenv) = ieq ! number of 'nf' ions in the 'nenv' shell
              typenv(nenv) = nf ! use in printing of information
              i = i + 1
              if (i.le.n) then
                ieq = 1 ! it initializes the 'ieq' counter for a new shell
                rr = rn(iord(i))
                xenv(ieq,nenv+1) = raux(iord(i),1)
                yenv(ieq,nenv+1) = raux(iord(i),2)
                zenv(ieq,nenv+1) = raux(iord(i),3)
              endif
            endif
          enddo
          nenvni(nf) = nenv - nenvold ! number of shells formed by 'nf' ions
                                      ! (impurity cluster)
        endif
      enddo ! nf
c
c.....compute the limits for the search
c
      xfrom = - (rlastshell / ared + 0.1d0)
      xto   = -xfrom
      yfrom = - (rlastshell / bred + 0.1d0)
      yto   = -yfrom
      zfrom = - (rlastshell / cred + 0.1d0)
      zto   = -zfrom
      ncex0 = int(xfrom+xneq(ni)-1)
      ncey0 = int(yfrom+yneq(ni)-1)
      ncez0 = int(zfrom+zneq(ni)-1)
      ncex1 = int(xto+xneq(ni)+1)
      ncey1 = int(yto+yneq(ni)+1)
      ncez1 = int(zto+zneq(ni)+1)
c
c.....loop over all types of ions
c
      icel = 0 ! counter to obtain the crystallographic coordinates
      do nf = nimneq+1, nneq  ! nneq = inequivalent host crystal ions 
c
c.....generate all ions of this type inside a cube of side rlastshell
c
        n = 0 ! counter for the total number of 'nf' ions
        do i = ncex0, ncex1 ! traslation in the a direction
          do j = ncey0, ncey1 ! traslation in the b direction
            do k = ncez0, ncez1 ! traslation in the c direction
              do m = icel+1, icel+mult(nf) 
c                m = counter for the crystallographic coordinates 
c                    of all the ions type 'nf' in the cell unit
                xx = xcel(m) + i
                yy = ycel(m) + j
                zz = zcel(m) + k
                ishole = .false.
                do nh = 1, nhole
                  ishole = ishole .or.
     &                     abs(xhole(nh)-xx) .le. epsfrac .and.
     &                     abs(yhole(nh)-yy) .le. epsfrac .and.
     &                     abs(zhole(nh)-zz) .le. epsfrac
                enddo
                if (.not. ishole) then
c                 rr = interionic distance between the ions 'ni' and 'nf'
                  rr = dis (xx-xneq(ni), yy-yneq(ni), zz-zneq(ni))
                  if ((rr.gt.epsdis) .and. (rr.le.rlastshell)) then
                    if (n.ge.mnbuff) call error ('envimp',
     &                  'too many atoms in the buffer list. '//
     &                  'try to increase mnbuff',faterr)
                    n = n + 1
c                 raux = interionic distance vector
c                        for the 'n' ion type 'nf'
                    raux(n,1) = xx - xneq(ni)
                    raux(n,2) = yy - yneq(ni)
                    raux(n,3) = zz - zneq(ni)
                    rn(n)   = rr 
                    iord(n) = n 
                  endif
                endif
              enddo ! m
            enddo ! k
          enddo ! j
        enddo ! i
        icel = icel + mult(nf)
c
c.....order the neighbors by distance:
c
        call sortrx (n, rn, iord)
c
c.....classify the ions at the same distance:
c
        i = 1 ! i = 1,...,n
        ieq = 1 ! counter for the ions 'nf' belonging to the same shell
        rr = rn(iord(i))
        xenv(ieq,nenv+1) = raux(iord(i),1)
        yenv(ieq,nenv+1) = raux(iord(i),2)
        zenv(ieq,nenv+1) = raux(iord(i),3)
        nenvold = nenv
        do while (i.le.n)
          mrat = i.lt.n
          if (mrat) mrat = abs(rn(iord(i+1))-rr).le.epsdis
          if (mrat) then ! 'i' e 'i+1' are the same shell
            i = i + 1
            ieq = ieq + 1
            if (ieq.gt.maxeq) call error ('envimp', 'too many atoms in
     & the same shell. try to increase maxeq', faterr) 
c           (x,y,z)env = interionic distance vector for the 'ieq'
c                        ion in the 'nenv+1' shell
            xenv(ieq,nenv+1) = raux(iord(i),1)
            yenv(ieq,nenv+1) = raux(iord(i),2)
            zenv(ieq,nenv+1) = raux(iord(i),3)
          else ! 'i+1' belong to a new shell
            nenv = nenv + 1
            if (nenv .ge. mred) call error ('envimp',
     &        'too many shells. Increase the parameter mred',faterr)
            renv(nenv) = rr ! interionic distance for the 'nenv' shell
            eqenv(nenv) = ieq ! number of 'nf' ions in the 'nenv' shell
            neqenv(nenv) = ieq * xocup(nf) ! use in printing of information
            typenv(nenv) = nf ! use in printing of information
            i = i + 1
            if (i.le.n) then
              ieq = 1 ! it initializes the 'ieq' counter for a new shell
              rr = rn(iord(i))
              xenv(ieq,nenv+1) = raux(iord(i),1)
              yenv(ieq,nenv+1) = raux(iord(i),2)
              zenv(ieq,nenv+1) = raux(iord(i),3)
            endif
          endif
        enddo
        nenvni(nf) = nenv - nenvold ! number of shells formed by 'nf' ions
                                    ! (host crystal)
      enddo ! nf
c
c.....create an ordered index vector 
c
      ish = 0
      do nf = 1, nneq
        do i = 1, nenvni(nf)
          ish = ish + 1
          iord(ish) = ish
        enddo
      enddo
c
c.....sort all shells by distances, and print the environment
c
      if (nimpre.ge.-2) then
        call sortrx (nenv, renv, iord)
        titenv(ni) = namneq(ni)(1:leng(namneq(ni)))//':'//
     &               titrun(1:leng(titrun))
        write (luw,600) ni, namneq(ni), xneq(ni), yneq(ni), zneq(ni)
        write (luw,610) titenv(ni)(1:leng(titenv(ni)))
        if (fractional) then
          write (luw,620) nenv,
     &      (neqenv(iord(k)), namneq(typenv(iord(k))),
     &       typenv(iord(k)), renv(iord(k)), k = 1, nenv)
        else
          write (luw,630) nenv,
     &      (eqenv(iord(k)), namneq(typenv(iord(k))),
     &       typenv(iord(k)), renv(iord(k)), k = 1, nenv)
        endif
      endif
c
c.....formats
c
 600  format (/1x, '*** ENVIRONMENT DESCRIPTION :'/
     & 1x, 'Active ion number: ', i2, '  Name: ', a6, '  Position: ',
     & 3f15.7)
 610  format (1x, 'Title: ', a)
 620  format (1x, 'Number of shells in environment: ', i4/
     & 1x, 'List of shells --> (number-of-ions name (type) distance)'/
     & ( 2(1x,'(',f9.5,1x,a6,' (',i2,')',f10.6,')',2x) ) )
 630  format (1x, 'Number of shells in environment: ', i4/
     & 1x, 'List of shells --> (number-of-ions name (type) distance)'/
     & ( 2(1x,'(',i3,1x,a6,' (',i2,')',f10.6,')',2x) ) )
c
c.....end
c
      return
      end
