c
c-----------------------------------------------------------------------
c
      subroutine fillmat ()
c
c.....fillmat - initialize auxiliar matrices.
c

      include           'implicit.inc'
      include           'global.inc'
      include           'celatm.inc'
      include           'coed.inc'
      include           'comb.inc'
      include           'consts.inc'
      include           'corr.inc'
      include           'converge.inc'
      include           'environ.inc'
      include           'fact.inc'
      include           'impurity.inc'
      include           'intg.inc'
      include           'lint.inc'
      include           'scf.inc'
      include           'vef.inc'
c
c.....default type of calculation
c
      infsystem = .false.
      symmetrize = .false.
c
c.....default epsilon for coordinates and distances
c
      epsfrac = 1d-7
      epsdis  = 1d-6
c
c.....default variables for SCF:
c
      mxxtrp = 100
      bias   = -2
      nxtrp  = 15
      scfdiv = 1d-2
      threm  = 1d-8
      dodamp = .false.
      damp   = zero
c
c.....default variables for coulomb hole:
c
      do i = 0, mnatom
         con1(i)   = 0.028816d0
         con2(i)   = 1.1d0
      enddo
c
c.....default variables for crystal shell:
c
      rlastshell  = 30.0d0
      maxpicycles = min(100,mcycles)
      piconv      = 3d-8
      ewconv      = 1d-9
      convnsr     = 1d-7
      convnl      = 1d-7
      convesr     = 1d-7
      convxp      = 1d-7
      pidiv       = 1d+1
c
c.....variables for the impurity shell:
c
      nimneq = 0
c
c.....Initiallize the integral matrices:
c
      enlsr   = zero
      enllr   = zero
      enlnol  = zero
      do i = 1, mdens
         ss(i)     = zero
         t(i)      = zero
         u(i)      = zero
         v(i)      = zero
         vmp(i)    = zero
         vmploc(i) = zero 
         vnsr(i)   = zero
         vesr(i)   = zero
         vex(i)    = zero
         pla(i)    = zero
         vnsrnol(i) = zero
         vesrnol(i) = zero
         vexnol(i) = zero
         planol(i) = zero
      enddo
      do i = 1, maxpq
         p(i) = zero
         q(i) = zero
         xk(i) = zero
      enddo
      do i = 1, mnt
         do j = 1, mdens
            aexch(j,i) = zero
            aproj(j,i) = zero
         enddo
      enddo
c
c.....Factorials:
c
      fact(0) = 1d0
      facd(-1) = 1d0
      facd(0) = 1d0
      facl(0) = 0d0
      do i = 1, mfact
         fact(i) = i*fact(i-1)
         facd(i) = i*facd(i-2)
         facl(i) = facl(i-1) + log(dble(i)) ! use in modified cgaunt function
      enddo      
c
c.....Negative double factorial matrix is used in the D coefficients:
c
      mf21 = mfact/2 - 1
      do i = 1, mf21
         facd(-i-i-1) = (-1)**i / facd(i+i-1)
      enddo
c
c.....The combinatorial numbers will be stored in a supervector. The
c     combinatorial number I over J will appear in the element:
c
c          ((I-1)*(I+2))/2 + J + 1
c
c.....MFACT over MFACT will be the maximum combinatorial number.
c
      comb(0) = 1
      comb(1) = 1
      comb(2) = 1
      do i = 2, mfact
         ii = ((i-1)*(i+2))/2+1
         ii1 = ((i-2)*(i+1))/2+1
         comb(ii) = 1
         comb(ii+i) = 1
         do j = 1, i-1
            comb(ii+j) = comb(ii1+j-1)+comb(ii1+j)
         enddo
      enddo
c
c.....Obtention of N constants appearing in multipolar expansion
c
      do l = 0, maxmp
         tmp=four*pi/(l+l+1)
         xncte(l) = sqrt(tmp)
      enddo
c
c.....Obtention of the coefq appearing in multipolar expansion
c                 xncte(l1) xncte(l2) [2(l1+l2)-1]!!
c coefq(l1,l2) =___________________________________________
c                      (2l1-1)!! (2l2-1)!!
c
      do l1 = 0, maxmp
         do l2 = l1, maxmp
            ld1 = l1 + l1
            ld2 = l2 + l2
            ld12 = ld1 + ld2
            coefq(l1,l2) = xncte(l1)*xncte(l2)*facd(ld12-1)
     &                    /(facd(ld1-1)*facd(ld2-1))
            coefq(l2,l1)=coefq(l1,l2)
         enddo
      enddo
c
c.....Obtention of the \Delta(l1,l2) coefficient appearing in 
c     intcoulmb routine
c
      do l1 = 0, maxmp 
        ld1 = l1 + l1
        do l2 = l1, maxmp
          ld2 = l2 + l2 
          ltot = l1 + l2
          isig = 1
          if (mod(ltot,2).eq.1) isig = -1
          cdelta(l1,l2) = isig * facd(ltot+ltot-1)
     &                  / (facd(ld1+1)*facd(ld2+1))
          if (l1.ne.l2) cdelta(l2,l1)=cdelta(l1,l2)
        enddo
      enddo
c
c.....Obtention of the D(is,l) coefficient appearing in 
c     coefe coefficients and intcoul and overlap
c     routines
c
c     D(is,l) = 1/{(2is)!![2(is-l)-1]!!}
c
      do is = 0, maxl+1 
        do l = 0, maxl 
          coefd(is,l) = one / (facd(is+is)*facd(is+is-l-l-1))
        enddo
      enddo
c
c.....Obtention of the C(n1,n2,iu) matrix appearing in
c     coefe coefficients and intcoul routine
c     
c                             min(iu,n1)           |n1||n2  |
c C(n1,n2,iu)=(1/2)**(n1+n2)   SUM         (-1)**i |  ||    |
c                             i=max(iu-n2,0)       |i ||iu-i|
c
      twon = one
      do n1 = 1, ncmax 
        twon = twon * half ! twon = (1/2)**n1
        twonn = twon
        nn1 = ((n1-1)*(n1+2))/2 + 1 ! contador combinatorio 1er ind. C
        do n2 = 1, ncmax 
          twonn = twonn*half ! twonn = (1/2)**(n1+n2)
          do iu = 0, min(n1+n2,numax) 
            nn2 = ((n2-1)*(n2+2))/2 + iu + 1 ! contador combinatorio 2 ind. C
            coef = zero ! initializations of C
            istart = max(iu-n2,0)
            isig = 1
            if (mod(istart,2).eq.1) isig = -1
            do i = istart, min(iu,n1)
               coef = coef
     &               + isig * comb(nn1+i) * comb(nn2-i)
               isig = -isig
            enddo
            coefc(n1,n2,iu) = twonn * coef
          enddo
        enddo
      enddo
c
c.....Obtention of the E(la,na-la,lb,nb-lb,ir,iu) matrix; it will be
c     stored with the first four indices as a supervector.
c     
c     E(l1,nl1,l2,nl2,ir,iu) = 
c       ir
c      SUM C(nl1-l1+2is,nl2-l2+2ir-2is,iu) D(is,l1) D(ir-is,l2)
c      is=0   
c
      iab = 0 ! contador que aglutina los indices (l1,nl1,l2,nl2)
      do l1 = 0, maxmp
        do nl1 = 1, 2*namax-l1-1 ! nl1 = n1-l1-1
          do l2 = 0, maxmp 
            labms = l1 + l2 + 1 ! labmas <-> s1(max)
            do nl2 = 1, 2*namax-l2-1 ! nl2 = n2-l2-1
              iab = iab + 1
              indcoefe(l1,nl1,l2,nl2) = iab
              nn = nl1 + nl2 ! nn <-> Nmax = n1+n2-(l1+l2+1) 
              do ir = 0, labms 
c               ir = 0,...,s1(max); s1(max)=l1+l2-1
                i2r = ir + ir ! i2r = 2ir
                do iu = 0, nn+i2r
c                 iu = 0,...,Nmax+2ir
                  coefe(iu,ir,iab) = zero ! Initialization E
                  do is = 0, ir ! is = 0,...,ir
                    i2s = is + is ! i2s = 2is
                    i2sr = i2r-i2s ! i2sr = 2ir-2is 
                    termd = coefd(is,l1) * coefd(ir-is,l2)
c                   termd = D(is,l1) D(ir-is,l2)
                    coefe(iu,ir,iab) = coefe(iu,ir,iab) +
     &                                 coefc(nl1+i2s,nl2+i2sr,iu)*termd
                 enddo
                enddo
              enddo
            enddo
          enddo
        enddo
      enddo
c
c.....create the rsh data structures
c
      call rshinit()
c
c.....End.
c
      end
