c
c-----------------------------------------------------------------------
c
      subroutine fockv (ni)
c
c.....fockv - computes the open and closed shell Fock matrices.
c
c     The monoelectronic part of the Hamilton operator is:
c
c     Hlpq = Tlpq + Ulpq + VREDlpq + PREDlpq
c
c     The closed- and open-shell Fock operators are:
c
c     FC = H + P Dt + Ro
c
c     FO = H + P Dt - Q Do + Rc
c
c     where Dt and Do are the total- and open shell density matrices,
c     respectively. Rc and Ro are coupling operators:
c
c             Ncl        closed sh.         +          +
c     Rcl = ---------      SUMA     (sil qil  + qil sil  )
c           Ncl - Nol       i
c
c             Nol        open shell         +          +
c     Rol = ---------      SUMA     (sil qil  + qil sil  )
c           Ncl - Nol       i
c
c     sil = Sl cil,     qil = Ql cil
c
c     See Roothaan & Bagus (1963).
c
      include           'implicit.inc'
      include           'global.inc'
      include           'cont.inc'
      include           'symmetry.inc'
      include           'intg.inc'
      include           'scf.inc'
c
c.....Local variables:
      dimension pcap(mdens),qcap(mdens),smin(mpgao,maos),
     &          gmin(mpgao,maos)
c
c.....Computes the P and Q matrices over AOs and stores them in PCAP()
c     and QCAP():
      nstep1=0
      k=1
      do 10 i=1,nirrep(ni)
c
c........N1I=Densities over STOs in this symetry.
         n1i=(npgao(i,ni)*(npgao(i,ni)+1))/2
         do 15 m=1,n1i
            m1=m+nstep1
            kk=1
            nstep2=0
            term1=0d0
            term2=0d0
            do 20 j=1,nirrep(ni)
               n1j=(npgao(j,ni)*(npgao(j,ni)+1))/2
               do 25 n=1,n1j
                  n1=n+nstep2
                  mnmin=min0(m1,n1)
                  mnmax=max0(m1,n1)
                  l=(mnmax*(mnmax-1))/2+mnmin
                  term1=term1+p(l)*dt(kk)
                  term2=term2+q(l)*dos(kk)
                  kk=kk+1
 25            continue
               nstep2=nstep2+n1j
 20         continue
            pcap(k)=term1
            qcap(k)=term2
            k=k+1
 15      continue
         nstep1=nstep1+n1i
 10   continue
c
c.....Compute SMIN() & GMIN():
      nstep1=0
      nstep2=0
      do 30 i=1,nirrep(ni)
         nsh=ncsh(i,ni)+nosh(i,ni)
         nbas1=npgao(i,ni)
         do 35 j=1,nsh
            j1=j+nstep2
            do 40 m=1,nbas1
               term1=0d0
               term2=0d0
               do 45 n=1,nbas1
                  mnmin=min0(m,n)
                  mnmax=max0(m,n)
                  k=(mnmax*(mnmax-1))/2+mnmin+nstep1
                  term1=term1+ss(k)*gc(n,j1,ni)
                  term2=term2+qcap(k)*gc(n,j1,ni)
 45            continue
               smin(m,j1)=term1
               gmin(m,j1)=term2
 40         continue
 35      continue
         nstep2=nstep2+nsh
         nstep1=nstep1+(nbas1*(nbas1+1))/2
 30   continue
c
c.....Compute FC() & FO():
      k=1
      nstep=0
      do 50 i=1,nirrep(ni)
         nosh1=nosh(i,ni)
         nsh=ncsh(i,ni)+nosh(i,ni)
         nbas1=npgao(i,ni)
         xcerr=2*nsub(i,ni)
         xabie=0
         if (nosh1.gt.0) xabie=ngelec(nstep+nsh,ni)
         do 55 m=1,nbas1
            do 60 n=1,m
               term1=0d0
               term2=0d0
               do 65 j=1,nsh
                  j1=j+nstep
                  if (j.eq.nsh.and.nosh1.gt.0) then
c....................Open shell:
                     term2=smin(m,j1)*gmin(n,j1)+gmin(m,j1)*smin(n,j1)
                  else
c
c....................Closed shell:
                     term1=term1+smin(m,j1)*gmin(n,j1)+
     &                     gmin(m,j1)*smin(n,j1)
                  endif
 65            continue
               hppc=pcap(k)+t(k)+u(k)+v(k)
               fc(k)=hppc+(term2*xabie)/(xcerr-xabie)
               fo(k)=hppc-qcap(k)+(term1*xcerr)/(xcerr-xabie)
               k=k+1
 60         continue
 55      continue
         nstep=nstep+nsh
 50   continue
c
c.....Return control to SCFAT.
      return
      end
