c
c-----------------------------------------------------------------------
c
      subroutine impurity (lur, luw, nimpre)
c
c.....impurity - shell specialized in PI calculations of local defects &
c     impurities within pure 3D crystals.
c
      include           'impurity.doc'
c
      include           'implicit.inc'
      include           'global.inc'
c
      include           'celatm.inc'
      include           'cont.inc'
      include           'consts.inc'
      include           'converge.inc'
      include           'corr.inc'
      include           'environ.inc'
      include           'error.inc'
      include           'files.inc'
      include           'hole.inc'
      include           'impurity.inc'
      include           'lint.inc'
      include           'modatm.inc'
      include           'neqatm.inc'
      include           'red.inc'
      include           'spgcom.inc'
      include           'strtools.inc'
      include           'types.inc'
      include           'vef.inc'
c
      integer           lur, luw, nimpre
c
      parameter (qeps = 1d-4)
      integer           typrun
      logical           conv, done, ok, setline, setdble, setint, div
      logical           setword
      character*(256)   line
      character*(3)     itoa
      character*(ilen)  uppcase, word, word2, spg
      real*8            eclus(mcycles)
      real*8            cemp(mnt), cenlsr(mnt), cenllr(mnt), censr(mnt),
     &                  ceesr(mnt), ceex(mnt), cepl(mnt)
      real*8            xpure(mpure), ypure(mpure), zpure(mpure),
     &                  xocpure(mpure)
      integer           npure
      character*(ilen)  fpureion(mpure), fpureulm(mpure),
     &                  fpurecoef(mpure)
c
c.....initialize variables :
c
      call prtimer (2, ipid, 'impurity', luw)
      nimneq = 0 ! non-equivalent ions in the impurity cluster
      npure  = 0 ! non-equivalent ions in the cell of the perfect crystal
      nneq   = 0 ! nneq = 1,...,nimneq,nimneq+1,...,nimneq+npure 
      nhole  = 0 ! holes in the perfect crystal
      nimtot = 0 ! ions in the impurity cluster
      nmod   = 0 ! nmod = 1,...,nhole,nhole+1,...,nhole+nimtot
      infsystem = .true.
      fractional = .false.
      if (nimpre.ge.-1) write (luw,600)
c
c.....read input orders:
c
      done = .false.
      do while (.not. done)
         if (.not. setline (lur, line, lp)) then
            call error ('impurity', 'premature end of data', warning)
            return
         endif
         ll = leng(line)
         if (.not. setword (word2, line, lp)) then
            call error ('impurity', 'premature end of line', warning)
            return
         endif
         word = uppcase (word2)
         lw = leng(word)
         ok = .false.
c
c........case sentences to interpret word:
c
         if (word(1:1) .eq. '#' .or. word(1:lw) .eq. 'REM') then
c
c...........skip comment lines:
c
            ok = .true.
c        
         else if (word(1:lw) .eq. 'HOLE') then
c        Remove perfect crystal ions from this position
            nhole = nhole + 1
            if (nhole .gt. mimp) call error ('impurity',
     &          'max num of allowed holes exceeded. Increase mimp.',
     &          faterr)
            ok = setdble (xhole(nhole), line, lp)
            ok = ok .and. setdble (yhole(nhole), line, lp)
            ok = ok .and. setdble (zhole(nhole), line, lp)
c
         else if (word(1:lw) .eq. 'NEQ') then
c        Crystallographic coordinates of a non-equivalent ion in the
c        principal cell of the perfect crystal
            npure = npure + 1
            if (npure .gt. mpure) call error ('impurity',
     &         'max host neq positions exceeded. Increase mpure.',
     &         faterr)
            ok = setdble (xpure(npure), line, lp)
            ok = ok .and. setdble (ypure(npure), line, lp)
            ok = ok .and. setdble (zpure(npure), line, lp)
            ok = ok .and. setword (fpureion(npure), line, lp)
            ok = ok .and. setword (fpureulm(npure), line, lp)
            ok = ok .and. setword (fpurecoef(npure), line, lp)
            xocpure(npure) = one
c
         else if (word(1:lw) .eq. 'FNEQ') then
c        Equivalent to the 'NEQ' order except that a fractional
c        occupancy of the crystallographic position is allowed
            npure = npure + 1
            fractional = .true.
            if (npure .gt. mpure) call error ('impurity',
     &         'max host neq positions exceeded. Increase mpure.',
     &         faterr)
            ok = setdble (xpure(npure), line, lp)
            ok = ok .and. setdble (ypure(npure), line, lp)
            ok = ok .and. setdble (zpure(npure), line, lp)
            ok = ok .and. setdble (xocpure(npure), line, lp)
            ok = ok .and. setword (fpureion(npure), line, lp)
            ok = ok .and. setword (fpureulm(npure), line, lp)
            ok = ok .and. setword (fpurecoef(npure), line, lp)
c
         else if (word(1:lw) .eq. 'NEQIMP') then
c        Enter the coordinates of a subset of equivalent ions of the
c        impurity cluster
            nneq = nneq + 1
            if (nneq .gt. mnt) call error ('impurity',
     &         'max num of atom types exceeded. Increase mnt.',
     &         faterr)
            nimneq = nimneq + 1
            xocup(nneq) = one
            ok = setint (mult(nneq), line, lp)
            ok = ok .and. setword (fion(nneq), line, lp)
            ok = ok .and. setword (fulm(nneq), line, lp)
            ok = ok .and. setword (fcoef(nneq), line, lp)
            if (typcorr .eq. typrCHF) then
               ok = ok .and. setword (fcint(nneq), line, lp)
            else if (typcorr .eq. typuCHF) then
               ok = ok .and. setword (fint(nneq), line, lp)
               ok = ok .and. setword (fcint(nneq), line, lp)
            else
               ok = ok .and. setword (fint(nneq), line, lp)
            endif
            ok = ok .and. setword (flint(nneq), line, lp)
            k = 1
            do while (k .le. mult(nneq) .and. ok)
               ok = setline (lur, line, lp)
               nimtot = nimtot + 1
               if (nimtot .gt. mimp) call error ('impurity',
     &            'max num of impur positions exceeded. Increase mimp.',
     &            faterr)
               typimp(nimtot) = nneq
               ok = ok .and. setdble (ximp(nimtot), line, lp)
               ok = ok .and. setdble (yimp(nimtot), line, lp)
               ok = ok .and. setdble (zimp(nimtot), line, lp)
               if (k .eq. 1) then
                  xneq(nneq) = ximp(nimtot)
                  yneq(nneq) = yimp(nimtot)
                  zneq(nneq) = zimp(nimtot)
               endif
               k = k + 1
            enddo
c
         else if (word(1:lw) .eq. 'SPG') then
c        Enter crystal space group symbol
            do while (line(lp:lp) .eq. blank)
               lp = lp + 1
            enddo
            spg = line(lp:)
            ls = leng(spg)
            if (nimpre.ge.0) then
               iun = luw
            else
               iun = -1
            endif
            call spgr (spg,iun,luw,ier)
            if (ier.ne.0) then
               write (luw,*) 'spgr: error',ier
               call error ('impurity',
     &              'space group handler error', faterr)
            endif
            if (nimpre.ge.-1) write (luw,610) spg(1:ls)
            ok = .true.
c
         else if (word(1:lw) .eq. 'CELL') then
c        Enter the cell side lengths (in bohrs) and angles
c        (in degrees)
            ok = setdble (ared, line, lp)
            ok = ok .and. setdble (bred, line, lp)
            ok = ok .and. setdble (cred, line, lp)
            ok = ok .and. setdble (alf, line, lp)
            ok = ok .and. setdble (bet, line, lp)
            ok = ok .and. setdble (gam, line, lp)
            if (nimpre.ge.-1)
     &         write (luw,605) ared, bred, cred, alf, bet, gam
            calf = cos (alf * rad)
            salf = sin (alf * rad)
            cbet = cos (bet * rad)
            sbet = sin (bet * rad)
            cgam = cos (gam * rad)
            sgam = sin (gam * rad)
            g(1) = ared * ared
            g(2) = ared * bred * cgam
            g(3) = bred * bred
            g(4) = ared * cred * cbet
            g(5) = bred * cred * calf
            g(6) = cred * cred
            if (nimpre.ge.-1)
     &         write (luw,606) g(1), g(2), g(4), g(2), g(3), g(5),
     &                         g(4), g(5), g(6)
c
         else if (word(1:lw) .eq. 'TITLE') then
            do while (line(lp:lp) .eq. blank)
               lp = lp + 1
            enddo
            titrun = line(lp:ll)
            lt = leng (titrun)
            if (nimpre.ge.-1) write (luw,607) titrun(:lt)
            ok = .true.
c
         else if (word(1:lw) .eq. 'ENDIMPURITY') then
            done = .true.
            ok = .true.
c
         else
            ok = .false.
         endif
c
c........trap read errors:
c
         if (.not. ok) then
            call error ('impurity', 'order not recognized'//
     &                  newline//line(1:leng(line)), faterr)
         endif
      enddo
c
c.....BEGIN THE RUN:
c
c.....(A): Build the unit cell of the pure crystal & the impur. cluster:
c     Note: the cell unit is the same for the perfect crystal and the 
c           impurity cluster
      do ni = 1, npure
c     We add to the non-equivalent imp. cluster ions the non-equivalent
c     ones of the perfect crystal 
         nneq = nneq + 1
c        nneq = nimneq+1,..,nimneq+npure
         if (nneq .gt. mnt) call error ('impurity',
     &      'max num of atom types exceeded. Increase mnt.', faterr)
         xneq(nneq)  = xpure(ni)
         yneq(nneq)  = ypure(ni)
         zneq(nneq)  = zpure(ni)
         xocup(nneq) = xocpure(ni)
         fion(nneq)  = fpureion(ni)
         fulm(nneq)  = fpureulm(ni)
         fcoef(nneq) = fpurecoef(ni)
      enddo
      do ni = 1, nneq
         call readion (fion(ni), ni, nimpre, luw)
         call readulm (fulm(ni), ni, nimpre, luw)
         call readcoef (fcoef(ni), ni, nimpre, luw)
         call vefion (fion(ni), ni, rdint, luw, nimpre-1)
         qneq(ni) = qclas(ni)
      enddo
      call create (nimneq + 1, nneq)
c     We generate the perfect crystal cell 
      zmult = mcdvec (mult, nimneq+1, nneq)
c     zmult = formulas/cell
      qtotcel = 0
      do ni = nimneq + 1, nneq
         qtotcel = qtotcel + qneq(ni) * xocup(ni) * (mult(ni)/zmult)
      enddo
      if (abs(qtotcel) .gt. 1d-5) then
         call error ('impurity','The Ewald routine is not warrantied '//
     &               'to give physical results when the unit cell has'//
     &               ' a net charge.',warning)
      endif
      root = sqrt(one-calf*calf-cbet*cbet-cgam*cgam+two*calf*cbet*cgam)
      omega = ared*bred*cred*root
      if (nimpre.ge.-1) then
         write (luw,615) (namneq(ni), mult(ni), ni = 1, nimneq)
         write (luw,616) (namneq(ni), (mult(ni)/zmult)*xocup(ni),
     &                   ni = nimneq+1, nneq)
         write (luw,617) zmult
         write (luw,618) qtotcel
         write (luw,620) omega
         write (luw,621) omega/zmult
         if (nimpre.ge.0) then
            write (luw,625) nimtot
            do ni = 1, nimtot
               ni1 = typimp(ni)
               write (luw,626) ni, ximp(ni), yimp(ni), zimp(ni),
     &                         qneq(ni1), namneq(ni1), ni1
            enddo
            write (luw,628) nhole
            do ni = 1, nhole
               write (luw,629) ni, xhole(ni), yhole(ni), zhole(ni)
            enddo
            write (luw,630) ncel
            do ni = 1, ncel
               ni1 = typcel(ni)
               write (luw,631) ni, xcel(ni), ycel(ni), zcel(ni),
     &                         xocup(ni1), qneq(ni1), namneq(ni1), ni1
            enddo
         endif
      endif
c
c.....(B1): Determine positions with charges different to the perfect
c     crystal. First, add all holes, checking that they don't coincide.
c
      do ni = 1, nhole
         nmod = nmod + 1
c        nmod = 1,...,nhole
         if (nmod .gt. mmod) call error ('impurity',
     &      'max mod positions at ewald exceeded. Increase mmod',
     &      faterr)
c        NEW ROUTINE 'TYPEHOLE'
         call typehole (xhole(ni), yhole(ni), zhole(ni),
     &                  typmod(nmod), ipure)
         if (ipure.eq.0) then
            write (luw,629) ni, xhole(ni), yhole(ni), zhole(ni)
            call error ('impurity','hole in empty site is meaningless',
     &                  faterr)
         endif
         xmod(nmod)     = xhole(ni)
         ymod(nmod)     = yhole(ni)
         zmod(nmod)     = zhole(ni)
c        Checking of which the coordinates of two 'holes' anyone
c        do not coincide
         j = 1 ! j = 1,...,nmod-1
         do while (j.lt.nmod)
            if (abs(xmod(j)-xmod(nmod)) .le. epsfrac  .and.
     &          abs(ymod(j)-ymod(nmod)) .le. epsfrac  .and.
     &          abs(zmod(j)-zmod(nmod)) .le. epsfrac  ) then
               write (luw,629) ni, xhole(ni), yhole(ni), zhole(ni)
               call error ('impurity','two holes in the same point'//
     &                     ' not allowed', faterr)
            endif
            j = j + 1
         enddo
      enddo ! nhole
c
c.....add the impurities, checking that they don't coincide; if two
c     positions are the same, either is a hole+impurity cancellation,
c     or two impurities coincide.
c
      do ni = 1, nimtot
         nmod = nmod + 1
c        nmod = nhole+1,...,nhole+nimtot
         if (nmod .gt. mmod) call error ('impurity',
     &      'max mod positions at ewald exceeded. Increase mmod',
     &      faterr)
         xmod(nmod)     = ximp(ni)
         ymod(nmod)     = yimp(ni)
         zmod(nmod)     = zimp(ni)
         typmod(nmod)   = typimp(ni)
c        Checking of which the coordinates of two 'impurities' anyone
c        do not coincide
         j = nhole+1 ! j = nhole+1,...,nmod-1
         do while (j.lt.nmod)
            if (abs(xmod(j)-xmod(nmod)) .le. epsfrac  .and.
     &          abs(ymod(j)-ymod(nmod)) .le. epsfrac  .and.
     &          abs(zmod(j)-zmod(nmod)) .le. epsfrac) then
               write (luw,629) ni, ximp(ni), yimp(ni), zimp(ni)
               call error ('impurity','two impurity atoms in the'//
     &                     ' same point not allowed', faterr)
            endif
            j = j + 1
         enddo
      enddo ! nimtot
c
c.....(B2): Precompute Madelung potential at the non-equivalent ion sites
c          (perfect crystal):
c
      call preewald ()
c
c.....(C): Build environment description for each non-equivalent cluster
c     impurity ion:
c
*     This has been moved into the intimp routine
c
c.....(D): perform PI cycles until convergence or MAXPICYCLES:
c
      ncycle = 0
      conv = .false.
      div  = .false.
      done = (ncycle .ge. MAXPICYCLES)
      typrun = wrint
      nprtloc = nimpre - 1
      do while (.not. done)
         ncycle = ncycle + 1
         eclus(ncycle) = zero
         do ni = 1, nimneq
            tition(ni) = namneq(ni)(1:leng(namneq(ni)))//':'//
     &                titrun(1:leng(titrun))//'. Cycle '//itoa(ncycle)
            if (typcorr .eq. typrCHF) then
               call istupq (fcint(ni),ni,typrCHF,typrun,luw,nprtloc)
            else
               call istupq (fint(ni),ni,typHF,typrun,luw,nprtloc)
            endif
            call intimp (flint(ni), ni, typrun, luw, nprtloc)
            call scfat (ni, luw, nprtloc)
            if (typcorr .eq. typuCHF) then
               call uchf (ni, typrun, luw, nprtloc)
            endif
            call vefion (fion(ni), ni, nwint, luw, nprtloc)
            cemp(ni)      = emp
            cenlsr(ni)    = enlsr
            cenllr(ni)    = enllr
            censr(ni)     = ensr
            ceesr(ni)     = eesr
            ceex(ni)      = eex
            cepl(ni)      = epl
c
c...........determine the local energy:
c
            call localenr (ni, luw, nprtloc)
            eclus(ncycle) = eclus(ncycle) + eloc(ni) * mult(ni)
         enddo
         if (nimpre.ge.0) write (luw,700)
     &      ncycle, eclus(ncycle), (eloc(ni), ni=1,nimneq)
         if (ncycle .gt. 1) then
            enerdif = abs (eclus(ncycle) - eclus(ncycle-1))
            if (enerdif .le. PICONV) then
               conv = .true.
               nprtloc = nimpre
            else if (enerdif .ge. PIDIV) then
               div = .true.
               nprtloc = nimpre
            endif
         else
            typrun = rdint
         endif
         if (conv .or. div .or. ncycle .gt. MAXPICYCLES) then
            done = .true.
         endif
      enddo
      if (nimpre.ge.-2) then
         do ni = 1, nimneq
            call writecoef (fcoef(ni), ni)
            call vefion (fion(ni), ni, wrint, luw, nprtloc)
         enddo
      endif
      if (div) then
         write (luw,711)
         call error ('impurity', 'Divergence condition for Eclus',
     &               faterr)
      else if (.not.conv) then
         write (luw,710)
         call error ('impurity', 'Convergence has not been reached',
     &               faterr)
      endif
      if (nimpre.eq.-1) write (luw,700)
     &   ncycle, eclus(ncycle), (eloc(ni), ni=1,nimneq)
c
c.....(E1): Add correlation energy (Density functionals)
c
c      if (typcorr.eq.typWig .or. typcorr.eq.typGom .or.
c     &    typcorr.eq.typLYP .or. typcorr.eq.typBR) then
c         eclus(ncycle) = zero
c         do ni = 1, nneq
c            call correl (ni, luw, nimpre)
c            eloc(ni) = eloc(ni) + ecorr(ni)
c            eclus(ncycle) = eclus(ncycle) + eloc(ni) * mult(ni)
c         enddo
c      endif
c
c.....(E2): If requested, symmetrize the cluster-lattice interaction
c
      if (symmetrize) then
         call symmimp (luw, nimpre, eclus(ncycle), ceex, cepl) ! MIRAR
      endif
c
c.....(F): Short resume of results:
c
      if (nimpre.ge.-1) then
         write (luw,671) ared, bred, cred, alf, bet, gam,
     &                   titrun(1:leng(titrun))
         if (typcorr .eq. typHF) then
            write (luw,673) 'HF (none)'
         else if (typcorr .eq. typuCHF) then
            write (luw,673) 'uCHF'
         else if (typcorr .eq. typrCHF) then
            write (luw,673) 'rCHF'
c         else if (typcorr .eq. typWig) then
c            write (luw,673) 'Wigner'
c         else if (typcorr .eq. typGom) then
c            write (luw,673) 'Gombas'
c         else if (typcorr .eq. typLYP) then
c            write (luw,673) 'Lee-Yang-Parr'
c         else if (typcorr .eq. typBR) then
c            write (luw,673) 'Brual-Rothstein'
         endif
         write (luw,677) ncycle
         write (luw,678)
         do ni = 1, nimneq
            write (luw,680) namneq(ni), eadd(ni), eeff(ni), eint(ni),
     &          ekin(ni), ecorr(ni), cemp(ni)+cenllr(ni),
     &          censr(ni)+ceesr(ni)+cenlsr(ni), ceex(ni), cepl(ni)
         enddo
         write (luw,900)
         do ni = 1, nimneq
            cenr = cenlsr(ni) + cenllr(ni)
            write (luw,910) namneq(ni), cemp(ni)+cenllr(ni)+censr(ni)
     &          +ceesr(ni)+cenlsr(ni),  cemp(ni)+censr(ni), cenr,
     &          cemp(ni), cenllr(ni), cenlsr(ni), censr(ni), ceesr(ni)
         enddo
         write (luw,690) eclus(ncycle)
c
c.....print resume of the HF subproduct when typuCHF is being run:
c
         if (typcorr .ne. typrCHF .and. typcorr.ne.typHF) then
            write (luw,673) 'HF (none)'
            write (luw,678)
            do ni = 1, nimneq
               write (luw,680) namneq(ni), eadd(ni)-ecorr(ni),
     &             eeff(ni)-ecorr(ni), eint(ni), ekin(ni), ecorr(ni),
     &             cemp(ni)+cenllr(ni), censr(ni)+ceesr(ni)+cenlsr(ni),
     &             ceex(ni), cepl(ni)
            enddo
            eclusHF = zero
            do ni = 1, nimneq
               eclusHF = eclusHF + (eloc(ni) - ecorr(ni)) * mult(ni)
            enddo
            write (luw,690) eclusHF, eclusmad, eclusHF-eclusmad
         endif
c
c.....(G): end-of-impurity-run. Return to the general shell.
c
         write (luw,800)
         call prtimer (4, ipid, 'impurity', luw)
c
c.....On a nimpre < -1, print out only the energy
c
      else
         write (luw,790) eclus(ncycle)
         call prtimer (4, ipid, 'impurity', -1)
      endif
      nimneq = 0
      return
c
 600  format ('1'/
     & 1x, '******************************************************'/
     & 1x, '**                - GPI: IMPURITY -                 **'/
     & 1x, '** Shell designed for easy runs on local impurities **'/
     & 1x, '** in pure 3D crystals.                             **'/
     & 1x, '**       (c) C.J.F Solano.  June 14, 2008           **'/
     & 1x, '******************************************************'// )
 605  format (1x, '*** CELL PARAMETERS  : ', 6(1x,f14.6))
 606  format (1x, '*** METRIC TENSOR    : ', 3(1x,f14.6) /
     &                                2(24x, 3(1x,f14.6) /) )
 607  format (1x, '*** CELL TITLE       : ', a)
 610  format (1x, '*** SPACE GROUP      : ', a)
 615  format (1x, '*** IMPURITY CLUSTER : ', (1x,4(a6,'(',i3,')',2x)))
 616  format (1x, '*** HOST CRYSTAL     : ', (1x,4(a6,'(',f9.6,')',2x)))
 617  format (1x, '*** FORMULAS/CELL (Z): ', i4)
 618  format (1x, '*** NET CHARGE/CELL  : ', f9.6)
 620  format (1x, '*** CELL VOLUME (au) : ', 1pe20.11)
 621  format (1x, '*** FORMULA VOLUME   : ', 1pe20.11)
 625  format (/1x,'Total number of ions in the impurity cluster : ', i5/
     & 1x, '---i------x-----------y-----------z-----------q----name',
     &     '---type')
 626  format (1x, i4, 3(1x,f11.8), 1x, f9.5, 1x, a6, 1x, i4)
 628  format (/1x,'Total number of holes : ', i5/
     & 1x, '---i------x-----------y-----------z-----')
 629  format (1x, i4, 3(1x,f11.8))
 630  format (/1x,'Ions in the host crystal unit cell : ', i5/
     & 1x, '---i-----x----------y----------z--------ocup-----q------',
     & '--name-type')
 631  format (1x, i4, 3(1x,f10.8), 1x, f8.6, 1x, f9.5, 1x, a6, 1x, i4)
 640  format (/1x, 'Ion # ', i3, 1x, a6, '  Madelung potential: ',
     & 1p, e17.8, 0p, ' Shells added up: ', i3,
     & ' Pure crystal charge: ', f10.3)
 671  format (/
     & 1x, '******************************************************'/
     & 1x, '***           IMPURITY RESUME OF RESULTS           ***'/
     & 1x, '******************************************************'//
     & 1x, '*** CELL SIZE : ', 6(1x,f14.6)/
     & 1x, '*** RUN TITLE : ', a)
 673  format (/1x,'*** CORRELATION  : ', a)
 677  format (1x,'*** PI CYCLES : ', i4)
 678  format (
     & 1x, 'Ion---------Eadd----------Eeff--------Eint-',
     &     '-------Ekin-------Ecorr-------Emp---',
     &     '----Ecsr--------Eex-------Epl--' )
 680  format (1x, a6, 2(1x,f13.6), 1x, f9.6, 1x, f12.6, 2(1x,f10.6),
     &        3(1x,f9.6))
 900  format (/1x, '*** Breakdown of Coulomb energy: ',/,
     & 1x, 'Ion----------Ec-------------Ecl----------Enuclat------',
     &     '--Eemp-----------Enmp--------Enlsr----------Ensr-------',
     &     '--Eesr----')
 910  format (1x,a6, 8(1x,f13.6))
 690  format (/
     & 1x, '*** Total effective energy of the cluster   : ', 1pe22.14 )
 700  format (1x, '*** Pi cycle # ',i3,'  Eclus: ', f25.15, '  Eloc: ',
     & 3f15.6, / (1x,6f15.6) )
 710  format (/
     & 1x, '***'/
     & 1x, '*** CAUTION: convergence has not been reached at impurity.'/
     & 1x, '***'/ )
 711  format (/
     & 1x, '***'/
     & 1x, '*** CAUTION: PI cycles stopped due to divergence in Eclus'/
     & 1x, '***'/ )
 790  format (1x,e24.16)
 800  format (/1x, '*** END OF IMPURITY RUN *************************')
      end
