c
c-----------------------------------------------------------------------
c
      subroutine intimp (fileint, ni, typrun, luw, nimpre)
c
c.....intimp - computes the ion-lattice interaction matrices over the
c     primitive basis functions for impurity calculations.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'cont.inc'
      include           'consts.inc'
      include           'converge.inc'
      include           'coed.inc'
      include           'celatm.inc'
      include           'modatm.inc'
      include           'iondens.inc'
      include           'environ.inc'
      include           'error.inc'
      include           'impurity.inc'
      include           'intg.inc'
      include           'lint.inc'
      include           'neqatm.inc'
      include           'strtools.inc'
      include           'types.inc'
      include           'vef.inc'
      include           'rsh.inc'
      include           'dgaunt.inc'
      include           'tfunc.inc'
      include           'hole.inc'
      include           'multipole.inc'
c
c.....input parameters
c
      character*(*)     fileint
      integer           typrun, luw, nimpre
c
c.....used indexes: ni, active ion; nf, frozen ion; ish, shell of ions;
c     lpq, active ion density index; lab, frozen ion density index
c
      integer           ni, nf, ish, lpq, lab
c
c.....used a temporal vector in the Madelung potential
c
      real*8            vmad(0:lmmax)
c
c.....used as temporal vector in vcl, and as integrals vector in vnc,
c     vex and pla; its index are lpq,lab, where lpq refers to densities
c     of the active center, and lab to densities of the frozen one
c
      real*8            o(mdens,mdens)
c
c.....stores the integrals between densities in one center and all
c     centers of a given type
c
      real*8            term(mdens,mdens), oneq(mdens,mdens)
c
      logical           done, converged
      integer           lualloc
      character*(128)   cdum
      real*8            tiny
      parameter         ( tiny = 1d-21 )
c
c.....online functions: modulus of a vector in cartesian coordinates
c
      dis(xd,yd,zd) = sqrt( xd*xd + yd*yd + zd*zd)
c
c.....determine type of run:
c
      if (typrun .eq. wrint) then
         call prtimer (2,ipid,'intimp(w)',luw)
         if (nimpre .ge. -1) write (luw,600) namneq(ni), ni
      else if (typrun .eq. nwint) then
         call prtimer (2,ipid,'intimp(n)',luw)
         if (nimpre .ge. -1) write (luw,600) namneq(ni), ni
      else if (typrun .eq. rdint) then
         call prtimer (2,ipid,'intimp(r)',luw)
         if (nimpre .ge. 0) write (luw,600) namneq(ni), ni
      else
         call error ('intimp','typrun requested not available', faterr)
      endif
c
c.....open lattice integrals file:
c
      if (typrun .ne. nwint) then
        luj = lualloc (luj)
        open (luj, file=fileint, form='unformatted', status='unknown',
     &        iostat=nios)
        if (nios .ne. 0) then
          cdum = 'can''t open integrals file'//fileint
          call error ('intimp', cdum, faterr)
        endif
      endif
c
c.....compute the integrals. We will divide the ion-lattice interaction
c     into several parts:
c
c     (a) multipolar electrostatic interaction            ====> vmp()
c     (b) nuclear short range electrostatic interaction   ====> vnsr()
c     (c) coulomb short range electrostatic interaction   ====> vesr()
c     (d) exchange interaction                            ====> vex() 
c     (e) projection operator contributions               ====> pla()
c
c     v() = vmp() + vnsr() + vesr() + vex() + pla(), where
c     vmp() = vnlr() + velr()
c
c     We will store the contributions due to every non-equivalent type
c     of impurity ion to b-d, but only global contributions to non-local
c     potentials and classical local potential.
c
c     First, we will add all contributions due to the impurity
c     ions, then all contributions due to the host crystal ones.
c
c.....initialize the matrices:
c
      potloc   = zero
      potnol   = zero
      potlr    = zero
      potlrloc = zero
      do lpq = 1, ndens(ni)
        vmp(lpq)     = zero
        vnsr(lpq)    = zero
        vesr(lpq)    = zero
        vex(lpq)     = zero
        pla(lpq )    = zero
        vmploc(lpq)  = zero
        vnsrnol(lpq) = zero
        vesrnol(lpq) = zero
        vexnol(lpq)  = zero  
        planol(lpq)  = zero
      enddo
c
c....Obtain the spherical multipole moments
c
      do nf = 1, nneq
         Qm(0,nf) = - zn(nf)
         do lm = 1, nlm(nf)
            Qm(lm,nf) = zero
         enddo
         do lpq = 1, ndens(nf)
            do lm = dmlmin(lpq,nf), dmlmax(lpq,nf)
               if (qmult(lpq,lm,nf).ne.zero) then
                  Qm(lm,nf) = Qm(lm,nf) + rhomat(lpq,nf)
     &                      * qmult(lpq,lm,nf)
               endif
            enddo
         enddo
      enddo
c
c....Multipolar electrostatic interaction 
c
      if (nimpre.ge.-1) write (luw,100) ni, namneq(ni)
      do lpq = 1, ndens(ni)
         l = mlmin(lpq,ni)
         do lm = dmlmin(lpq,ni), dmlmax(lpq,ni)
            if (lm.gt.il0(l)+l) l = l + 1
            if (qmult(lpq,lm,ni).ne.zero) then
               m = lm - il0(l)
               qpur  = charge(xneq(ni),yneq(ni),zneq(ni))
               vlm = tewald(xneq(ni),yneq(ni),zneq(ni),qpur,l,m,nsh)
               if (nimpre.ge.-1)
     &            write (luw,200) lpq, lm, vlm, nsh
               vmp(lpq) = vmp(lpq) + qmult(lpq,lm,ni)*vlm
            endif
         enddo
      enddo
c......Add modified potential (holes + impurities)
      do lpq = 1, ndens(ni)
         l1 = mlmin(lpq,ni)
         do lm1 = dmlmin(lpq,ni),dmlmax(lpq,ni)
            if (lm1.gt.il0(l1)+l1) l1 = l1 + 1
            if (qmult(lpq,lm1,ni).ne.zero) then
               m1 = lm1 - il0(l1)
               do nf = 1, nmod
                  rr1 = xmod(nf) - xneq(ni)
                  rr2 = ymod(nf) - yneq(ni)
                  rr3 = zmod(nf) - zneq(ni)
                  call tocart (rr1,rr2,rr3,xa,xb,xc)
                  r = dis(xa,xb,xc)
                  if (r.gt.epsdis) then ! S .ne. R
                     call rsh (xa,xb,xc,r,slm,maxl)
                     l2 = 0
                     do lm2 = 0, nlm(typmod(nf))
                        if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                        if (Qm(lm2,typmod(nf)).ne.zero) then
                           isig = 1
                           if (mod(l2,2).eq.1) isig = -1
                           m2 = lm2 - il0(l2)
                           ltot = l1 + l2
                           iltot = il0(ltot)
                           i = ig0(lm1,lm2,ltot)
                           rexp = r**(ltot+1)
                           tslm = zero
                           do im3 = 1, m3num(m1,m2,ltot)
                              mtot = m3val(m1,m2,im3)
                              lmtot = iltot + mtot
                              tslm = tslm + dgaunt(i+im3)*slm(lmtot)
                           enddo
                           if (nf.le.nhole) then ! hole
                              Qlm2 = - Qm(lm2,typmod(nf))
                           else ! impurity
                              Qlm2 = Qm(lm2,typmod(nf))
                           endif
                           vmp(lpq) = vmp(lpq) + qmult(lpq,lm1,ni)
     &                               *isig*coefq(l1,l2)*Qlm2*tslm/rexp
                           if (nf.gt.nhole) then ! impurity cluster
                              vmploc(lpq) = vmploc(lpq) +
     &                                      qmult(lpq,lm1,ni)*isig
     &                                     *coefq(l1,l2)*Qlm2*tslm/rexp
                           endif 
                        endif
                     enddo
                  endif
               enddo
            endif
         enddo
      enddo
c
c....Obtain the long range local potential (nucleus-lattice interaction)
c
      qpur  = charge(xneq(ni),yneq(ni),zneq(ni))
      potlr = sewald(xneq(ni),yneq(ni),zneq(ni),qpur,nsh)
      if (nimpre.ge.-1)
     &   write (luw,300) potlr, nsh
c......Add modified potential (holes + impurities)
      do nf = 1, nmod
         rr1 = xmod(nf) - xneq(ni)
         rr2 = ymod(nf) - yneq(ni)
         rr3 = zmod(nf) - zneq(ni)
         call tocart (rr1,rr2,rr3,xa,xb,xc)
         r = dis(xa,xb,xc)
         if (r.gt.epsdis) then ! S .ne. R
            call rsh (xa,xb,xc,r,slm,maxmp)
            l = 0
            do lm = 0, nlm(typmod(nf))
               if (lm.gt.il0(l) + l) l = l + 1
               if (Qm(lm,typmod(nf)).ne.zero) then
                  isig = 1
                  if (mod(l,2).eq.1) isig = -1
                  if (nf.le.nhole) then ! hole
                     Qlm = - Qm(lm,typmod(nf))
                  else ! impurity
                     Qlm = Qm(lm,typmod(nf))
                  endif
                  rexp = r**(l+1)
                  potlr = potlr + isig*xncte(l)*Qlm*slm(lm)/rexp
                  if (nf.gt.nhole) then ! impurity cluster
                     potlrloc = potlrloc + isig*xncte(l)*Qlm
     &                         *slm(lm)/rexp
                  endif
               endif
            enddo
         endif
      enddo
c
c.....(1) wrint/nwint: compute the integrals from scratch
c
      if (typrun .ne. rdint) then
c
c.....Create the environment around each impurity ion (only when
c     computing the integrals).
c
        call envimp (ni, nimpre, luw)
c
c.....(I) First, all local interactions are added
c
        isha = 0
        do nf = 1, nimneq 
          xs = xocup(nf) 
c
c.....(b) Nuclear electrostatic interaction: short range term
c
          do ish = isha+1, isha+nenvni(nf) 
c nenvni(nf) = shells formed by 'nf'-type imp. ions
            rs = renv(ish) ! distance of the 'ish' shell
            do lpq = 1, ndens(ni)
              oneq(lpq,1) = zero
            enddo
            do ieq = 1, eqenv(ish) 
c eqenv(ish) = number of 'nf'-type imp. ions in the 'ish' shell
c...........Coordinate conversion to cartesian coordinates
              call tocart (xenv(ieq,ish), yenv(ieq,ish),
     &                     zenv(ieq,ish), xa, xb, xc)
c...........Compute the spherical harmonics
              call rsh (xa, xb, xc, rs, slm, maxl)
c...........Nuclear attraction integral
              call nuclear (ni, rs, .false., o)
              do lpq = 1, ndens(ni)
                oneq(lpq,1) = oneq(lpq,1) + o(lpq,1)
              enddo
            enddo ! ieq (ions 'nf' that compose the shell 'ish')
            do lpq = 1, ndens(ni)
              oneq(lpq,1) = - xs * zn(nf) * oneq(lpq,1)
              vnsr(lpq) = vnsr(lpq) + oneq(lpq,1)
            enddo
            if (nimpre.ge.4) then
              write (luw,620) ish-isha, 'Vnsr'
              write (luw,710) (oneq(lpq,1), lpq = 1, ndens(ni))
            endif
          enddo ! ish
c
c.....determine the nucleus-lattice interaction energy:
c
          do lab = 1, ndens(nf)
            oneq(lab,1) = zero
          enddo
          do ish = isha+1, isha+nenvni(nf) 
c nenvni(nf) = shells formed by 'nf'-type imp. ions
            rs = renv(ish) ! distance of the 'ish' shell 
            do lab = 1, ndens(nf)
              term(lab,1) = zero
            enddo
            do ieq = 1, eqenv(ish) 
c eqenv(ish) = number of 'nf'-type imp. ions in the 'ish' shell
c...........Coordinate conversion to cartesian coordinates
              call tocart (xenv(ieq,ish), yenv(ieq,ish),
     &                     zenv(ieq,ish), xa, xb, xc)
c...........Compute the spherical harmonics
              call rsh (xa, xb, xc, rs, slm, maxl)
c...........Local potential (short range)
              call nuclear (nf, rs, .true., o)
              do lab = 1, ndens(nf)
                term(lab,1) = term(lab,1) + o(lab,1)
              enddo
            enddo ! ieq (ions 'nf' that compose the shell 'ish')
            do lab = 1, ndens(nf)
              term(lab,1) = xs * term(lab,1)
              oneq(lab,1) = oneq(lab,1) + term(lab,1)
              potloc = potloc + term(lab,1) * rhomat(lab,nf)
            enddo
            if (nimpre.ge.4) then
              write (luw,620) ish-isha, 'Vnlsr'
              write (luw,710) (term(lab,1), lab = 1, ndens(nf))
            endif
          enddo ! ish
          if (typrun.eq.wrint)
     &      write (luj) (oneq(lab,1), lab = 1, ndens(nf))
c
c.....(c) Coulomb electrostatic interaction: short range term
c
          do lpq = 1, ndens(ni)
            do lab = 1, ndens(nf)
              oneq(lab,lpq) = zero
            enddo
          enddo
          do ish = isha+1, isha+nenvni(nf) 
            rs = renv(ish)
            do lpq = 1, ndens(ni)
              do lab = 1, ndens(nf)
                term(lab,lpq) = zero
              enddo
            enddo
            do ieq = 1, eqenv(ish) 
c...........Coordinate conversion to cartesian coordinates
              call tocart (xenv(ieq,ish), yenv(ieq,ish),
     &                     zenv(ieq,ish), xa, xb, xc)
c...........Compute the spherical harmonics
              call rsh (xa, xb, xc, rs, slm, maxl)
c...........Obtention of the T^l_{lama,lbmb}(R)
              j = 0
              l1 = 0
              do lm1 = 0, lmmax
                if (lm1.gt.il0(l1)+l1) l1 = l1 + 1
                 m1 = lm1 - il0(l1)
                 l2 = 0
                 do lm2 = 0, lm1
                   if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                   m2 = lm2 - il0(l2)
                   do l3 = l1-l2, l1+l2, 2
                     j = j + 1
                     i = ig0(lm1,lm2,l3)
                     itfunc(lm1,lm2,l3) = j
                     itfunc(lm2,lm1,l3) = j
                     tfunc(j) = zero ! initializations
                     do im3 = 1, m3num(m1,m2,l3)
                       i = i + 1
                       m3 = m3val(m1,m2,im3)
                       lm3 = il0(l3) + m3
                       tfunc(j) = tfunc(j) + dgaunt(i) * slm(lm3)
                     enddo
                   enddo
                 enddo
              enddo
              call coulint (ni, nf, rs, o)
              do lpq = 1, ndens(ni)
                do lab = 1, ndens(nf)
                  term(lab,lpq) = term(lab,lpq) + o(lpq,lab)
                enddo
              enddo
            enddo 
            do lpq = 1, ndens(ni)
              vetmp = zero ! initializations
              do lab = 1, ndens(nf)
                term(lab,lpq) = term(lab,lpq) * xs
                oneq(lab,lpq) = oneq(lab,lpq) + term(lab,lpq)
                vetmp = vetmp + term(lab,lpq)*rhomat(lab,nf)
              enddo
              vesr(lpq) = vesr(lpq) + vetmp
            enddo
            if (nimpre.ge.4) then
              write (luw,620) ish-isha, 'Vesr'
              do lpq = 1, ndens(ni)
                write (luw,710) (term(lab,lpq), lab = 1, ndens(nf))
              enddo
            endif
          enddo ! ish
          if (typrun.eq.wrint) then
            do lpq = 1, ndens(ni)
              write (luj) (oneq(lab,lpq), lab = 1, ndens(nf))
            enddo
          endif
c
c.....(d) exchange & (e) projection operator contributions
c
          do lpq = 1, ndens(ni)
            do lab = 1, ndens(nf)
              oneq(lab,lpq) = zero
            enddo
          enddo
          do ish = isha+1, isha+nenvni(nf) 
            rs = renv(ish)
            do lpq = 1, ndens(ni)
              do lab = 1, ndens(nf)
                term(lab,lpq) = zero
              enddo
            enddo
            do ieq = 1, eqenv(ish) 
c...........Coordinate conversion to cartesian coordinates
              call tocart (xenv(ieq,ish), yenv(ieq,ish),
     &                     zenv(ieq,ish), xa, xb, xc)
c...........Compute the spherical harmonics
              call rsh (xa, xb, xc, rs, slm, maxl)
c...........Obtention of the T^l_{lama,lbmb}(R)
              j = 0
              l1 = 0
              do lm1 = 0, maxmp*(maxmp+2)
                if (lm1.gt.il0(l1)+l1) l1 = l1 + 1
                 m1 = lm1 - il0(l1)
                 l2 = 0
                 do lm2 = 0, lm1
                   if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                   m2 = lm2 - il0(l2)
                   do l3 = l1-l2, l1+l2, 2
                     j = j + 1
                     i = ig0(lm1,lm2,l3)
                     itfunc(lm1,lm2,l3) = j
                     itfunc(lm2,lm1,l3) = j
                     tfunc(j) = zero ! initializations
                     do im3 = 1, m3num(m1,m2,l3)
                       i = i + 1
                       m3 = m3val(m1,m2,im3)
                       lm3 = il0(l3) + m3
                       tfunc(j) = tfunc(j) + dgaunt(i) * slm(lm3)
                     enddo
                   enddo
                 enddo
              enddo
              call overlap (ni, nf, rs, o)
              do lpq = 1, ndens(ni)
                do lab = 1, ndens(nf)
                  term(lab,lpq) = term(lab,lpq) + o(lpq,lab)
                enddo
              enddo
            enddo ! ieq
            do lpq = 1, ndens(ni)
              vxtmp = zero ! initializations
              vptmp = zero ! initializations
              do lab = 1, ndens(nf)
                term(lab,lpq) = term(lab,lpq) * xs
                oneq(lab,lpq) = oneq(lab,lpq) + term(lab,lpq)
                vxtmp = vxtmp + term(lab,lpq) * aexch(lab,nf)
                vptmp = vptmp + term(lab,lpq) * aproj(lab,nf)
              enddo
              vex(lpq) = vex(lpq) - vxtmp
              pla(lpq) = pla(lpq) + vptmp
            enddo 
            if (nimpre.ge.4) then
              write (luw,620) ish-isha, 'Vxp'
              do lpq = 1, ndens(ni)
                write (luw,710) (term(lab,lpq), lab = 1, ndens(nf))
              enddo
            endif
          enddo ! ish
          if (typrun.eq.wrint) then
            do lpq = 1, ndens(ni)
              write (luj) (oneq(lab,lpq), lab = 1, ndens(nf))
            enddo
          endif
c
c.....end loop of local ions
c
          isha = isha + nenvni(nf)
        enddo ! 'nf'-type imp. ions
        if (typrun.eq.wrint) write (luj) (vnsr(lpq), lpq = 1, ndens(ni))
c
c.....(II) Second, non-local interactions
c
        do nf = nimneq+1, nneq 
c       'nf'-type host crystal ions
          if (nimpre.ge.-1) write (luw,610) namneq(nf), nf
          xs = xocup(nf)
c
c.....(b) Nuclear electrostatic interaction: short range term
c
          converged = .false.
          done = nenvni(nf).le.0 
c         nenvni(nf) = shells formed by 'nf'-type host crystal ions
          ish = isha + 1
          do while (.not. done)
            rs = renv(ish) ! distance of the shell 'ish'
            termax = zero
            do lpq = 1, ndens(ni)
              oneq(lpq,1) = zero
            enddo
            do ieq = 1, eqenv(ish) 
c           eqenv (ish) = number of 'nf'-type host crystal 
c                         ions in the 'ish' shell
c...........Coordinate conversion to cartesian coordinates
              call tocart (xenv(ieq,ish), yenv(ieq,ish),
     &                     zenv(ieq,ish), xa, xb, xc)
c...........Compute the spherical harmonics
              call rsh (xa, xb, xc, rs, slm, maxl)
c...........NUclear attraction integral
              call nuclear (ni, rs, .false., o)
              do lpq = 1, ndens(ni)
                oneq(lpq,1) = oneq(lpq,1) + o(lpq,1)
              enddo
            enddo  ! ieq (ions 'nf' that compose the shell 'ish')
            do lpq = 1, ndens(ni)
              oneq(lpq,1) = - xs * zn(nf) * oneq(lpq,1)
              vnsrnol(lpq) = vnsrnol(lpq) + oneq(lpq,1)
              relerr = abs(oneq(lpq,1)/(vnsrnol(lpq)+tiny))
              if (relerr.gt.termax) then
                termax = relerr
              endif
            enddo
            if (nimpre.ge.4) then
              write (luw,620) ish-isha, 'Vnsr'
              write (luw,710) (oneq(lpq,1), lpq = 1, ndens(ni))
            endif
            converged = termax.le.convnsr
            ish = ish + 1
            done = converged .or. ish.gt.isha+nenvni(nf)
          enddo
          if (.not. converged) then
            write (luw,810) 'Vnsr', ish-isha-1, termax
            cdum = 'Vnsr is not fully converged'
            call error ('intimp', cdum, warning) 
          else
            if (nimpre.ge.-1) write (luw,800) 'Vnsr', ish-isha-1
          endif
c
c.....determine the nucleus-lattice interaction energy:
c
          do lab = 1, ndens(nf)
            oneq(lab,1) = zero
          enddo
          converged = .false.
          done = nenvni(nf).le.0 
          ish = isha + 1
          do while (.not.done)
            rs = renv(ish)
            termax = zero
            do lab = 1, ndens(nf)
              term(lab,1) = zero
            enddo
            do ieq = 1, eqenv(ish) 
c...........Coordinate conversion to cartesian coordinates
              call tocart (xenv(ieq,ish), yenv(ieq,ish),
     &                     zenv(ieq,ish), xa, xb, xc)
c...........Compute the spherical harmonics
              call rsh (xa, xb, xc, rs, slm, maxl)
c...........Local potential (short range)
              call nuclear (nf, rs, .true., o)
              do lab = 1, ndens(nf)
                term(lab,1) = term(lab,1) + o(lab,1)
              enddo
            enddo  
            do lab = 1, ndens(nf)
              term(lab,1) = xs * term(lab,1)
              oneq(lab,1) = oneq(lab,1) + term(lab,1)
              potnol = potnol + term(lab,1) * rhomat(lab,nf)
              relerr = abs(term(lab,1)/(oneq(lab,1)+tiny))
              if (relerr.gt.termax) then
                termax = relerr
              endif
            enddo
            if (nimpre.ge.4) then
              write (luw,620) ish-isha, 'Vnlsr'
              write (luw,710) (term(lab,1), lab = 1, ndens(nf))
            endif
            converged = termax.le.convnl
            ish = ish + 1
            done = converged .or. ish.gt.isha+nenvni(nf)
          enddo
          if (.not. converged) then
            write (luw,810) 'Vnlsr', ish-isha-1, termax
            cdum = 'Vnlsr is not fully converged'
            call error ('intimp', cdum, warning) 
          else
            if (nimpre.ge.-1) write (luw,800) 'Vnlsr', ish-isha-1
          endif
c
c.....(c) Coulomb electrostatic interaction: short range term
c
          do lpq = 1, ndens(ni)
            do lab = 1, ndens(nf)
              oneq(lab,lpq) = zero
            enddo
          enddo
          converged = .false.
          done = nenvni(nf).le.0 
          ish = isha + 1
          do while (.not.done)
            rs = renv(ish)
            termax = zero
            do lpq = 1, ndens(ni)
              do lab = 1, ndens(nf)
                term(lab,lpq) = zero
              enddo
            enddo
            do ieq = 1, eqenv(ish) 
c...........Coordinate conversion to cartesian coordinates
              call tocart (xenv(ieq,ish), yenv(ieq,ish),
     &                     zenv(ieq,ish), xa, xb, xc)
c...........Compute the spherical harmonics
              call rsh (xa, xb, xc, rs, slm, maxl)
c...........Obtention of the T^l_{lama,lbmb}(R)
              j = 0
              l1 = 0
              do lm1 = 0, lmmax
                if (lm1.gt.il0(l1)+l1) l1 = l1 + 1
                 m1 = lm1 - il0(l1)
                 l2 = 0
                 do lm2 = 0, lm1
                   if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                   m2 = lm2 - il0(l2)
                   do l3 = l1-l2, l1+l2, 2
                     j = j + 1
                     i = ig0(lm1,lm2,l3)
                     itfunc(lm1,lm2,l3) = j
                     itfunc(lm2,lm1,l3) = j
                     tfunc(j) = zero ! initializations
                     do im3 = 1, m3num(m1,m2,l3)
                       m3 = m3val(m1,m2,im3)
                       lm3 = il0(l3) + m3
                       tfunc(j) = tfunc(j) + dgaunt(i+im3) * slm(lm3)
                     enddo
                   enddo
                 enddo
              enddo
              call coulint (ni, nf, rs, o)
              do lpq = 1, ndens(ni)
                do lab = 1, ndens(nf)
                  term(lab,lpq) = term(lab,lpq) + o(lpq,lab)
                enddo
              enddo
            enddo 
            do lpq = 1, ndens(ni)
              vetmp = zero ! initializations
              do lab = 1, ndens(nf)
                term(lab,lpq) = term(lab,lpq) * xs
                oneq(lab,lpq) = oneq(lab,lpq) + term(lab,lpq)
                vetmp = vetmp + term(lab,lpq)*rhomat(lab,nf)
                relerr = abs(term(lab,lpq)/(oneq(lab,lpq)+tiny))
                if (relerr.gt.termax) then
                  termax = relerr
                endif
              enddo
              vesrnol(lpq) = vesrnol(lpq) + vetmp
            enddo
            converged = termax.le.convesr 
            if (nimpre.ge.4) then
              write (luw,620) ish-isha, 'Vesr'
              do lpq = 1, ndens(ni)
                write (luw,710) (term(lab,lpq), lab = 1, ndens(nf))
              enddo
            endif
            ish = ish + 1
            done = converged .or. ish.gt.isha+nenvni(nf)
          enddo
          if (.not.converged) then
            write (luw,810) 'Vesr', ish-isha-1, termax
            cdum = 'Vesr is not fully converged'
            call error ('intimp', cdum, warning) 
          else
            if (nimpre.ge.-1) write (luw,800) 'Vesr', ish-isha-1
          endif
c
c.....(d) exchange & (e) projection operator contributions
c
          do lpq = 1, ndens(ni)
            do lab = 1, ndens(nf)
              oneq(lab,lpq) = zero
            enddo
          enddo
          converged = .false. 
          done = nenvni(nf).le.0
          ish = isha + 1
          do while (.not.done)
            rs = renv(ish)
            termax = zero
            do lpq = 1, ndens(ni)
              do lab = 1, ndens(nf)
                term(lab,lpq) = zero
              enddo
            enddo
            do ieq = 1, eqenv(ish) 
c...........Coordinate conversion to cartesian coordinates
              call tocart (xenv(ieq,ish), yenv(ieq,ish),
     &                     zenv(ieq,ish), xa, xb, xc)
c...........Compute the spherical harmonics
              call rsh (xa, xb, xc, rs, slm, maxl)
c...........Obtention of the T^l_{lama,lbmb}(R)
              j = 0
              l1 = 0
              do lm1 = 0, lmmax
                if (lm1.gt.il0(l1)+l1) l1 = l1 + 1
                 m1 = lm1 - il0(l1)
                 l2 = 0
                 do lm2 = 0, lm1
                   if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                   m2 = lm2 - il0(l2)
                   do l3 = l1-l2, l1+l2, 2
                     j = j + 1
                     i = ig0(lm1,lm2,l3)
                     itfunc(lm1,lm2,l3) = j
                     itfunc(lm2,lm1,l3) = j
                     tfunc(j) = zero ! initializations
                     do im3 = 1, m3num(m1,m2,l3)
                       m3 = m3val(m1,m2,im3)
                       lm3 = il0(l3) + m3
                       tfunc(j) = tfunc(j) + dgaunt(i+im3) * slm(lm3)
                     enddo
                   enddo
                 enddo
              enddo
              call overlap (ni, nf, rs, o)
              do lpq = 1, ndens(ni)
                do lab = 1, ndens(nf)
                  term(lab,lpq) = term(lab,lpq) + o(lpq,lab)
                enddo
              enddo
            enddo
            do lpq = 1, ndens(ni)
              vxtmp = zero ! initializations
              vptmp = zero ! initializations
              do lab = 1, ndens(nf)
                term(lab,lpq) = term(lab,lpq) * xs
                oneq(lab,lpq) = oneq(lab,lpq) + term(lab,lpq)
                vxtmp = vxtmp + term(lab,lpq) * aexch(lab,nf)
                vptmp = vptmp + term(lab,lpq) * aproj(lab,nf)
                relerr = abs(term(lab,lpq)/(oneq(lab,lpq)+tiny))
                if (relerr.gt.termax) then
                  termax = relerr
                endif
              enddo
              vexnol(lpq) = vexnol(lpq) - vxtmp
              planol(lpq) = planol(lpq) + vptmp
            enddo 
            if (nimpre.ge.4) then
              write (luw,620) ish-isha, 'Vxp'
              do lpq = 1, ndens(ni)
                write (luw,710) (term(lab,lpq), lab = 1, ndens(nf))
              enddo
            endif
            converged = termax.le.convxp
            ish = ish + 1
            done = converged .or. ish.gt.isha+nenvni(nf)
          enddo
          if (.not.converged) then
            write (luw,810) 'Vxp', ish-isha-1, termax
            cdum = 'Vxp is not fully converged'
            call error ('intimp', cdum, warning) 
          else
            if (nimpre.ge.-1) write (luw,800) 'Vxp', ish-isha-1
          endif
c
c.....end loop of non-local ions: write accumulated contributions
c
          isha = isha + nenvni(nf)
        enddo
c......Note: we store the 'host crystal' contributions ultimately because
c            their values remain frozen in the SCF calculations.
        if (typrun.eq.wrint) then
          write (luj) (vnsrnol(lpq), lpq = 1, ndens(ni))
          write (luj) potnol
          write (luj) (vesrnol(lpq), lpq = 1, ndens(ni))
          write (luj) (vexnol(lpq), lpq = 1, ndens(ni))
          write (luj) (planol(lpq), lpq = 1, ndens(ni))
        endif
        if (nimpre.ge.-1) write (luw,900)
c
c.....(2) rdint: read the integrals from a file
c
      else
c
c.....(I) Local contributions
c
        do nf = 1, nimneq
          read (luj) (oneq(lab,1), lab = 1, ndens(nf))
          do lab = 1, ndens(nf)
            potloc = potloc + rhomat(lab,nf) * oneq(lab,1)
          enddo
          do lpq = 1, ndens(ni)
            read (luj) (oneq(lab,lpq), lab = 1, ndens(nf))
            do lab = 1, ndens(nf)
              vesr(lpq) = vesr(lpq) + rhomat(lab,nf) * oneq(lab,lpq)
            enddo
          enddo
          do lpq = 1, ndens(ni)
            read (luj) (oneq(lab,lpq), lab = 1, ndens(nf))
            do lab = 1, ndens(nf)
              vex(lpq) = vex(lpq) - aexch(lab,nf) * oneq(lab,lpq)
              pla(lpq) = pla(lpq) + aproj(lab,nf) * oneq(lab,lpq)
            enddo
          enddo
        enddo
        read (luj) (vnsr(lpq), lpq = 1, ndens(ni))
c
c.....(II) Non-local contributions
c
        read (luj) (vnsrnol(lpq), lpq = 1, ndens(ni))
        read (luj) potnol
        read (luj) (vesrnol(lpq), lpq = 1, ndens(ni))
        read (luj) (vexnol(lpq), lpq = 1, ndens(ni))
        read (luj) (planol(lpq), lpq = 1, ndens(ni))
      endif
c
c.....(3) Add contributions together
c
      do lpq = 1, ndens(ni)
        vnsr(lpq)   = vnsr(lpq) + vnsrnol(lpq)  
        vesr(lpq)   = vesr(lpq) + vesrnol(lpq)
        vex(lpq)    = vex(lpq) + vexnol(lpq)
        pla(lpq)    = pla(lpq) + planol(lpq)
      enddo
      enllr = -zn(ni)*potlr
      enlsr = -zn(ni)*(potloc + potnol)
      enlnol  = - zn(ni) * (potlr - potlrloc + potnol)
      do lpq = 1, ndens(ni)
         v(lpq) = vmp(lpq) + vnsr(lpq) + vesr(lpq) + vex(lpq) 
     &            + pla(lpq)
      enddo
c
c.....close integral file:
c
      if (typrun.ne.nwint) then
        close (luj)
        call lunalloc (luj)
      endif
c
c.....print out the results:
c
      if (nimpre .ge. 0) then
         write (luw,700) potlr, potloc+potnol, enllr+enlsr
      endif
      if (nimpre .ge. 2) then
         write (luw,705) 'Multipolar potential'
         write (luw,710) (vmp(k), k = 1, ndens(ni))
         write (luw,705) 'Nuclear potential (short range)'
         write (luw,710) (vnsr(k), k = 1, ndens(ni))
         write (luw,705) 'Coulomb potential (short range)'
         write (luw,710) (vesr(k), k = 1, ndens(ni))
         write (luw,705) 'Exchange operators'
         write (luw,710) (vex(k), k = 1, ndens(ni))
         write (luw,705) 'Projection operators'
         write (luw,710) (pla(k), k = 1, ndens(ni))
      endif
c
      if (typrun .eq. wrint) then
         call prtimer (4,ipid,'intimp(w)',-1)
      else if (typrun .eq. nwint) then
         call prtimer (4,ipid,'intimp(n)',-1)
      else
         call prtimer (4,ipid,'intimp(r)',-1)
      endif
      return
c
 100  format (1x, ' Madelung Potential',/,
     &        1x, ' Ion Active # ', i3, 1x, a6)
 200  format (1x,' Madelung potential Vmad(',i3,',',i3,'): ', 1pe17.8,
     &           '  Shells added up: ', i3)
 300  format (1x,' Madelung potential Vloc,lr: ', 1pe17.8,
     &           '  Shells added up: ', i3)
 600  format (/'*** LATTICE INTEGRALS FOR THE ACTIVE ION : ',
     &        a, ' (no. ', i2, ' )'/)
 610  format (1x, ' Lattice integrals due to ', a, ' (no. ', i2, ' )')
 620  format (/1x,'+ Shell no. ', i2, 1x, a, ' integrals +')
 700  format (/
     & 1x, 'Long range local potential at the active site : ', f21.14/
     & 1x, 'Short range local potential at the active site: ', f21.14/
     & 1x, 'Total nucleus-lattice energy                  : ',1pe21.14 )
 705  format (/1x, '+ Lattice integrals due to: ', a, '+')
 710  format (1x, 1p, 10e13.5)
 800  format (1x, 'Shells used to converge ', a, ' : ', i4)
 810  format (a, ' not converged. Last shell added: ', i4,
     &           ' Integral error (rel.): ', e13.6)
 900  format (/)
c
      end
