c
c-----------------------------------------------------------------------
c
      subroutine intred (fileint, ni, typrun, luw, nimpre)
c
c.....intred - computes the ion-lattice interaction matrices over the
c     primitive basis functions.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'celatm.inc'
      include           'cont.inc'
      include           'consts.inc'
      include           'converge.inc'
      include           'iondens.inc'
      include           'environ.inc'
      include           'error.inc'
      include           'impurity.inc'
      include           'intg.inc'
      include           'lint.inc'
      include           'neqatm.inc'
      include           'strtools.inc'
      include           'types.inc'
      include           'vef.inc'
      include           'rsh.inc'
      include           'dgaunt.inc'
      include           'tfunc.inc'
      include           'multipole.inc'
      include           'coed.inc'
      include           'symmetry.inc'
      include           'fact.inc'
      include           'star.inc'
      include           'rotmat.inc'
c
c.....input parameters
c
      character*(*)     fileint
      integer           typrun, luw, nimpre
c
c.....used indexes: ni, active ion; nf, frozen ion; ish, shell of ions;
c     lpq, active ion density index; lab, frozen ion density index
c
      integer           ni, nf, ish, lpq, lab
      integer           lmm(maxmp+1), nshell(maxmp+1), iord(maxmp+1)
      integer           maxmult
      parameter        (maxmult = 20)
      integer           norig(maxmult), ndest(maxmult) 
      real*8            udploc, qmultloc
      real*8            vmadlm(maxmp+1)
c
c.....used as temporal vector in vn, and as integrals vector in ve,
c     vex and pla; its index are lpq,lab, where lpq refers to densities
c     of the active center, and lab to densities of the frozen one
c
      real*8            o(mdens,mdens)
c
c.....stores the integrals between densities in one center and all
c     centers of a given type
c
      real*8            tnuc1, tnuc2, tcoul, txp 
      real*8            tvnuc2(mdens), tvcoul(mdens,mdens),
     &                  tvxp(mdens,mdens)
      real*8            onuc1(mdens), onuc2(mdens)
      real*8            oncoul(mdens,mdens), onxp(mdens,mdens)       
      real*8            oneq(mdens,mdens)
c
c.....Controls the integrals convergence 
c
      logical           convnuc1, convnuc2, convcoul, convover 
c
      logical           done, ok, ok2
      integer           lualloc
      character*(128)   cdum
      real*8            tiny
      parameter         ( tiny = 1d-21 )
c
c.....online functions: modulus of a vector in cartesian coordinates
c
      dis(xd,yd,zd) = sqrt( xd*xd + yd*yd + zd*zd)
c
c.....determine type of run:
c
      if (typrun .eq. wrint) then
         call prtimer (2,ipid,'intred(w)',luw)
         if (nimpre .ge. -1) write (luw,600) namneq(ni), ni
      else if (typrun .eq. nwint) then
         call prtimer (2,ipid,'intred(n)',luw)
         if (nimpre .ge. -1) write (luw,600) namneq(ni), ni
      else if (typrun .eq. rdint) then
         call prtimer (2,ipid,'intred(r)',luw)
         if (nimpre .ge. 0) write (luw,600) namneq(ni), ni
      else
         call error ('intred','typrun requested not available', faterr)
      endif
c
c.....open lattice integrals file:
c
      if (typrun .ne. nwint) then
         luj = lualloc (luj)
         open (luj, file=fileint, form='unformatted', status='unknown',
     &         iostat=nios)
         if (nios .ne. 0) then
            cdum = 'can''t open integrals file'//fileint
            call error ('intred', cdum, faterr)
         endif
      endif
c
c.....compute the integrals. We will divide the ion-lattice interaction
c     into several parts:
c
c     (a) multipolar electrostatic interaction            ====> vmp()
c     (b) nuclear short range electrostatic interaction   ====> vnsr() 
c     (c) coulomb short range electrostatic interaction   ====> vesr()
c     (d) exchange interaction                            ====> vex()
c     (e) projection operator contributions               ====> pla()
c
c     v() = vmp() + vnsr() + vesr() + vex() + pla(), where
c     vmp() = vnlr() + velr()
c
c     We will store the contributions due to every non-equivalent type
c     of ion to b-d, but only global contributions to (a).
c
c.....initialize the matrices:
c
      potnl = zero
      potlr = zero
      do lpq = 1, ndens(ni)
         vmp(lpq)  = zero 
         vnsr(lpq) = zero
         vesr(lpq) = zero
         vex(lpq)  = zero
         pla(lpq)  = zero
      enddo
c
c....Obtain the spherical multipole moments
c
c.....NEW OBTAINING OF THE MULTIPOLE MOMENTS
      icel = 0 
c.....External loop: non-equivalent atoms in the unit cell
      do nf = 1, nneq
        itrot = indcel(icel+1)
        num = 0
c.....STEP I: Initializations
        do j = icel+1, icel+mult(nf)
          if (j.gt.icel+1) then
            k = icel + 1
            done = .false.
            do while (.not.done .and. k.le.(j-1))
              if (indcel(k).eq.indcel(j)) done = .true.
              k = k + 1
            enddo 
            if (done) then
              num = num + 1
              if (num.gt.maxmult) call error ('intred', 
     & 'the multiplicity is very large. try to increase maxmult', 
     &  faterr)
              norig(num) = k - 1
              ndest(num) = j
            endif
          endif  
          do lm = 0, nlm(nf)
            if (lm.eq.0) then
              Qm(0,j) = -zn(nf)
            else
              Qm(lm,j) = zero
            endif
          enddo
        enddo  
c.....STEP II: Obtaining the multipole moments
c              (only the differents moments)
        do lpq = 1, ndens(nf)
          l = mlmin(lpq,nf) !  mlmin(lpq,nf) = abs(lp-lq)
          do lm = dmlmin(lpq,nf), dmlmax(lpq,nf)
            if (lm.gt.il0(l)+l) l = l + 1
            m = lm - il0(l)
c......STEP IIa: Calculate the first multipole moment
            if (abs(itrot).eq.1) then
              isig = 1 
              if (itrot.lt.0 .and. mod(l,2).eq.1) isig = -1
              if (qmult(lpq,lm,nf).ne.zero) then
                Qm(lm,icel+1) = Qm(lm,icel+1) + rhomat(lpq,nf)
     &                         * isig * qmult(lpq,lm,nf)
              endif
            else
c             indmat(23) ... pointer that indicates the Wigner 
c                            coefficients for proper rotation
              ij = indmat(abs(itrot))
              isig = 1
              if (itrot.lt.0 .and. mod(l,2).eq.1) isig = -1
              udpqloc = zero
              do m2 = -l, l
                lm2 = il0(l) + m2
                indrot = irot0(l)+(l+m)*(2*l+1)+m2+l
                udpqloc = udpqloc + isig*rotr(indrot,ij)
     &                   * udpq(lpq,lm2,nf)
              enddo
              if (udpqloc.ne.zero) then
                qmultloc = udpqloc * xncte(l)
     &                    * xpq(lpq,nf) * fact(npq(lpq,nf)+l)
     &                    / zpq(lpq,nf)**(npq(lpq,nf)+l+1)
                Qm(lm,icel+1) = Qm(lm,icel+1) + rhomat(lpq,nf)
     &                         * qmultloc
              endif
            endif
c......STEP IIb: Calculate the remaining multipole moments,
c                but only the different moments  
            do j = icel+2, icel+mult(nf)
              k = icel + 1
              done = .false.
              do while (.not.done .and. k.le.(j-1))
                if (indcel(k).eq.indcel(j)) done = .true.
                k = k + 1
              enddo
              if (.not.done) then
                if (abs(indcel(j)).eq.1) then
                  isig = 1
                  if (indcel(j).lt.0 .and. mod(l,2).eq.1) isig = -1
                  if (qmult(lpq,lm,nf).ne.zero) then
                    Qm(lm,j) = Qm(lm,j) + rhomat(lpq,nf)
     &                        * isig * qmult(lpq,lm,nf)
                  endif
                else
c                 indmat(23) ... pointer that indicates the Wigner 
c                                coefficients for proper rotation
                  ij = indmat(abs(indcel(j)))
                  isig = 1
                  if (indcel(j).lt.0 .and. mod(l,2).eq.1) isig = -1
                  udpqloc = zero
                  do m2 = -l, l
                    lm2 = il0(l) + m2
                    indrot = irot0(l)+(l+m)*(2*l+1)+m2+l
                    udpqloc = udpqloc + isig*rotr(indrot,ij)
     &                       * udpq(lpq,lm2,nf)
                  enddo
                  if (udpqloc.ne.zero) then
                    qmultloc = udpqloc * xncte(l)
     &                        * xpq(lpq,nf) * fact(npq(lpq,nf)+l)
     &                        / zpq(lpq,nf)**(npq(lpq,nf)+l+1)
                    Qm(lm,j) = Qm(lm,j) + rhomat(lpq,nf)
     &                        * qmultloc
                  endif
                endif
              endif
            enddo 
          enddo ! lm = dmlmin(lpq,nf), ..., dmlmax(lpq,nf)
        enddo ! lpq = 1, ..., ndens(nf)
c.....STEP III: Store the rest of multipole moments
        do lm = 0, nlm(nf)
          do j = 1, num 
            Qm(lm,ndest(j)) = Qm(lm,norig(j))
          enddo
        enddo
c
        icel = icel + mult(nf)
      enddo ! nf = 1, ..., nneq
c                    
c....Multipolar electrostatic interaction and nucleus-lattice
c        interaction energy (long range)
c
      if (infsystem) then
       if (nimpre.ge.-1) write (luw,100) ni, namneq(ni)
       i = 0
       do lpq = 1, ndens(ni)
         l = mlmin(lpq,ni)
         do lm = dmlmin(lpq,ni), dmlmax(lpq,ni)
           if (lm.gt.il0(l)+l) l = l + 1
           if (qmult(lpq,lm,ni).ne.zero) then
             ok = .false. 
             j = 1
             do while (j.le.i .and. .not.ok)
               if (lmm(j).eq.lm) then
                 vlm = vmadlm(j)
                 ok = .true.
               endif
               j = j + 1
             enddo
             if (.not.ok) then
               i = i + 1
               lmm(i) = lm 
               m = lm - il0(l)
               vmadlm(i) = tewald(xneq(ni),yneq(ni),zneq(ni),qneq(ni),
     &                            l,m,nshell(i))
               vlm = vmadlm(i)
             endif
             vmp(lpq) = vmp(lpq) + qmult(lpq,lm,ni)*vlm
           endif
         enddo
       enddo
       call sortrx (i,vmadlm,iord)
       if (nimpre.ge.-1) then
         do j = 1, i
           write (luw,200) lmm(iord(j)), vmadlm(iord(j)),
     &                     nshell(iord(j))
         enddo
       endif
      else
       do lpq = 1, ndens(ni)
         l1 = mlmin(lpq,ni)
         do lm1 = dmlmin(lpq,ni), dmlmax(lpq,ni)
           if (lm1.gt.il0(l1)+l1) l1 = l1 + 1
           m1 = lm1 - il0(l1)
           do nf = 1, nimtot
             rr1 = ximp(nf) - xneq(ni)
             rr2 = yimp(nf) - yneq(ni)
             rr3 = zimp(nf) - zneq(ni)
             call tocart (rr1,rr2,rr3,xa,xb,xc)
             r = dis(xa,xb,xc)
             if (r.gt.epsdis) then ! S .ne. R
               call rsh (xa,xb,xc,r,slm,maxl)
               l2 = 0
               do lm2 = 0, nlm(typimp(nf))
                 if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                 if (Qm(lm2,nf).ne.zero) then
                   isig = 1
                   if (mod(l2,2).eq.1) isig = -1
                   m2 = lm2 - il0(l2)
                   ltot = l1 + l2
                   iltot = il0(ltot)
                   i = ig0(lm1,lm2,ltot)
                   rexp = r**(ltot+1)     
                   tslm = zero
                   do im3 = 1, m3num(m1,m2,ltot)
                     mtot = m3val(m1,m2,im3)
                     lmtot = iltot + mtot
                     tslm = tslm + dgaunt(i+im3)*slm(lmtot)
                   enddo
                   vmp(lpq) = vmp(lpq) + qmult(lpq,lm1,ni)
     &             *isig*coefq(l1,l2)*Qm(lm2,nf)*tslm/rexp
                 endif
               enddo 
             endif
           enddo 
         enddo 
       enddo
      endif    
c....Obtain the long range local potential (nucleus-lattice interaction)
      if (infsystem) then
        potlr = sewald(xneq(ni),yneq(ni),zneq(ni),qneq(ni),nsh)
        if (nimpre.ge.-1)
     &    write (luw,300) potlr, nsh
      else
        do nf = 1, nimtot
          rr1 = ximp(nf) - xneq(ni)
          rr2 = yimp(nf) - yneq(ni)
          rr3 = zimp(nf) - zneq(ni)
          call tocart (rr1,rr2,rr3,xa,xb,xc)
          r = dis(xa,xb,xc)
          if (r.gt.epsdis) then ! S .ne. R
            call rsh (xa,xb,xc,r,slm,maxmp)
            l = 0
            do lm = 0, nlm(typimp(nf))
              if (lm.gt.il0(l) + l) l = l + 1
              if (Qm(lm,nf).ne.zero) then
                isig = 1
                if (mod(l,2).eq.1) isig = -1
                rexp = r**(l+1)
                potlr = potlr + isig*xncte(l)*Qm(lm,nf)*slm(lm)/rexp    
              endif
            enddo
          endif 
        enddo
      endif  
c
c.....(1) wrint/nwint: compute the integrals from scratch
c
      if (typrun .ne. rdint) then
c
c.....Create the environment around each ion (only when computing the
c     integrals)
c
        if (infsystem) then
          call envcrys (ni, nimpre, luw)
        else
          call envclus (ni, nimpre, luw)
        endif
c
c.....Loop over different types of environment ions
c
        isha = 0
        do nf = 1, nneq
          if (nimpre.ge.-1) write (luw,610) namneq(nf), nf
          xs = xocup(nf) 
          do lab = 1, ndens(nf)
            onuc2(lab) = zero
            do lpq = 1, ndens(ni)
              oncoul(lab,lpq) = zero
              onxp(lab,lpq)   = zero
            enddo
          enddo
          convnuc1 = .false.
          convnuc2 = .false.
          convcoul = .false.
          convover = .false.
          done = nenvni(nf).le.0 
c         nenvni(nf) -> shells formed by nf--ions
          ish = isha + 1
          do while (.not.done)
            rs = renv(ish)
c           rs -> distance between the active ion 
c                 and the ish--shell
            tnuc1 = zero
            tnuc2 = zero
            tcoul = zero
            txp   = zero
            do lpq = 1, ndens(ni)
              onuc1(lpq) = zero
              do lab = 1, ndens(nf)
                if (lpq.eq.1) then
                  tvnuc2(lab) = zero
                endif
                tvcoul(lab,lpq) = zero
                tvxp(lab,lpq)   = zero 
              enddo
            enddo  
            do istar = 1, nstar(ish) 
c           nstar(ish) -> number of stars for the ish--shell
c...........Coordinate conversion to cartesian coordinates
              call tocart (xxstar(1,istar,ish), xxstar(2,istar,ish),
     &                     xxstar(3,istar,ish), xa, xb, xc)
c...........Compute the spherical harmonics
              call rsh (xa, xb, xc, rs, slm, maxl)
c...........Obtaining the T^l_{lama,lbmb}(R) functions
              j = 0
              l1 = 0
              do lm1 = 0, lmmax
                if (lm1.gt.il0(l1)+l1) l1 = l1 + 1
                 m1 = lm1 - il0(l1)
                 l2 = 0
                 do lm2 = 0, lm1
                   if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                   m2 = lm2 - il0(l2)
                   do l3 = l1-l2, l1+l2, 2
                     j = j + 1
                     i = ig0(lm1,lm2,l3)
                     itfunc(lm1,lm2,l3) = j
                     itfunc(lm2,lm1,l3) = j
                     tfunc(j) = zero ! initializations
                     do im3 = 1, m3num(m1,m2,l3)
                       m3 = m3val(m1,m2,im3)
                       lm3 = il0(l3) + m3
                       tfunc(j) = tfunc(j) + dgaunt(i+im3) * slm(lm3)
                     enddo
                   enddo
                 enddo
              enddo
c...........Obtaining the coefUm and udpq coefficients for the
c           representative of the star
              if (abs(indstar(istar,ish)).eq.1) then
                ivstar = 1
                if (indstar(istar,ish).lt.0) ivstar = 2
              else
                ivstar = 3
                ij = indmat(abs(indstar(istar,ish)))
                ok = indstar(istar,ish).lt.0
                do lab = 1, ndens(nf)
                  la  = denlp(lab,nf)
                  lb  = denlq(lab,nf)   
                  ok2 = mod(la+lb,2).eq.1
c.................STEP I: Obtaining the new coefUm for the representative
c                         of the star
                  do ma = -la, la
                    do mb= -lb, lb
                      Umstar(lab,ma,mb) = zero 
                      do ma2 = -la, la
                        irot1 = irot0(la) + (la+ma)*(2*la+1)+ma2+la
                        temp = zero
                        do mb2 = -lb, lb
                          irot2 = irot0(lb) + (lb+mb)*(2*lb+1)+mb2+lb
                          temp = temp + coefUm(lab,ma,mb,nf)
     &                                  *rotr(irot2,ij)
                        enddo
                        Umstar(lab,ma,mb) = Umstar(lab,ma,mb) +
     &                                      rotr(irot1,ij)*temp
                      enddo
                      if (ok.and.ok2) then 
                        Umstar(lab,ma,mb) = - Umstar(lab,ma,mb)
                      endif 
                    enddo 
                  enddo
c.................STEP II: Obtaining the new udpq for the representative
c                          of the star
                  l = mlmin(lab,nf)
                  do lm = dmlmin(lab,nf), dmlmax(lab,nf)
                    if (lm.gt.il0(l)+l) l = l + 1
                    m = lm - il0(l)
                    udstar(lab,lm) = zero
                    do m2 = -l, l
                      lm2 = il0(l) + m2
                      irot1 = irot0(l) + (l+m)*(2*l+1)+m2+l
                      udstar(lab,lm) = udstar(lab,lm) +
     &                                 rotr(irot1,ij)*udpq(lab,lm2,nf) 
                    enddo
                    if (ok.and.ok2) udstar(lab,lm) = - udstar(lab,lm)
                  enddo
                enddo ! lab      
              endif
c.....(a) Nuclear electrostatic interaction: short range term
              if (.not.convnuc1) then
c...........Nuclear attraction integral
                call nuclear (ni, rs, .false., o, 1)
                do lpq = 1, ndens(ni)
                  onuc1(lpq) =  onuc1(lpq) + 
     &                          nelst(istar,ish)*o(lpq,1)
                enddo
              endif 
c.....(b) Determine the nucleus-lattice interaction energy
              if (.not.convnuc2) then
c...........Local potential (short range)
                call nuclear (nf, rs, .true., o, ivstar)
                do lab = 1, ndens(nf)
                  tvnuc2(lab) = tvnuc2(lab) +  
     &                          nelst(istar,ish)*o(lab,1)
                enddo
              endif
c.....(c) Coulomb electrostatic interaction: short range term
              if (.not.convcoul) then
c...........Coulomb integral
                call coulint (ni, nf, rs, o, ivstar)
                do lpq = 1, ndens(ni)
                  do lab = 1, ndens(nf)
                    tvcoul(lab,lpq) = tvcoul(lab,lpq) +
     &                                nelst(istar,ish)*o(lpq,lab)
                  enddo
                enddo
              endif
              if (.not.convover) then
c.....(d) Exchange & (d) projection operator contributions
c...........Overlap integral
                call overlap (ni, nf, rs, o, ivstar)
                do lpq = 1, ndens(ni)
                  do lab = 1, ndens(nf)
                    tvxp(lab,lpq) = tvxp(lab,lpq) + 
     &                              nelst(istar,ish)*o(lpq,lab)
                  enddo
                enddo
              endif 
            enddo  ! istar = 1, ..., nstar(ish)
            if (.not.convnuc1) then
              do lpq = 1, ndens(ni)     
                onuc1(lpq) = - xs * zn(nf) * onuc1(lpq)
                vnsr(lpq) = vnsr(lpq) + onuc1(lpq)
                relerr = abs(onuc1(lpq)/(vnsr(lpq)+tiny))
                if (relerr.gt.tnuc1) then
                  tnuc1 = relerr
                endif
              enddo
              if (nimpre.ge.4) then
                write (luw,620) ish-isha, 'Vnsr'
                write (luw,710) (onuc1(lpq), lpq = 1, ndens(ni))
              endif
              ishnuc = ish - isha 
              convnuc1 = tnuc1.le.convnsr
            endif
            if (.not.convnuc2) then
              do lab = 1, ndens(nf)
                tvnuc2(lab) = xs * tvnuc2(lab)
                onuc2(lab) = onuc2(lab) + tvnuc2(lab)
                potnl = potnl + tvnuc2(lab) * rhomat(lab,nf)
                relerr = abs(tvnuc2(lab)/(onuc2(lab)+tiny))
                if (relerr.gt.tnuc2) then
                  tnuc2 = relerr
                endif
              enddo
              if (nimpre.ge.4) then
                write (luw,620) ish-isha, 'Vnlsr'
                write (luw,710) (tvnuc2(lab), lab = 1, ndens(nf))
              endif
              ishloc = ish - isha 
              convnuc2 = tnuc2.le.convnl
            endif
            if (.not.convcoul) then
              do lpq = 1, ndens(ni)
                vetmp = zero ! initializations
                do lab = 1, ndens(nf)
                  tvcoul(lab,lpq) = tvcoul(lab,lpq) * xs
                  oncoul(lab,lpq) = oncoul(lab,lpq) + tvcoul(lab,lpq)
                  vetmp = vetmp + tvcoul(lab,lpq)*rhomat(lab,nf)
                  relerr = abs(tvcoul(lab,lpq)/(oncoul(lab,lpq)+tiny))
                 if (relerr.gt.tcoul) then
                   tcoul = relerr
                 endif
                enddo
                vesr(lpq) = vesr(lpq) + vetmp
              enddo
              if (nimpre.ge.4) then
                write (luw,620) ish-isha, 'Vesr'
                do lpq = 1, ndens(ni)
                  write (luw,710) (tvcoul(lab,lpq), lab = 1, ndens(nf))
                enddo
              endif
              ishcoul = ish - isha 
              convcoul = tcoul.le.convesr
            endif 
            if (.not.convover) then
              do lpq = 1, ndens(ni)
                vxtmp = zero ! initializations
                vptmp = zero ! initializations
                do lab = 1, ndens(nf)
                 tvxp(lab,lpq) = tvxp(lab,lpq) * xs
                 onxp(lab,lpq) = onxp(lab,lpq) + tvxp(lab,lpq)
                 vxtmp = vxtmp + tvxp(lab,lpq) * aexch(lab,nf)
                 vptmp = vptmp + tvxp(lab,lpq) * aproj(lab,nf)
                 relerr = abs(tvxp(lab,lpq)/(onxp(lab,lpq)+tiny))
                 if (relerr.gt.txp) then
                  txp = relerr
                 endif
                enddo
                vex(lpq) = vex(lpq) - vxtmp
                pla(lpq) = pla(lpq) + vptmp
              enddo
              if (nimpre.ge.4) then
                write (luw,620) ish-isha, 'Vxp'
                do lpq = 1, ndens(ni)
                  write (luw,710) (tvxp(lab,lpq), lab = 1, ndens(nf))
                enddo
              endif
              ishover = ish - isha 
              convover = txp.le.convxp
            endif
            ish = ish + 1
            done = convnuc1.and.convnuc2.and.convcoul.and.convover 
     &             .or. ish.gt.isha+nenvni(nf)
          enddo ! while
          if (.not. convnuc1 .and. infsystem) then
            write (luw,810) 'Vnsr', ishnuc, tnuc1
            cdum = 'Vnsr is not fully converged'
            call error ('intred', cdum, warning) 
          else
            if (nimpre.ge.-1) write (luw,800) 'Vnsr', ishnuc
          endif
          if (.not. convnuc2 .and. infsystem) then
            write (luw,810) 'Vnlsr', ishloc, tnuc2
            cdum = 'Vnlsr is not fully converged'
            call error ('intred', cdum, warning) 
          else
            if (nimpre.ge.-1) write (luw,800) 'Vnlsr', ishloc
          endif
          if (.not. convcoul .and. infsystem) then
            write (luw,810) 'Vesr', ishcoul, tcoul
            cdum = 'Vesr is not fully converged'
            call error ('intred', cdum, warning) 
          else
            if (nimpre.ge.-1) write (luw,800) 'Vesr', ishcoul
          endif
          if (.not. convover .and. infsystem) then
            write (luw,810) 'Vxp', ishover, txp
            cdum = 'Vxp is not fully converged'
            call error ('intred', cdum, warning) 
          else
            if (nimpre.ge.-1) write (luw,800) 'Vxp', ishover
          endif
          if (typrun.eq.wrint) then
            write (luj) (onuc2(lab), lab = 1, ndens(nf))
            do lpq = 1, ndens(ni)
              write (luj) (oncoul(lab,lpq), lab = 1, ndens(nf))
            enddo
            do lpq = 1, ndens(ni)
              write (luj) (onxp(lab,lpq), lab = 1, ndens(nf))
            enddo
          endif
c
c.....end loop of local ions
c
          isha = isha + nenvni(nf)
        enddo
        if (typrun.eq.wrint) write (luj) (vnsr(lpq), lpq = 1, ndens(ni))
        if (nimpre.ge.-1) write (luw,900)
c
c.....(2) rdint: read the integrals from a file
c
      else
        do nf = 1, nneq
          read (luj) (oneq(lab,1), lab = 1, ndens(nf))
          do lab = 1, ndens(nf)
            potnl = potnl + rhomat(lab,nf) * oneq(lab,1)
          enddo
          do lpq = 1, ndens(ni)
            read (luj) (oneq(lab,lpq), lab = 1, ndens(nf))
            do lab = 1, ndens(nf)
              vesr(lpq) = vesr(lpq) + rhomat(lab,nf) * oneq(lab,lpq)
            enddo
          enddo
          do lpq = 1, ndens(ni)
            read (luj) (oneq(lab,lpq), lab = 1, ndens(nf))
            do lab = 1, ndens(nf)
              vex(lpq) = vex(lpq) - aexch(lab,nf) * oneq(lab,lpq)
              pla(lpq) = pla(lpq) + aproj(lab,nf) * oneq(lab,lpq)
            enddo
          enddo
        enddo
        read (luj) (vnsr(lpq), lpq = 1, ndens(ni))
      endif
c
c.....(3) store global contributions
c
      enllr = -zn(ni)*potlr
      enlsr = -zn(ni)*potnl 
      do lpq = 1, ndens(ni)
         v(lpq) = vmp(lpq) + vnsr(lpq) + vesr(lpq) + vex(lpq) 
     &           + pla(lpq)
      enddo
c
c.....close integral file:
c
      if (typrun.ne.nwint) then
        close (luj)
        call lunalloc (luj)
      endif
c
c.....print out the results:
c
      if (nimpre .ge. 0) then
         write (luw,700) potlr, potnl, enllr+enlsr
      endif
      if (nimpre .ge. 2) then
         write (luw,705) 'Multipolar potential'
         write (luw,710) (vmp(k), k = 1, ndens(ni))
         write (luw,705) 'Nuclear potential (short range)'
         write (luw,710) (vnsr(k), k = 1, ndens(ni))
         write (luw,705) 'Coulomb potential (short range)'
         write (luw,710) (vesr(k), k = 1, ndens(ni))
         write (luw,705) 'Exchange operators'
         write (luw,710) (vex(k), k = 1, ndens(ni))
         write (luw,705) 'Projection operators'
         write (luw,710) (pla(k), k = 1, ndens(ni))
      endif
c
      if (typrun .eq. wrint) then
         call prtimer (4,ipid,'intred(w)',-1)
      else if (typrun .eq. nwint) then
         call prtimer (4,ipid,'intred(n)',-1)
      else
         call prtimer (4,ipid,'intred(r)',-1)
      endif
      return
c
 100  format (1x, ' Madelung Potential',/,
     &        1x, ' Ion Active # ', i3, 1x, a6)
 200  format (1x,' Madelung potential Vmad(',i3,'): ', 1pe17.8,
     &           '  Shells added up: ', i3)
 300  format (1x,' Madelung potential Vloc,lr: ', 1pe17.8,
     &           '  Shells added up: ', i3)
 600  format (/'*** LATTICE INTEGRALS FOR THE ACTIVE ION : ',
     &        a, ' (no. ',i2,' )'/)
 610  format (/1x, ' Lattice integrals due to ', a, ' (no. ', i2, ' )')
 620  format (/1x,'+ Shell no. ', i2, 1x, a, ' integrals +')
 700  format (/
     & 1x, 'Long range local potential at the active site : ', f21.14/
     & 1x, 'Short range local potential at the active site: ', f21.14/
     & 1x, 'Total nucleus-lattice energy                  : ',1pe21.14 )
 705  format (/1x,'+ Global lattice integrals due to: ', a, '+')
 710  format (1x, 1p, 10e13.5)
 800  format (1x,'Shells used to converge ',a,' : ',i4)
 810  format (a, ' not converged. Last shell: ', i4,
     &           ' Integral error (rel.): ', e13.6)
 900  format (/)
c
      end
