c
c-----------------------------------------------------------------------
c
      subroutine istupq (file, ni, typcloc, typrun, luw, nimpre)
c
c.....istupq - computes the monocentric integrals of NI.
c
c.....Input parameters:
c     file ..... (read/write) integrals (from/to) this file.
c     ni ....... active ion.
c     typcloc .. local correlation type (either typHF or typrCHF)
c     typrun ... rdint : read in integrals from file.
c                wrint : compute the integrals and write them to file.
c                nwint : compute the integrals without writing them.
c     luw ...... print output to this LU.
c     nimpre ... <0 : no print out.
c                0  : print headings.
c                >0 : verbose (print every matrix element).
c
      include           'implicit.inc'
      include           'global.inc'
      include           'coed.inc'
      include           'cont.inc'
      include           'consts.inc'
      include           'corr.inc'
      include           'coup.inc'
      include           'error.inc'
      include           'fact.inc'
      include           'intg.inc'
      include           'rsh.inc'
      include           'dgaunt.inc'
      include           'iondens.inc'
      include           'stos.inc'
      include           'symmetry.inc'
      include           'strtools.inc'
      include           'types.inc'
      include           'neqatm.inc'
c
      character*(*)     file
      integer           ni, nimpre, luw, typrun, typcloc
c
      integer           ip, iq, ir, is, lpq, lrs
      integer           lualloc
      real*8            coef(msa)
      character*(128)   cdum
      logical           writelux, alleq
c
c.....Begin:
c
      numpq = (ndens(ni) * (ndens(ni)+1)) / 2
      if (numpq .gt. maxpq) call error ('istupq',
     &   'too many charge densities. Increase mdens parameter', faterr)
c
c.....Open the integral file:
c
      lux = lualloc (lux)
c     writelux = .true. ('wrint') / .false. ('nwint')
      writelux = .true. 
c
c.....Use rdint mode if requested and possible:
c
      if (typrun .eq. rdint) then
         open (lux, file=file, form='unformatted', status='old',
     &         iostat=nios)
         if (nios .ne. 0) then
            cdum = 'Cannot open old int file:'//file(1:leng(file))//
     &         newline//'NWINT mode will be used'
            call error ('istupq', cdum, warning)
            call lunalloc (lux)
            writelux = .false.
         else
            goto 2
         endif
      endif
c
c.....Use wrint mode if requested, nwint otherwise:
c
 1    if (typrun .eq. wrint) then
        open (lux, file=file, form='unformatted', status='unknown',
     &        iostat=nios)
        if (nios .ne. 0)  then
          cdum = 'Cannot open int file:'//file(1:leng(file))//
     &       newline//'NWINT mode will be used'
          call error ('istupq', cdum, warning)
          call lunalloc (lux)
          writelux = .false.
          call prtimer (2, ipid, 'istupq(n)', -1)
        else
          call prtimer (2, ipid, 'istupq(w)', -1)
        endif
      else
        call lunalloc (lux)
        writelux = .false.
        call prtimer (2, ipid, 'istupq(n)', -1)
      endif
c
c     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c     +++           WRINT/NWINT mode                                 +++
c     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c.....Compute the monoelectronic integrals over the primitives:
c
      if (nimpre .ge. 2) write (luw,600) namneq(ni), ni
      do lpq = 1, ndens(ni)
         lp = denlp(lpq,ni) 
         lq = denlq(lpq,ni) 
         kp = denkp(lpq,ni) 
         kq = denkq(lpq,ni) 
         if (lp.eq.lq .and. kp.eq.kq) then 
           iq = denq(lpq,ni) + nasto(lq+1,ni)
           nq = n(iq,ni) 
           zq = z(iq,ni) 
           nnpq = npq(lpq,ni) 
           zzpq = zpq(lpq,ni) 
           xxpq = xpq(lpq,ni) 
           ss(lpq) = xxpq * fact(nnpq) / (zzpq**(nnpq+1))
           u(lpq)  = - (zn(ni) * xxpq * fact(nnpq-1) / zzpq**nnpq)
           t(lpq)  = - (half * xxpq * (
     &             (nq*(nq-1)-lp*(lp+1)) * fact(nnpq-2) / zzpq**(nnpq-1)
     &             - two * nq * zq * fact(nnpq-1) / zzpq**nnpq +
     &             zq * zq * fact(nnpq) / zzpq**(nnpq+1) ))
         else
           ss(lpq) = zero
           u(lpq) = zero
           t(lpq) = zero
         endif  
      enddo
c
c.....Compute the bielectronic integrals over the primitives:
c
c.....A) Compute closed-shell P=J-K/2 supermatrix:
c
      k = 0
      do lpq = 1, ndens(ni)
         irrep1 = denirrep(lpq,ni)
         lp = denlp(lpq,ni)
         lp0 = il0(lp)
         lq = denlq(lpq,ni)
         lq0 = il0(lq)
         ip = denp(lpq,ni) + nasto(lp+1,ni) 
         iq = denq(lpq,ni) + nasto(lq+1,ni) 
         npl = n(ip,ni) 
         zpl = z(ip,ni) 
         xpl = xnsto(ip,ni) 
         nql = n(iq,ni) 
         zql = z(iq,ni) 
         xql = xnsto(iq,ni) 
         do lrs = 1, lpq
            irrep2 = denirrep(lrs,ni)
            lr  = denlp(lrs,ni)
            lr0 = il0(lr)
            ls = denlq(lrs,ni)
            ls0 = il0(ls)
            ir = denp(lrs,ni) + nasto(lr+1,ni)   
            is = denq(lrs,ni) + nasto(ls+1,ni) 
            nrl = n(ir,ni) 
            zrl = z(ir,ni) 
            xrl = xnsto(ir,ni) 
            nsl = n(is,ni) 
            zsl = z(is,ni) 
            xsl = xnsto(is,ni) 
            k = k + 1
            ipq = 0
            ips = 0
            ipr = 0
            irs = 0
            irq = 0
            isq = 0
            if (mod(lp+lq,2).eq.1) ipq = 1
            if (mod(lp+ls,2).eq.1) ips = 1
            if (mod(lp+lr,2).eq.1) ipr = 1
            if (mod(lr+ls,2).eq.1) irs = 1
            if (mod(lr+lq,2).eq.1) irq = 1
            if (mod(ls+lq,2).eq.1) isq = 1
c
c..........First, J integrals: 
            p(k) = zero
            if (ipq.eq.irs) then
              lmin = max(abs(lp-lq),abs(lr-ls))
              lmaxx = min(lp+lq,lr+ls)
c.............Obtain coef
              il = 0
              do l = lmin, lmaxx, 2
                l0 = il0(l)
                il = il + 1
                coef(il) = zero
                do m = -l, l
                  lm = l0 + m
                  coef(il) = coef(il) + udpq(lpq,lm,ni)*udpq(lrs,lm,ni)
                enddo
                coef(il) = four*pi*coef(il)/(l+l+1)
              enddo ! l
c
              il = 0
              if (typcloc .eq. typuCHF .or. typcloc .eq. typrCHF)
     &            alleq = lp.eq.lq .and. lq.eq.lr .and. lr.eq.ls
              do l = lmin, lmaxx, 2
                il = il + 1
                if (typcloc .eq. typuCHF .or. typcloc .eq. typrCHF) then
                  p(k) = p(k) + coef(il)*rcsupk (npl, zpl, xpl, nrl,
     &                        zrl, xrl, nql, zql, xql, nsl, zsl, xsl, l,
     &                        alleq, con1(int(zn(ni))),
     &                        con2(int(zn(ni))) )
                else 
                  p(k) = p(k) + coef(il)*rsupk (npl, zpl, xpl, nrl,
     &                             zrl, xrl, nql, zql, xql, nsl, zsl,
     &                             xsl, l)
                endif
              enddo
            endif       
c
c..........Then K integrals: 
            xk(k) = zero
            if (ips.eq.irq) then
              lmin = max(abs(lp-ls),abs(lr-lp))
              lmaxx = min(lp+ls,lr+lq)
c.............Obtain coef
              il = 0
              do l = lmin, lmaxx, 2
                il = il + 1
                coef(il) = zero
                do mp = -lp, lp
                  lmp = lp0 + mp
                  do mq = -lq, lq
                    lmq = lq0 + mq
                    Umpmq = coefUm(lpq,mp,mq,ni)
                    tmp = zero
                    do mr = -lr, lr
                      lmr = lr0 + mr
                      ilrq = ig0(lmr,lmq,l)
                      do ms = -ls, ls
                        lms = ls0 + ms
                        Umrms = coefUm(lrs,mr,ms,ni)
                        ilps = ig0(lmp,lms,l)
                        tmp2 = zero
                        do imps = 1, m3num(mp,ms,l)
                          m1 = m3val(mp,ms,imps)
                          do imrq = 1, m3num(mr,mq,l)
                            m2 = m3val(mr,mq,imrq)
                            if (m2.eq.m1) then
                               aux = dgaunt(ilps+imps)*dgaunt(ilrq+imrq)
                               tmp2 = tmp2 + aux
                            endif
                          enddo
                        enddo  
                        tmp = tmp + Umrms*tmp2
                      enddo ! ms
                    enddo ! mr
                    coef(il) = coef(il) + Umpmq*tmp
                  enddo ! mq
                enddo ! mp
                coef(il) = four*pi*coef(il)/(l+l+1)
              enddo ! l
c
              il = 0
              do l = lmin, lmaxx, 2
                il = il + 1
                xk(k) = xk(k) + coef(il)*rsupk(npl,zpl,xpl,nrl,zrl,
     &                                   xrl,nsl,zsl,xsl,nql,zql,xql,l)
              enddo
            endif
            if (ipr.eq.isq) then 
              lmin = max(abs(lp-lr),abs(ls-lq))
              lmaxx = min(lp+lr,ls+lq)
c.............Obtain coef
              il = 0
              do l = lmin, lmaxx, 2
                il = il + 1
                coef(il) = zero
                do mp = -lp, lp
                  lmp = lp0 + mp
                  do mq = -lq, lq
                    lmq = lq0 + mq
                    Umpmq = coefUm(lpq,mp,mq,ni)
                    tmp = zero
                    do mr = -lr, lr
                      lmr = lr0 + mr
                      ilpr = ig0(lmp,lmr,l)
                      do ms = -ls, ls
                        lms = ls0 + ms
                        Umrms = coefUm(lrs,mr,ms,ni)
                        ilsq = ig0(lms,lmq,l)
                        tmp2 = zero
                        do impr = 1, m3num(mp,mr,l)
                          m1 = m3val(mp,mr,impr)
                          do imsq = 1, m3num(ms,mq,l)
                            m2 = m3val(ms,mq,imsq)
                            if (m2.eq.m1) then
                              aux = dgaunt(ilpr+impr)*dgaunt(ilsq+imsq)
                              tmp2 = tmp2 + aux
                            endif
                          enddo
                        enddo  
                        tmp = tmp + Umrms*tmp2
                      enddo ! ms
                    enddo ! mr
                    coef(il) = coef(il) + Umpmq*tmp
                  enddo ! mq
                enddo ! mp
                coef(il) = four*pi*coef(il)/(l+l+1)
              enddo ! l
c
              il = 0
              do l = lmin, lmaxx, 2
                il = il + 1
                xk(k) = xk(k) + coef(il)*rsupk(npl,zpl,xpl,nsl,zsl,
     &                                   xsl,nrl,zrl,xrl,nql,zql,xql,l)
              enddo
            endif 
c
c...........Finally, add together J and K to construct the P matrix
            p(k) = p(k) - fourth * xk(k)
         enddo ! lrs
      enddo ! lpq
c
c.....B) Compute open-shell supermatrix:
c
      if (openshell(ni)) then
c
c........J terms:
c
         k = 0
         do lpq = 1, ndens(ni) 
            irrep1 = denirrep(lpq,ni)
            lp = denlp(lpq,ni)
            lq = denlq(lpq,ni)
            ip = denp(lpq,ni) + nasto(lp+1,ni)
            iq = denq(lpq,ni) + nasto(lq+1,ni)
            npl = n(ip,ni) 
            zpl = z(ip,ni) 
            xpl = xnsto(ip,ni) 
            nql = n(iq,ni) 
            zql = z(iq,ni) 
            xql = xnsto(iq,ni) 
            do lrs = 1, lpq 
               irrep2 = denirrep(lrs,ni)
               lr = denlp(lrs,ni) 
               ls = denlq(lrs,ni) 
               ir = denp(lrs,ni) + nasto(lr+1,ni) 
               is = denq(lrs,ni) + nasto(ls+1,ni) 
               nrl = n(ir,ni) 
               zrl = z(ir,ni) 
               xrl = xnsto(ir,ni) 
               nsl = n(is,ni) 
               zsl = z(is,ni) 
               xsl = xnsto(is,ni)
               k = k + 1 
               q(k) = zero ! initializations
               if (typcloc .eq. typuCHF .or. typcloc .eq. typrCHF)
     &             alleq = lp.eq.lq .and. lq.eq.lr .and. lr.eq.ls
               do j = 1, numjcoup(ni)
c              numjcoup(ni) -> number of k's in J(lpq,lrs,k) coupling constants 
                  nu = njcoup(lpq,lrs,j,ni)
                  if (typcloc .eq. typuCHF .or.
     &                typcloc .eq. typrCHF) then
                     xj = rcsupk (npl, zpl, xpl, nrl, zrl, xrl,
     &                            nql, zql, xql, nsl, zsl, xsl, nu,
     &                            alleq, con1(int(zn(ni))),
     &                            con2(int(zn(ni))) )
                  else
                     xj = rsupk (npl, zpl, xpl, nrl, zrl, xrl,
     &                           nql, zql, xql, nsl, zsl, xsl, nu)
                  endif
                  q(k) = q(k) + xjcoup(lpq,lrs,j,ni) * xj
               enddo
            enddo
         enddo
c
c........K terms:
c
         k = 0
         do lpq = 1, ndens(ni)
            lp = denlp(lpq,ni) 
            lq = denlq(lpq,ni)
            ip = denp(lpq,ni) + nasto(lp+1,ni)  
            iq = denq(lpq,ni) + nasto(lq+1,ni)
            npl = n(ip,ni) 
            zpl = z(ip,ni) 
            xpl = xnsto(ip,ni) 
            nql = n(iq,ni) 
            zql = z(iq,ni) 
            xql = xnsto(iq,ni) 
            do lrs = 1, lpq 
               lr = denlp(lrs,ni)
               ls = denlq(lrs,ni) 
               ir = denp(lrs,ni) + nasto(lr+1,ni)
               is = denq(lrs,ni) + nasto(ls+1,ni)
               nrl = n(ir,ni)
               zrl = z(ir,ni)
               xrl = xnsto(ir,ni)
               nsl = n(is,ni)
               zsl = z(is,ni)
               k = k + 1
               do j = 1, numkcoup(ni)
c              numkcoup(ni) -> number of k's in K(lpq,lrs,k) 
c              coupling constants
                  nu = nkcoup(lpq,lrs,j,ni)
                  xk1 = rsupk (npl, zpl, xpl, nrl, zrl, xrl,
     &                         nsl, zsl, xsl, nql, zql, xql, nu)
                  xk2 = rsupk (npl, zpl, xpl, nsl, zsl, xsl, 
     &                         nrl, zrl, xrl, nql, zql, xql, nu)
                  q(k) = q(k) - fourth * (xkcoup(lpq,lrs,k,1,ni)*xk1
     &                                  + xkcoup(lpq,lrs,k,2,ni)*xk2)
               enddo  
            enddo
         enddo
      endif
c
c.....Write the integrals in the printer file:
c
      if (nimpre.ge.2) then
         if (typcloc .eq. typuCHF .or. typcloc .eq. typrCHF)
     &      write (luw,700) con1(int(zn(ni))),con2(int(zn(ni)))
         write (luw,705)
         write (luw,710) (ss(i),i=1,ndens(ni))
         write (luw,715)
         write (luw,710) (t(i),i=1,ndens(ni))
         write (luw,720)
         write (luw,710) (-(u(i)/zn(ni)),i=1,ndens(ni))
         write (luw,725)
         write (luw,710) (p(i),i=1,numpq)
         if (openshell(ni)) then
            write (luw,730)
            write (luw,710) (q(i),i=1,numpq)
         endif
         write (luw,735)
         write (luw,710) (xk(i),i=1,numpq)
      endif
c
c.....Write the integrals in fich1. A binary format will be used.
c     The matrices will be written in the order: S,T,U,P,Q & K.
c
      if (writelux) then
        write (lux) (ss(i), i = 1, ndens(ni))
        write (lux) (t(i), i = 1, ndens(ni))
        write (lux) (u(i), i = 1, ndens(ni))
        write (lux) (p(i), i = 1, numpq)
        if (openshell(ni)) then
           write (lux) (q(i), i = 1, numpq)
        endif
        write (lux) (xk(i), i = 1, numpq)
        close (lux)
        call lunalloc (lux)
        call prtimer (4,ipid,'istupq(w)',-1)
      else
        call prtimer (4,ipid,'istupq(n)',-1)
      endif
c
c.....Return control to the main program:
c
      return
c
c     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c     +++           RDINT mode                                       +++
c     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c.....Read in the integrals from fich1:
c
 2    call prtimer (2, ipid, 'istupq(r)', -1)
      if (nimpre.ge.2) write (luw,600) namneq(ni), ni
      read (lux,err=1000,end=1000,iostat=nios) (ss(i),i=1,ndens(ni))
      read (lux,err=1000,end=1000,iostat=nios) (t(i),i=1,ndens(ni))
      read (lux,err=1000,end=1000,iostat=nios) (u(i),i=1,ndens(ni))
      read (lux,err=1000,end=1000,iostat=nios) (p(i),i=1,numpq)
      if (openshell(ni)) then
         read (lux,err=1000,end=1000,iostat=nios) (q(i),i=1,numpq)
      endif
      read (lux,err=1000,end=1000,iostat=nios) (xk(i),i=1,numpq)
      close (lux)
      call lunalloc (lux)
c
c.....Write the integrals in the printer file:
c
      if (nimpre.ge.2) then
         if (typcloc .eq. typuCHF .or. typcloc .eq. typrCHF)
     &      write (luw,700) con1(int(zn(ni))),con2(int(zn(ni)))
         write (luw,705)
         write (luw,710) (ss(i),i=1,ndens(ni)) 
         write (luw,715)
         write (luw,710) (t(i),i=1,ndens(ni))
         write (luw,720)
         if (zn(ni) .ne. zero) then
c        el condicional es innecesario si 'readion' verifica zn(ni)
            write (luw,710) (-(u(i)/zn(ni)),i=1,ndens(ni))
         endif
         write (luw,725)
         write (luw,710) (p(i),i=1,numpq)
         if (openshell(ni)) then
            write (luw,730)
            write (luw,710) (q(i),i=1,numpq)
         endif
         write (luw,735)
         write (luw,710) (xk(i),i=1,numpq)
      endif
c
c.....Return control to the main program:
c
      call prtimer (4,ipid,'istupq(r)',-1)
      return
c
c.....Error while reading data:
c
 1000 write (luw,500) nios
      call prtimer (4,ipid,'istupq(r)',-1)
      cdum = 'error while reading integrals from:'//
     &  file(1:leng(file))//newline//'NWINT mode will be used'
      call error ('istupq', cdum, warning)
      close (lux)
      goto 1
c
c.....Formats:
c
 500  format (//1x, 'Reading error   :',i5)
 600  format (/'*** MONOCENTRIC INTEGRALS FOR THE ACIVE ION : ',
     &        a, ' (no. ',i2,' )'/)
 700  format (///1x, '*** Coulomb hole treatment for the J integrals'/
     & 1x, '*** CON1 and CON2 values:',2f15.6/)
 705  format (/1x, 'S Integrals:')
 710  format (10f12.6)
 715  format (/1x, 'T Integrals:')
 720  format (/1x, 'U Integrals:')
 725  format (/1x, 'P Integrals:')
 730  format (/1x, 'Q Integrals:')
 735  format (/1x, 'K Integrals:')
      end
