c
c-----------------------------------------------------------------------
c
      subroutine jacvao (a,n,eivu,eivr)
c
c.....jacvao - diagonalization by Jacobi method.
c
c     This routine diagonalizes, using the Jacobi method, the real and
c     symmetric matrix contained in A(). Eigenvalues and eigenvectors
c     are returned in EIVU() and EIVR(), respectively. N represents the
c     actual dimensions of the matrices.
c
      include           'implicit.inc'
      dimension         a(n,n),eivu(n),eivr(n,n)
c
c.....parameters:
c     epstop (original 1d-9) : relative decrease tolerance before
c                              recomputing off-diagonal norm
c     eps0 (original 1d-12)  : rotate only if |aij/(aii-ajj)| > eps0
c                              (otherwise, rotation is negligible)
c     eps45 (original 1d-10) : if |(aii-ajj)/aij| < eps45, consider the
c                              rotation equal to 45 (pi/4)
      parameter         ( epstop = 1d-9 )
      parameter         ( eps0 = 1d-12 )
      parameter         ( eps45 = 1d-10 )
c
      if (n.le.1) then
         eivr(1,1)=1d0
         eivu(1)=a(1,1)
      endif
      if (n.le.1) return
      do 10 j=1,n
         do 15 i=1,n
            eivr(i,j)=0d0
 15      continue
         eivr(j,j)=1d0
 10   continue
c
c.....Find the largest element in A:
      atop=0d0
      do 20 i=1,n
         do 25 j=i,n
            if (abs(a(i,j)).gt.atop) atop=abs(a(i,j))
 25      continue
 20   continue
      if (atop.le.0d0) return
c
c.....Obtain the stop criterium (DSTOP):
      avgf=dble(n*(n-1))*0.55d0
      d=0d0
      do 30 jj=2,n
         do 35 ii=2,jj
            s=a(ii-1,jj)/atop
            d=d+s*s
 35      continue
 30   continue
      dstop=d*epstop
c
c.....Obtain the convergence criterium (THRSH):
      thrsh=sqrt(abs(d/avgf))*atop
c
c.....Begins a cycle:
 115  iflag=0
         do 40 jcol=2,n
            do 45 irow=1,jcol-1
               aij=a(irow,jcol)
c
c..............Compare the non-diagonal elements with THRSH:
               if (abs(aij).gt.thrsh) then
                  aii=a(irow,irow)
                  ajj=a(jcol,jcol)
                  s=ajj-aii
c
c.................Rotate if the rotation is larger than the round-off
c                 error:
                  if (abs(aij).gt.abs(s)*eps0) then
c
c....................Sets the control for one more rotation:
                     iflag=1
c
c....................Compute sin (S) ans cosine (C) of the rotation:
                     if (abs(s).le.abs(aij)*eps45) then
                        s=1d0/sqrt(2d0)
                        c=s
                     else
                        t=aij/s
                        s=0.25d0/sqrt(0.25d0+t*t)
                        c=sqrt(0.5d0+s)
                        s=2d0*t*s/c
                     endif
c
c....................Compute the new A():
                     do 50 i=1,irow
                        t=a(i,irow)
                        u=a(i,jcol)
                        a(i,irow)=c*t-s*u
                        a(i,jcol)=s*t+c*u
 50                  continue
                     if (irow+2.le.jcol) then
                        do 55 i=irow+2,jcol
                           t=a(i-1,jcol)
                           u=a(irow,i-1)
                           a(i-1,jcol)=s*u+c*t
                           a(irow,i-1)=c*u-s*t
 55                     continue
                     endif
                     a(jcol,jcol)=s*aij+c*ajj
                     a(irow,irow)=c*a(irow,irow)-s*(c*aij-s*ajj)
                     do 60 j=jcol,n
                        t=a(irow,j)
                        u=a(jcol,j)
                        a(irow,j)=c*t-s*u
                        a(jcol,j)=s*t+c*u
 60                  continue
                     do 65 i=1,n
                        t=eivr(i,irow)
                        eivr(i,irow)=c*t-eivr(i,jcol)*s
                        eivr(i,jcol)=s*t+eivr(i,jcol)*c
 65                  continue
c
c....................Compute the new norm D, and compare it with DSTOP:
                     s=aij/atop
                     d=d-s*s
                     if (d.lt.dstop) then
c
c.......................Compute again DSTOP & THRSH to prevent round-
c                       off errors:
                        d=0d0
                        do 70 jj=2,n
                           do 75 ii=2,jj
                              s=a(ii-1,jj)/atop
                              d=s*s+d
 75                        continue
 70                     continue
                        dstop=d*epstop
                     endif
                     thrsh=sqrt(abs(d/avgf))*atop
                  endif
               endif
 45         continue
 40      continue
c
c.....More rotations?
      if (iflag.ne.0) goto 115
c
c.....End of the rotations. Put the eigenvalues in EIVU():
      do 80 j=1,n
         eivu(j)=a(j,j)
 80   continue
      return
      end
