c
c-----------------------------------------------------------------------
c
      subroutine jdaaa (ni, prtmat, luw)
c
c.....jdaaa - diagonalizes, symetry by symetry, the Fock matrices
c     FC() & FO().
c
c     The eigenvalue problem to be solved is:
c
c     F C = S C e
c
c     The solution is obtained in three steps:
c
c     (1) Diagonalize the overlap matrix S:
c
c          t
c         T  S T = Im         (T is unitary)
c
c     (2) Construct the V matrix:
c
c              t  -1/2     -1/2  t
c         V = T  S     = Im     T
c
c     (3) Diagonalize the product  M = V F V(t) :
c
c          t
c         U  M U = I e
c
c     e is the eigenvalue matrix, and the coefficient matrix is
c     obtained as  C = V(t) U.
c
c.....Input Parameters:
c     prtmat ..... print out the eigenvalue and eigenvector
c                           matrices when it's value is .true.
c     luw ................. logical unit of the output file.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'cont.inc'
      include           'symmetry.inc'
      include           'intg.inc'
      include           'scf.inc'
c
c.....Parameters of the routine:
      logical           prtmat
      integer           luw
c
c.....Local variables:
      dimension vv(mpgao*mpgao), ww(mpgao*mpgao), epsi(mpgao),
     & ddd(mpgao*mpgao)
      integer iorden(mpgao)
      logical abierta
c
c.....Run over the symmetries:
      iloop = 0
      m1 = 0
      m = 0
      nstep1 = 1
      do 10 n = 1, nirrep(ni)
c
c........Store ss() as a square matrix in DDD() and diagonalize it:
         nbas1 = npgao(n,ni)
         do 15 i = 1, nbas1
            nstep = nbas1*(i-1)
            do 20 j = 1, i
               ij = nstep+j
               ji = nbas1*(j-1)+i
               m = m+1
               ddd(ij) = ss(m)
               ddd(ji) = ss(m)
 20         continue
 15      continue
         call jacvao (ddd, nbas1, epsi, vv)
c
c........Computes vv(). vv() will contain Vt/SQRT(SO) (SO=eigenvalue):
         do 25 i = 1, nbas1
            nstep = nbas1*(i-1)
            term = 1d0/sqrt(abs(epsi(i)))
            do 30 j = 1, nbas1
               ji = nstep+j
               vv(ji) = vv(ji)*term
 30         continue
 25      continue
         noshic = nosh(n,ni)
         ncshic = ncsh(n,ni)
         if (noshic.eq.0 .or. ncshic.eq.0) then
            ifin = 1
         else
            ifin = 2
         endif
c
c........Compute DDD()=vv F vv(t)   (F= FC o FO):
         do 35 ish = 1, ifin
            if ((ifin.eq.1.and.noshic.gt.0).or.
     &          (ifin.ne.1.and.ish.ge.ifin)) then
               abierta = .true.
            else
               abierta = .false.
            endif
            do 40 i = 1, nbas1
               nstep = nbas1*(i-1)
               do 45 j = 1, i
                  ji = nstep+j
                  ij = nbas1*(j-1)+i
                  term = 0d0
                  do 50 k = 1, nbas1
                     do 55 l = 1, nbas1
                        if (k.lt.l) then
                           kl = ((l-1)*l)/2+k+m1
                        else
                           kl = ((k-1)*k)/2+l+m1
                        endif
                        lj = nbas1*(j-1)+l
                        ki = nstep+k
                        if (abierta) then
c
c..........................Open shell. Find just one orbital:
                           iloop = 1
                           a = fo(kl)
                        else
c
c..........................Closed shell. Find NCSHIC orbitals:
                           iloop = ncshic
                           a = fc(kl)
                        endif
                        term = term+vv(ki)*vv(lj)*a
 55                  continue
 50               continue
                  ddd(ij) = term
                  ddd(ji) = term
 45            continue
 40         continue
c
c...........Diagonalize DDD() & obtain the eigenvalues:
            call jacvao (ddd, nbas1, epsi, ww)
            do 60 i = 1, nbas1
               do 65 j = 1, nbas1
                  term = 0d0
                  nstep = nbas1*(j-1)
                  ji = nstep+i
                  do 70 k = 1, nbas1
                     ik = nbas1*(k-1)+i
                     kj = nstep+k
                     term = term+vv(ik)*ww(kj)
 70               continue
                  ddd(ji) = term
 65            continue
 60         continue
c
c...........Print out the matrices:
            if (prtmat) then
               if (abierta) then
                  write (luw,200) n,'Open'
               else
                  write (luw,200) n,'Closed'
               endif
               write (luw,205) (epsi(indao),indao=1,nbas1)
               do 75 i = 1, nbas1
                  indao = nbas1*(i-1)
                  write (luw,210) i,(ddd(indao+j),j=1,nbas1)
 75            continue
            endif
c
c...........Sort eigenvalues and eigenvectors in this symmetry:
            call jacsort (nbas1, epsi, iorden, nbas1, mpgao)
c
c...........Select the appropriate eigenvalues & eigenvectors:
            do 80 i = 1, iloop
               if (abierta) then
c
c.................Skip the closed shell orbitals:
                  min = iorden(ncshic+noshic)
               else
                  min = iorden(i)
               endif
               georb(nstep1,ni) = epsi(min)
               do 85 j = 1, nbas1
                  jj = nbas1*(min-1)+j
                  gc(j,nstep1,ni) = ddd(jj)
 85            continue
               nstep1 = nstep1+1
 80         continue
 35      continue
c
c........How many charge densities have been obtained?
         m1 = m1+(nbas1*(nbas1+1))/2
c
c.....(End of the symmetry loop).
 10   continue
c
c.....Return control to SCFAT.
      return
c
c.....Formats:
 200  format (//1x,'Diagonalization report. Symmetry:',i3,
     &        5x,'Shell:',a)
 205  format (1x,'Eigenvalues:'/(1p8e15.7))
 210  format (1x,'Eigenvector #',i3/(1p8e15.7))
      end
