c
c-----------------------------------------------------------------------
c
      subroutine localenr (ni, luw, nprtloc)
c
c.....localenr - computes the local energy for an active ion that
c     forms part of a cluster within the crystal.
c
c     The local energy is defined to be additive with respect to the
c     cluster and effective with respect to the crystal (see diagram).
c
c     ... Z  Y  X  (  C  B  [  A  ]  B  C  )  X  Y  Z  ...
c                             ^^^ active ion
c                  ---------CLUSTER---------
c     ... ________________HOST CRYSTAL_______________ ...
c
c      A      A                 AS          AQ
c     E    = E    + (1/2) SUM  E    + SUM  E          S in cluster
c      loc    net          S    int    Q    int       Q in host crystal
c
      include           'implicit.inc'
      include           'global.inc'
      include           'aos.inc'
      include           'cont.inc'
      include           'consts.inc'
      include           'iondens.inc'
      include           'environ.inc'
      include           'impurity.inc'
      include           'intg.inc'
      include           'lint.inc'
      include           'neqatm.inc'
      include           'scf.inc'
      include           'stos.inc'
      include           'vef.inc'
c
      integer           ni, luw, nprtloc
c.....LOCAL VARIABLES:
      real*8            ensrnol, eesrnol, eexnol, eplnol
      real*8            emploc, ensrloc, eesrloc, eexloc, eplloc,
     &                  enlloc
c
c.....initialize local energy
c
      eloc(ni) = eeff(ni)
c     eeff(ni) = enet + eint(ni)
c
c.....determine the interaction energy with the cluster neighbor shells:
c
c.....(A) Non-local contributions:
c
      emploc  = zero
      ensrnol = zero
      eesrnol = zero
      eexnol  = zero
      eplnol  = zero
      do lpq = 1, ndens(ni)
         emploc  = emploc  + vmploc(lpq)  * dt(lpq)
         ensrnol = ensrnol + vnsrnol(lpq) * dt(lpq)
         eesrnol = eesrnol + vesrnol(lpq) * dt(lpq)
         eexnol  = eexnol  + vexnol(lpq)  * dt(lpq)
         eplnol  = eplnol  + planol(lpq)  * dt(lpq)
      enddo
c
c.....(B) Obtain local electronic contributions
c
      ensrloc = ensr - ensrnol 
      eesrloc = eesr - eesrnol
      eexloc  = eex - eexnol
      eplloc  = epl - eplnol
      eintloc = emploc + ensrloc + eesrloc + eexloc + eplloc 
c
c.....(C) Nucleus-cluster interaction energy:
c
      enlloc = enllr + enlsr - enlnol
c
c.....Glue together all contributions:
c
      eloc(ni) = eeff(ni) - half * (enlloc + eintloc)
c
c.....Print out:
c
      if (nprtloc .lt. 0) return
      write (luw,600) ni, namneq(ni), eloc(ni)
      write (luw,610) vmploc(ni), ensrloc, enlloc, eesrloc, eexloc,
     &                eplloc
      return
c
 600  format (/
     & 1x, 'Local energy of an atom in a cluster in a crystal:'/
     & 1x, 'Atom number & name         : ', i4, 2x, a/
     & 1x, 'Local energy               : ', 1pe21.14)
 610  format (
     & 1x, 'Cluster Madelung potential      : ', 1pe21.14/
     & 1x, 'Cluster nuclear short range int.: ', 1pe21.14/
     & 1x, 'Nucleus-cluster interaction: ', 1pe21.14/
     & 1x, 'Cluster coulomb short range int.: ', 1pe21.14/
     & 1x, 'Cluster exchange interaction    : ', 1pe21.14/
     & 1x, 'Cluster projection interaction  : ', 1pe21.14/ )
      end
