c
c-----------------------------------------------------------------------
c
      subroutine nuclear (nai, rs, loc, vnsr, ivstar)
c
c.....coulsr: computes the nuclear attraction integrals
c
c.....input parameters:
c
c     nai ..... active center label
c     rs ...... distance between the two centers
c     loc ..... .true. <-> Vloc,sr / .false. <-> Vnsr
c     ivstar ... 
c
c.....output parameters:
c
c     vnsr(lpq) ...... nuclear attraction of the frozen center over
c                      the lpq density of the active center
c
      include           'implicit.inc'
      include           'global.inc'
      include           'consts.inc'
      include           'cont.inc'
      include           'iondens.inc'
      include           'rsh.inc'
      include           'symmetry.inc'
      include           'star.inc'
c
      integer           nai, ivstar
      real*8            rs
      real*8            vnsr(mdens)
      logical           loc
c
      real*8            rexp(2*namax)
      real*8            hmu(0:2*namax+maxmp)
      real*8            zpqr, zpqinv, zpqexp
      real*8            xfact, tmp, vnsrtmp
c
      integer           lpq, lp, lq, lp0, lq0, mp, mq, n1
      integer           lmp, lmq, l1, l1menos, l1mas, k
      integer           nh
c
c.....Calculation of powers of the distance
c
      rexp(1) = rs
      do i = 2, 2*namax
        rexp(i) = rexp(i-1) * rs ! rexp(i) = rs**i
      enddo
c
      do lpq = 1, ndens(nai)
        lp = denlp(lpq,nai) 
        lq = denlq(lpq,nai)
        lp0 = il0(lp)
        lq0 = il0(lq)
        l1menos = abs(lp - lq)
        l1mas = lp + lq 
        n1 = npq(lpq,nai)
        xfact = four*pi*xpq(lpq,nai)*rexp(n1)
        nh = n1 + l1mas 
        zpqr = zpq(lpq,nai) * rs 
c
c.....STAGE 1: mu integrals for the active ion densities
c
        zpqinv = one / zpqr
        zpqexp = exp(-zpqr) * zpqinv
        hmu(0) = zpqexp
        do n = 1, nh
          hmu(n) = n*hmu(n-1)*zpqinv + zpqexp
        enddo
           
c
c.....STAGE 2: Compute nuclear attraction integrals
c
        vnsrtmp = zero 
        do l1 = l1menos, l1mas, 2 
         l10 = il0(l1) 
         tmp = zero
         do m1 = -l1, l1
          lm1 = l10 + m1 
          if (loc) then ! Vlocsr calculation 
           if (ivstar.eq.3) then 
             tmp = tmp + udstar(lpq,lm1)*slm(lm1)
           else
             isig = 1
             if (ivstar.eq.2 .and. mod(l1,2).eq.1) isig = -1
             tmp = tmp +  isig*udpq(lpq,lm1,nai)*slm(lm1)
           endif
          else ! Vnsr calculation
           tmp = tmp + udpq(lpq,lm1,nai)*slm(lm1)
          endif
         enddo
         if (loc) then ! Vlocsr calculation
          isig = 1
          if (mod(l1,2).eq.1) isig = -1
          vnsrtmp = vnsrtmp + 
     &              isig*tmp*(hmu(n1-l1-1)-hmu(n1+l1))/(l1+l1+1)
         else ! Vnsr calculation
          vnsrtmp = vnsrtmp + tmp*(hmu(n1-l1-1)-hmu(n1+l1))/(l1+l1+1)
         endif
        enddo
        vnsr(lpq) =  vnsrtmp * xfact
c
      enddo ! lpq
c
      return
      end
