c
c-----------------------------------------------------------------------
c
      subroutine overlap (nai, nfi, rs, o, ivstar)
c
c.....overlap - computes the overlap matrix supervector O(lpq,lab).
c
c.....History:
c     Original aipi routine, see AMP & EF, Phys. Rev. A, 43, 3384 (1991)
c     pi7r7: precompute Alaf integrals with N^2 algorithm instead of N^4
c     pi7r12: Hierse and Oppeneer algorithm, see MAB, PhD Thesis.
c     gpi: average is now over irrep subspecies, changes by CJFS & MAB
c
c.....input parameters:
c
c     nai ...... active center label
c     nfi ...... frozen center label
c     rs ....... distance between the two centers
c     ivstar ...
c
c.....output parameters:
c
c     o(lpq,lab) ..... integral between lpq and lab densities of the
c                      active and frozen centers, respectively
c
      include           'implicit.inc'
      include           'global.inc'
      include           'coed.inc'
      include           'consts.inc'
      include           'cont.inc'
      include           'stos.inc'
      include           'iondens.inc'
      include           'rsh.inc'
      include           'tfunc.inc'
      include           'symmetry.inc'
      include           'star.inc'
c
c.....toosmall is a tested value for the difference between two orbital
c     exponents to consider them equal; toobig is a number such that the
c     hyperbolic functions will overflow, so exp(-alpha) is multiplied
c     in nu integrals, and divided in mu ones; conv is the relative
c     convergence required in nu integrals when they are evaluated as
c     infinite series
c
      real*8            toosmall, toobig, conv
      parameter ( toosmall = 1d-8 )
      parameter ( toobig = 500d0 )
      parameter ( conv = 1d-16 )
c
      integer           mdenmix, nsmmax
      parameter ( mdenmix = msto*msto )
      parameter ( nsmmax  = (msa*(msa+1))/2 )
c
      real*8            rs, o(mdens,mdens)
      real*8            rexp(2*namax+1)
*      real*8            auxmu(0:2*namax+maxmp)
      real*8            alfa(mdenmix), beta(mdenmix)
      real*8            hnu(0:2*namax+maxmp,mdenmix)
      real*8            hmu(0:2*namax+maxmp,mdenmix)
      real*8            xst(0:maxmp,mdenmix), alaf(msa,mdenmix)
      real*8            za, zb, zp, zq
      real*8            aa, bb, sh, ch
      real*8            alfainv, aexp, binv, term, u1, u2
      real*8            alafaux, bcoef, oaux, xnor1, xnorm
c
      integer           nai, nfi, ivstar
      integer           nh(mdenmix), nsm(mdenmix), nx(mdenmix)
      integer           idm(msto,msto), ide(mdenmix)
      integer           labmas(nsmmax), labmenos(nsmmax)
      integer           lab, lpq, i
      integer           la, lb, lp, lq
      integer           na, nb, np, nq
      integer           lma, lmb, lmp, lmq
      integer           la0, lb0, lp0, lq0
      integer           nf1, nf2, na1, na2
      integer           labms, labmn
      integer           nsmix, idx, ndenmix
      integer           nn, ilim, it, is, istart, ist
      integer           inu, inumax, ie, istm, ll
      integer           labmn1, labms1, labmn2, labms2
      integer           idx1, idx2, ll1, ll2, l1, l2, it1, it2
c
      logical           odd
c
c.....STAGE 1: several initializations
c

c.....initialization of the distance powers table
      rexp(1) = rs
      do i = 2, 2*namax+1
        rexp(i) = rexp(i-1)*rs
      enddo
c.....initialization of several supervectors
      idx = 0
      nsmix = 0
      do l1 = 1, msa
        do l2 = l1, msa
          nsmix = nsmix+1
          labms = l1+l2-2
          labmenos(nsmix) = l2-l1
          labmas(nsmix) = labms
          if (l1.le.nsym(nai) .and. l2.le.nsym(nfi)) then
            lp = l1-1
            la = l2-1
            do isa = nasto(l1,nai)+1, nasto(l1,nai)+nsto(l1,nai)
              np = n(isa,nai)-lp
              zp = z(isa,nai)
c$dir select(,*,*)
              do isf = nasto(l2,nfi)+1, nasto(l2,nfi)+nsto(l2,nfi)
                na = n(isf,nfi)-la
                za = z(isf,nfi)
                idx = idx+1
                idm(isa,isf) = idx
                ide(idx) = indcoefe(lp,np,la,na)
                nsm(idx) = nsmix
                nx(idx) = np + na
                nh(idx) = np+na+labms+labms
                alfa(idx) = (zp+za)*rs*half
                beta(idx) = (zp-za)*rs*half
              enddo
            enddo
          endif
          if (l1.ne.l2 .and. l1.le.nsym(nfi) .and. l2.le.nsym(nai)) then
            lp = l2-1
            la = l1-1
            do isf = nasto(l1,nfi)+1, nasto(l1,nfi)+nsto(l1,nfi)
              na = n(isf,nfi)-la
              za = z(isf,nfi)
c$dir select(,*,*)
              do isa = nasto(l2,nai)+1, nasto(l2,nai)+nsto(l2,nai)
                np = n(isa,nai)-lp
                zp = z(isa,nai)
                idx = idx+1
                idm(isa,isf) = idx
                ide(idx) = indcoefe(lp,np,la,na)
                nsm(idx) = nsmix
                nx(idx) = np + na
                nh(idx) = np+na+labms+labms
                alfa(idx) = (zp+za)*rs*half
                beta(idx) = (zp-za)*rs*half
              enddo
            enddo
          endif
        enddo
      enddo
      ndenmix = idx
c
c.....STAGE 2: calculation of hnu, hmu integrals for all mixed densities
c     (sto(isa,na)*sto(isf,nf) for all isa, isf of the active and
c     frozen center, respectively).
c
      do idx = 1, ndenmix
        nn = nh(idx)
        bb = beta(idx)
        aa = alfa(idx)
        alfainv = one / aa
        aexp = exp(-aa) * alfainv
c.....test special case beta < toosmall for nu integrals
        if (abs(bb).lt.toosmall) then
          do is = 0, nn-1, 2
            hnu(is,idx) = one / (is+1)
            hnu(is+1,idx) = zero
          enddo
          if (mod(nn,2).eq.0) hnu(nn,idx) = one / (nn+1)
c.....test special case beta > toobig
        else
          if (abs(bb).gt.toobig) then
            aexp = alfainv
            sh = (exp(bb-aa) - exp(-bb-aa)) * half
            ch = (exp(bb-aa) + exp(-bb-aa)) * half
          else
            sh = (exp(bb) - exp(-bb)) * half
            ch = (exp(bb) + exp(-bb)) * half
          endif
c.....classify integrals for up or down recursion
          ilim = min(int(abs(bb))-1,nn)
          odd = mod(ilim+2,2).eq.1
c.....run up recursion from 0 to ilim
          if (ilim.ge.0) then
            binv = one / bb
            hnu(0,idx) = sh * binv
            do is = 1, ilim-1, 2
              hnu(is,idx) = (ch - is*hnu(is-1,idx)) * binv
              hnu(is+1,idx) = (sh - (is+1)*hnu(is,idx)) * binv
            enddo
            if (odd)
     &        hnu(ilim,idx) = (ch - ilim*hnu(ilim-1,idx)) * binv
          endif
c.....run down recursion from nn to ilim+1
          if (ilim.ne.nn) then
c.....sum the top element until convergence
            it = nn + 1
            term = one / it
            u1 = term
            u2 = zero
            do while (abs(term).gt.conv*abs(u1))
              it = it + 1
              term = term * bb / it
              u2 = u2 + term
              it = it + 1
              term = term * bb / it
              u1 = u1 + term
            enddo
c.....now follow the down recursion
            if (mod(nn,2).eq.1) then
              hnu(nn,idx) = u1*sh - u2*ch
              istart = nn - 1
            else
              hnu(nn,idx) = u1*ch - u2*sh
              if (nn.ne.ilim+1)
     &          hnu(nn-1,idx) = (sh - bb*hnu(nn,idx)) / nn
              istart = nn - 2
            endif
            do is = istart, ilim+2, -2
              hnu(is,idx) = (ch - bb*hnu(is+1,idx)) / (is+1)
              hnu(is-1,idx) = (sh - bb*hnu(is,idx)) / is
            enddo
            if (odd .and. nn.ne.ilim+1) hnu(ilim+1,idx) =
     &        (ch - bb*hnu(ilim+2,idx)) / (ilim+2)
          endif
        endif
c.......calculation of mu integrals
        hmu(0,idx) = aexp
        do is = 1, nn
          hmu(is,idx) = is*hmu(is-1,idx)*alfainv + aexp
        enddo
*Algoritmo alternativo aipi:
*        auxmu(1) = aexp / aa
*        do it = 2, nn
*          auxmu(it) = auxmu(it-1) / aa
*        enddo
*        do is = 0, nn
*          hmu(is,idx) = aexp
*        enddo
*        do is = 1, nn
*          tmp = is
*          do it = 1, is
*            hmu(is,idx) = hmu(is,idx) + auxmu(it)*tmp
*            tmp = tmp * (is-it)
*          enddo
*        enddo
      enddo
c
c.....STAGE 3: calculation of the Xst integrals
c
      do idx = 1, ndenmix
        ie = ide(idx)
        labms = labmas(nsm(idx))
        do ist = 0, labms
          inumax = nx(idx) + ist + ist
          xst(ist,idx) = zero
**********The xst integral loop can be started at this non-zero init
**********only when the maximum ist value is la+lb as it is here; in the
**********Coulomb integ. we cannot do this if the loop reaches la+lb+1
c$dir select(,*,*)
          do inu = max0(ist+ist-labms,0), inumax
            xst(ist,idx) = xst(ist,idx) + coefe(inu,ist,ie) *
     &                         hnu(inu,idx) * hmu(inumax-inu,idx)
          enddo
        enddo
      enddo
c
c.....STAGE 4: calculation of the Alaf radial integrals
c
c     Caution! We include R^(labms-1) on each Alaf integral to achieve
c     the correct final scaling but with half the maximum powers; the
c     exponent here is nh-labms+1 = np+nq+1, where (labms-1) come from
c     the final O integral factor, (-2s-2t) from the A integral, and
c     (np+nq+2s+2t-labms+2) from the Xst integral
c
      do idx = 1, ndenmix
        ll = 0
        labms = labmas(nsm(idx))
        do l = labmenos(nsm(idx)), labms, 2
          ll = ll + 1
          alafaux = zero
          istm = (labms+l)/2
          do ist = 0, istm
            alafaux = alafaux + xst(ist,idx) * coefd(istm-ist,l)
          enddo
          alaf(ll,idx) = alafaux * rexp(nh(idx)-labms+1)
        enddo
      enddo
c
c.....STAGE 5: main lpq,lab loop computing o(lpq,lab); first, loop over
c              active center densities
c
c.....loop over densities of acitve center
      do lpq = 1, ndens(nai)
        lp = denlp(lpq,nai)
        lq = denlq(lpq,nai)
        lp0 = il0(lp)
        lq0 = il0(lq)
        na1 = denp(lpq,nai)+nasto(lp+1,nai)
        na2 = denq(lpq,nai)+nasto(lq+1,nai)
        xnor1 = four*pisquare*xpq(lpq,nai)
        if (mod(lp+lq,2).eq.1) xnor1 = - xnor1
c.....loop over densities of frozen center
        do lab = 1, ndens(nfi)
          la = denlp(lab,nfi)
          lb = denlq(lab,nfi)
          la0 = il0(la)
          lb0 = il0(lb)
          nf1 = denp(lab,nfi)+nasto(la+1,nfi)
          nf2 = denq(lab,nfi)+nasto(lb+1,nfi)
          xnorm = xnor1*xpq(lab,nfi)
c.....limits for l1 and l2
          idx1 = idm(na1,nf1)
          labmn1 = labmenos(nsm(idx1))
          labms1 = labmas(nsm(idx1))
          idx2 = idm(na2,nf2)
          labmn2 = labmenos(nsm(idx2))
          labms2 = labmas(nsm(idx2))
c.....l1 and l2 loops
*     although a B(mq,mb) matrix precursor could be computed, each
*     precursor will need the four m loops, so no real gain there
          oaux = zero
          ll1 = 0
          do l1 = labmn1, labms1, 2
            ll1 = ll1 + 1
            ll2 = 0
            do l2 = labmn2, labms2, 2
              ll2 = ll2 + 1
c.....calculation of B^l1l2(lpq,lab)
              bcoef = zero
              do mp = -lp, lp
               lmp = lp0 + mp
               do mq = -lq, lq
                lmq= lq0 + mq
                u1 = coefUm(lpq,mp,mq,nai)
                do ma = -la, la
                 lma = la0 + ma
                 it1 = itfunc(lmp,lma,l1)
                 do mb = -lb, lb
                  lmb = lb0 + mb
                  if (ivstar.eq.3) then
                    u2 = Umstar(lab,ma,mb)
                  else
                    isig = 1
                    if (ivstar.eq.2 .and. mod(la+lb,2).eq.1) isig = -1 
                    u2 = isig*coefUm(lab,ma,mb,nfi)
                  endif
                  it2 = itfunc(lmq,lmb,l2)
                  bcoef = bcoef + u1*u2*tfunc(it1)*tfunc(it2)
                 enddo
                enddo
               enddo
              enddo
c.....add to o(lpq,lab) alaf integrals and B coefficients
              oaux = oaux + bcoef*alaf(ll1,idx1)*alaf(ll2,idx2)
            enddo
          enddo
c
c.....when a.ne.b, the exchanged term must be included; however, if
c     p.eq.q this term is equal to the one just computed
c
          if (nf1.ne.nf2 .and. na1.eq.na2) then
            oaux = oaux + oaux
c
c.....if a.ne.b, we need another exchanged term
c
          else if (nf1.ne.nf2) then
            idx1 = idm(na1,nf2)
            labmn1 = labmenos(nsm(idx1))
            labms1 = labmas(nsm(idx1))
            idx2 = idm(na2,nf1)
            labmn2 = labmenos(nsm(idx2))
            labms2 = labmas(nsm(idx2))
            ll1 = 0
            do l1 = labmn1, labms1, 2
              ll1 = ll1 + 1
              ll2 = 0
              do l2 = labmn2, labms2, 2
                ll2 = ll2 + 1
c
c.....calculation of B^l1l2(lpq,lba)
c
                bcoef = zero
                do mp = -lp, lp
                 lmp = lp0 + mp
                 do mq = -lq, lq
                  lmq= lq0 + mq
                  u1 = coefUm(lpq,mp,mq,nai)
                  do ma = -la, la
                   lma = la0 + ma
                   it2 = itfunc(lmq,lma,l2)
                   do mb = -lb, lb
                    lmb = lb0 + mb
                    it1 = itfunc(lmp,lmb,l1)
                    u2 = coefUm(lab,ma,mb,nfi)
                    bcoef = bcoef + u1*u2*tfunc(it1)*tfunc(it2)
                   enddo
                  enddo
                 enddo
                enddo
c.....add to o(lpq,lab) alaf integrals and B coefficients
                oaux = oaux + bcoef*alaf(ll1,idx1)*alaf(ll2,idx2)
              enddo
            enddo
          endif
c.....compute o(lpq,lab) including the normalization constants
          o(lpq,lab)=oaux*xnorm
        enddo
      enddo
c
      return
      end
