c
c-----------------------------------------------------------------------
c
      subroutine preewald ()
c
c.....preewald - computes some variables nedded in the sewald and 
c                tewald functions.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'error.inc'
      include           'neqatm.inc'
      include           'celatm.inc'
      include           'red.inc' 
      include           'ewald.inc' 
      include           'rsh.inc'
      include           'consts.inc'
c
      real*8            omegar, num, TINY
      parameter         (TINY = 1d-10)
c
c.....computing lattice reciprocal parameters and eta:
c
      arec   = bred * cred * salf / omega 
      brec   = cred * ared * sbet / omega
      crec   = ared * bred * sgam / omega
      calfr  = (cbet * cgam - calf) / (sbet * sgam)
      cbetr  = (cgam * calf - cbet) / (sgam * salf)
      cgamr  = (calf * cbet - cgam) / (salf * sbet)
      rootr  = sqrt(one-calfr*calfr-cbetr*cbetr-cgamr*cgamr+
     &         two*calfr*cbetr*cgamr)
      omegar = one / omega
      sgamr  = cred * omegar / (arec * brec)
      num = sgamr * sgamr + cgamr * cgamr
      if (abs(num-1d0).gt.TINY) call error ('preewald',
     & 'sgamr^2 + cgamr^2 .ne. 1', faterr) 
c     eta = parameter in Ewald summation
      armax = max (arec, brec, crec)
      armin = min (arec, brec, crec)
      eta = one / sqrt (armax*armin*pi)
      pieta2 = pi / (armax*armin) 
      piomega = pi * omega
c
c.....computing the metric tensor of the reciprocal lattice:
c
      gr(1) = arec * arec
      gr(2) = arec * brec * cgamr
      gr(3) = brec * brec
      gr(4) = arec * crec * cbetr
      gr(5) = brec * crec * calfr
      gr(6) = crec * crec
c
c.....initialization of lattice vectors up to mlayer shells of cells
c     kvec = h veca* + k vecb* + c vecc*, where (h,k,l) are integers
c
c     shell ish=0
      do i = 1, 3
         kvec(i,0) = 0
      enddo
      iavec(0) = 0 ! Number of vectors accumulated for the shell 0
c     shells ish=1,2,...,15
      ivec = 0 ! counter of reciprocal lattice vectors
      do ish = 1, mlayer 
         do ixx = 0, ish ! ixx=h
            do iyy = -ish, ish ! iyy=k
c
c.....we include only those z's that make up the shell
c
               if (abs(iyy).eq.ish .or. ixx.eq.ish) then
                  istep = 1
               else
                  istep = ish + ish
               endif
               do izz = -ish, ish, istep ! izz=l
c
c.....and among them, only half, since there's always a symmetrical
c     lattice vector -K
c
                  if ( (ixx.ne.0 .or. iyy.ge.0) .and.
     &                 (ixx.ne.0 .or. iyy.ne.0 .or. izz.gt.0) ) then
                     ivec = ivec + 1
                     kvec(1,ivec) = ixx
                     kvec(2,ivec) = iyy
                     kvec(3,ivec) = izz
                  endif
               enddo ! l
            enddo ! k
         enddo ! h
         iavec(ish) = ivec ! it accumulates the number of reciprocal
                           ! vectors considered up to the shell ish
      enddo ! ish
c
c.....end of the routine
c
      return
      end
