c
c-----------------------------------------------------------------------
c
      real*8 function rcsupk (np, zp, xnp, nq, zq, xnq,
     &                        nr, zr, xnr, ns, zs, xns, k,
     &                        alleq, con1, con2)
c
c.....rcsupk - computes the correlated integral R sup K over STOs.
c
c     The integral is defined as:
c      k                            k   k+1
c     R (p,q;r,s) = <X (1) X (2)! (r  /r   ) !X (1) X (2)>
c      c              p     q       <    >     r     s
c
c     where Xi is an STO and the numbers in parenthesis are electrons.
c     The integral contains a coulomb hole of radius delta around the
c     electron 1. The hole radius varies according to the parameters
c     of the STO's.
c
c.....Input parameters:
c     Ni ........ principal quantum number of the STO Xi.
c     Zi ........ orbital exponent of the STO Xi.
c     XNi ....... normalization constant of the STO Xi.
c                 (where i may be: a, b, c or d).
c     K ......... k exponent in the integral.
c     alleq ..... true if all STOs have the same angular momentum.
c     con1 ...... con1 CHF parameter.
c     con2 ...... con2 CHF parameter.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'fact.inc'
      include           'comb.inc'
c
      logical           alleq
c
      idx(i,j) = ((i-1)*(i+2))/2+j+1
c
      spr=xnp*xnr*fact(np+nr)/(zp+zr)**(np+nr+1)
      sqs=xnq*xns*fact(nq+ns)/(zq+zs)**(nq+ns+1)
      spq=xnp*xnq*fact(np+nq)/(zp+zq)**(np+nq+1)
      sps=xnp*xns*fact(np+ns)/(zp+zs)**(np+ns+1)
      sqr=xnq*xnr*fact(nq+nr)/(zq+zr)**(nq+nr+1)
      srs=xnr*xns*fact(nr+ns)/(zr+zs)**(nr+ns+1)
      if (alleq) then
         delta = (con1/16.0d0) * ((np/zp/zp+nr/zr/zr)*spr +
     &           (nq/zq/zq+ns/zs/zs)*sqs)* (spq+sps+sqr+srs)
      else
         delta = (con1/24.0d0/con2) * ((np/zp/zp+nr/zr/zr)*spr*spr +
     &           (nq/zq/zq+ns/zs/zs)*sqs*sqs) *
     &           (spq*spq + sps*sps + sqr*sqr + srs*srs)
      endif
c
      npr=np+nr
      nqs=nq+ns
      zpr=zp+zr
      zqs=zq+zs
      zpqrs=zpr+zqs
      xpqrs=xnp*xnq*xnr*xns
      t1=xpqrs*(fact(nqs+k)/zqs**(nqs+k+1))*
     &   (fact(npr-k-1)/zpr**(npr-k))
      t2=0.0d0
      do 10 m=0,nqs+k
         t2a=0.0d0
         do 15 i=0,m
            if (m.eq.i) then
               signo=1.0d0
            else if (mod(m-i,2).eq.1) then
               signo=-(delta**(m-i))
            else
               signo=delta**(m-i)
            endif
            signo=signo*comb(idx(m,i))
            t2a=t2a+signo*fact(npr+i-k-1)/zpqrs**(npr+i-k)
 15      continue
         t2=t2+t2a*zqs**m/fact(m)
 10   continue
      t2=-(t2*xpqrs*fact(nqs+k)/zqs**(nqs+k+1)*exp(zqs*delta))
      t3=0.0d0
      do 20 m=0,nqs-k-1
         t3a=0.0d0
         do 25 i=0,m
            if (i.eq.m) then
               signo=comb(idx(m,i))
            else
               signo=comb(idx(m,i))*delta**(m-i)
            endif
            t3a=t3a+signo*fact(npr+i+k)/zpqrs**(npr+i+k+1)
 25      continue
         t3=t3+t3a*zqs**m/fact(m)
 20   continue
      t3=t3*xpqrs*fact(nqs-k-1)/zqs**(nqs-k)*exp(-(zqs*delta))
      rcsupk=t1+t2+t3
      return
      end
