      real*8 function rgaunt (l1,m1,l2,m2,l3,m3)
c
c.....rgaunt - compute Gaunt coefficient for real spherical harmonics
c
c     The routine evaluates the integral between three real spherical
c     harmonics:
c      l1 m1
c     d            = < S      S      S     >
c      l2 m2 l3 m3      l1 m1  l2 m2  l3 m3
c
c     This integral is symmetric with respect to the interchange of
c     functions, i.e. (li,mi) pairs. It is sometimes called Gaunt
c     integral or Gaunt coefficient. This routine calls cgaunt to
c     calculate the \theta part, expecting to receive a Gaunt
c     coefficient between complex spherical harmonics, defined as:
c
c      l1 m1
c     c            = < Y     | Y     | Y     >       (m3=m1-m2)
c      l2 m2 l3 m3      l1 m1   l2 m2   l3 m3
c
c     This version assumes that it will be called with (m1,m2,m3)
c     values with non-zero I_{m1,m2,m3} (\phi integral); see
c     dsupk.inc and "Generacion de los indices m3num y m3val"
c     (GPI - Dise~no codigo)
c
      include           'implicit.inc'
c
      integer           l1, m1, l2, m2, l3, m3
      integer           m1a, m2a, m3a
      real*8            dvalue, cgaunt
c
c.....Initialize variables
c
      m1a = abs(m1)
      m2a = abs(m2)
      m3a = abs(m3)
c
c.....All m's are zero (case d)
c
      if (m1a+m2a+m3a.eq.0) then
        rgaunt = cgaunt(l3,l1,0,l2,0)
        return
c
c.....Cases e and h
c
      else if (m1*m2*m3.lt.0) then
        rgaunt = 0d0
        return
c
c.....Case f
c
      else if (m1+m2+m3.eq.0.and.m1*m2*m3.eq.0) then
        rgaunt = 0d0
        return
c
c.....|m1|=|m2|+|m3|
c
      else if (m1a.eq.m2a+m3a) then
        dvalue = cgaunt(l3,l1,m1a,l2,m2a)
c
c.....|m2|=|m1|+|m3|
c
      else if (m2a.eq.m1a+m3a) then
        dvalue = cgaunt(l3,l2,m2a,l1,m1a)
c
c.....|m3|=|m1|+|m2| 
c
      else if (m3a.eq.m1a+m2a) then
        dvalue = cgaunt(l1,l3,m3a,l2,m2a)
c
c.....Cases c, g and the others
      else
        rgaunt = 0d0
        return
      endif
c
c.....All positive or two negative and one positive
c
      if (m1*m2*m3.gt.0) then
        dvalue = dvalue/sqrt(2d0)
c
c.....If the positive value equals the negatives' sum
c
        if (m1+m2+m3.eq.0) dvalue = -dvalue
      endif
c
c.....Everything done
c
      rgaunt = dvalue
      return
      end function
