c
c.....rotmat.inc - Wigner coefficients for real spherical harmonics
c     data structures
c
c     Wigner coefficients or rotation matrices R_{m1,m2}^l will be
c     stored on a correlative vector rotr(i); the matrices are defined
c     so that R_{m1,m2}^l = < S_{lm1} | \hat{R} | S_{lm2} >, acting on
c     row vectors of functions:
c       \hat{R} S_{lm2} (\vec{r}) = S_{lm2} (\hat{R}^{-1} \vec{r})
c                                 = \sum_{m1} S_{lm1} R_{m1,m2}^l
c     each R^l matrix will be stored in regular Fortran array ordering
c       (-l,-l; 1-l,-l; ...; l,-l; -l,1-l; ...; l,l)
c     and the vector irot0(l) will point to the first element of matrix
c     R^l in rotr(), so that rotr(irot0(l)) can be passed as a pointer
c     to the R^l rotation matrix of (2l+1)x(2l+1) elements. In this way,
c     the elements can be recovered as:
c
c     i = 1
c     do l = 0, maxmp
c       irot0(l) = i
c       do m2 = -l, l
c         do m1 = -l, l
c           R_{m1,m2}^l = rotr(i) = rotr(irot0(l)+(l+m2)*(2*l+1)+m1+l)
c           i = i + 1
c         enddo
c       enddo
c     enddo
c
c     indmat() .......  pointer that indicates the vector of Wigner
c                       coefficients
c     rotr() .......... vector of Wigner coefficients/rotation matrices
c     irot0() ......... pointer to first element of R^l in rotr(), which
c                       is 1+l*(2l-1)*(2l+1)/3
c     maxrotr ......... maximum number of elements in rotr, equal to
c                       irot0(maxmp+1)-1 or (maxmp+1)(2maxmp+1)(2maxmp+3)/3
c                       (maxmp/maxrotr)=(6/455),(12/2925),(20/12341),(24/20825)..
c
      integer           maxrotr
      parameter         ( maxrotr = 455 )
      integer           indmat(23), irot0(0:maxmp)
      real*8            rotr(maxrotr,23)
      common /rotri/    indmat, irot0
      common /rotrr/    rotr
