      subroutine rsh (x, y, z, r, slmm, llmax)
c
c.....rsh - computes real spherical harmonics up to l=llmax for a
c     relative position (x,y,z), distance r; results are stored as
c     a supervector in slmm, where S_{l,m} (\hat{r}) is stored in the
c     slmm(l*(l+1)+m) element
c
c     The routine WILL NOT test if r is zero; if it is, it will return
c     the same value given for \theta=0. It does not test whether llmax
c     is valid (0.ge.llmax .and. llmax.le.maxl), either.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'consts.inc'
      include           'rsh.inc'
c
      integer           llmax
      real*8            x, y, z, r
      real*8            slmm(0:llmax*(llmax+2))
c
      parameter         (eps = 1d-9)
c
c.....S_00 is always 1/sqrt(4*pi)
c
      slmm(0) = one / sqrt(four*pi)
c
c.....compute xy projection of r
c
      rxy = x*x + y*y
c
c.....test if (x,y,z) points in the z direction
c
      if (rxy.lt.eps) then
c.......positive z => \theta=0
        if (z.ge.zero) then
          do l = 1, llmax
            slmm(il0(l)) = slmm(0) * sqrt(dble(l+l+1))
          enddo
c.......negative z => \theta=\pi
        else
          do l = 1, llmax-1, 2
            slmm(il0(l)) = - slmm(0) * sqrt(dble(l+l+1))
            slmm(il0(l+1)) = slmm(0) * sqrt(dble(l+l+3))
          enddo
          if (mod(llmax,2).eq.1) then
            slmm(il0(llmax)) = - slmm(0) * sqrt(dble(llmax+llmax+1))
          endif
        endif
c.......RSH with m<>0 are 0
        i = -1
        do l = 0, llmax-1
          i = i + 1
          do m = 0, l+l
            i = i + 1
            slmm(i) = zero
          enddo
        enddo
        i = i + 1
        do m = 1, llmax
          i = i + 1
          slmm(i) = zero
        enddo
c
c.....(x,y,z) pointing elsewhere
c
      else
c.......sines and cosines
        rxy = sqrt(rxy)
        ct = z/r
        st = sqrt((one-ct)*(one+ct))
        cf = x/rxy
        sf = y/rxy
c.......compute \Theta functions, storing them in Slm (m>0)
        do l = 1, llmax
          m = l - 1
          im = il0(m) + m
          il = il0(l) + l
          slmm(il) = sqrt(dble(l+l+1)/dble(l+l)) * st * slmm(im)
          g = sqrt(dble(l+l+1))
          slmm(il-1) = g * ct * slmm(im)
          do ll = l+1, llmax
            f = sqrt( dble((ll+ll+1)*(ll+ll-1)) / dble((ll+m)*(ll-m)) )
            slmm(il0(ll)+m) = f * (ct*slmm(il0(ll-1)+m)
     &                              - slmm(il0(ll-2)+m)/g)
            g = f
          enddo
        enddo
c.......compute \Phi functions and multiply them by \Theta
        cc = sqrt(two)
        ss = zero
        do m = 1, llmax
          tmp = cc*cf - ss*sf
          ss = ss*cf + cc*sf
          cc = tmp
          m2 = m+m
          ip = il0(m-1)+m
          do l2 = m2, llmax+llmax, 2
            ip = ip + l2
            slmm(ip-m2) = slmm(ip) * ss
            slmm(ip) = slmm(ip) * cc
          enddo
        enddo
      endif
c
c.....end
c
      return
      end
