c
c-----------------------------------------------------------------------
c
      real*8 function rsupk (na,za,xna,nb,zb,xnb,
     &                       nc,zc,xnc,nd,zd,xnd,k)
c
c.....rsupk - computes the integral R sup K over STOs.
c
c     The integral is defined:
c
c      k                            k   k+1
c     R (a,b;c,d) = <X (1) X (2)| (r  /r   ) |X (1) X (2)>
c                     a     b       <    >     c     d
c
c     where Xi is an STO and the numbers in parenthesis are electrons.
c     The algorithm is based on that from Roothaan and Bagus.
c
c.....Input parameters:
c     Ni ........ principal quantum number of the STO Xi.
c     Zi ........ orbital exponent of the STO Xi.
c     XNi ....... normalization constant of the STO Xi.
c     (where i may be: a, b, c or d).
c     K ......... k exponent in the integral.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'fact.inc'
      include           'comb.inc'
c
c.....Macro definitions
c
      v(x,i) = fact(i) / x**(i+1)
c
c.....auxiliar variables
c
      nac=na+nc
      nbd=nb+nd
      nabcd=nac+nbd
      idx0 = ((nabcd-1)*(nabcd+2))/2 + 1
      zac=za+zc
      zbd=zb+zd
c
c.....first term
c
      t = zac/zbd
      term1=1d0
      tm = 1d0
      do m=1,nac-k-1
         tm = tm*t
         term1=term1+comb(idx0+m)*tm
      enddo
      term1=term1*v(zac,nac-k-1)*v(zbd,nbd+k)/(1+t)**nabcd
c
c.....second term
c
      t = zbd/zac
      term2=1d0
      tm = 1d0
      do m=1,nbd-k-1
         tm = tm*t
         term2=term2+comb(idx0+m)*tm
      enddo
      term2=term2*v(zac,nac+k)*v(zbd,nbd-k-1)/(1+t)**nabcd
c
c.....add both terms
c
      rsupk=xna*xnb*xnc*xnd*(term1+term2)
      return
      end
