c
c-----------------------------------------------------------------------
c
      subroutine scfat (ni, luw, nimpre)
c
c.....scfat - atomic SCF process based on the ATOM program by Roothaan &
c     Bagus, as modified by Clementi et al.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'consts.inc'
      include           'cont.inc'
      include           'coup.inc'
      include           'error.inc'
      include           'scf.inc'
      include           'symmetry.inc'
c
      integer           ni, luw, nimpre
c
      logical           prtmat
c
c     NitSCF=1     !Number of SCF cycles.
c     NitSC1=1     !Extrapolate coefficients every 3 cycles.
c     NitSC2=1     !Increase convergence umbral every Nxtrp iterations.
c     Nconv=0      !Set to 1 when the convergence is reached.
      nitscf = 1
      nitsc1 = 1
      nitsc2 = 1
      nconv = 0
      nbias = bias
      if (nimpre .ge. 2) write (luw,600)
      call prtimer (2, ipid, 'scfat', -1)
c
c.....save original values for damping
c
      if (dodamp .or. Mxxtrp.le.0) then
         nstep2 = 0
         do i = 1, nirrep(ni)
            do j = 1, ncsh(i,ni)+nosh(i,ni)
               j1 = j+nstep2
               do m = 1, npgao(i,ni)
                  c0scf(m,j1) = gc(m,j1,ni)
               enddo
            enddo
            nstep2 = nstep2+ncsh(i,ni)+nosh(i,ni)
         enddo
         do i = 1, ntgaos(ni)
            e0orb(i) = georb(i,ni)
         enddo
      endif
c
c-----begin the SCF process:-------------------------------------------
c     DO WHILE (Nconv.EQ.0 .AND. NitSCF.LE.Mxxtrp)
c
 1    if (nconv.eq.0 .and. nitscf.le.mxxtrp) then
c
c........control the printing of matrices:
c
         prtmat = (nitscf.eq.1) .and. (nimpre.ge.2)
c
c........compute the density matrices:
c
         call densi (ni)
c
c........compute the Fock matrices:
c
         call fockv (ni)
c
c........print out density and Fock matrices:
         if (prtmat) then
            write (luw,*) 'Iteration: ', nitscf
            write (luw,705)
            write (luw,710) (dt(i),i=1,ndens(ni))
            write (luw,715)
            write (luw,710) (fc(i),i=1,ndens(ni))
            if (openshell(ni)) then
               write (luw,720)
               write (luw,710) (dos(i),i=1,ndens(ni))
               write (luw,725)
               write (luw,710) (fo(i),i=1,ndens(ni))
            endif
         endif
c
c........store the results from the two previous SCF cycles:
c
         nstep2 = 0
         do 10 i = 1, nirrep(ni)
            do 15 j = 1, ncsh(i,ni)+nosh(i,ni)
               j1 = j+nstep2
               do 20 m = 1, npgao(i,ni)
                  cm2scf(m,j1) = cm1scf(m,j1)
                  cm1scf(m,j1) = gc(m,j1,ni)
 20            continue
 15         continue
            nstep2 = nstep2+ncsh(i,ni)+nosh(i,ni)
 10      continue
c
c........Jacobi diagonalization procedure (and printing of results):
c
         call jdaaa (ni, prtmat, luw)
c
c........convergence control:
c        (threm = 1d-8 is the default)
c........obtain the actual convergence umbral:
c
         thrscf = threm*(2d0**nbias)
         if (thrscf.gt.scfdiv) then
            write (luw,500) scfdiv
            call error ('scfat', 'SCF process diverges !', faterr)
         endif
         superior = 0d0
         nstep = 0
         do 25 i = 1, nirrep(ni)
            nbas1 = npgao(i,ni)
            nsh = ncsh(i,ni)+nosh(i,ni)
            do 30 j = 1, nsh
               j1 = j+nstep
               do 35 m = 1, nbas1
                  term = abs(abs(gc(m,j1,ni))-abs(cm1scf(m,j1)))
                  if (term.gt.superior) superior = term
 35            continue
 30         continue
            nstep = nstep+nsh
 25      continue
         if (superior.le.thrscf) then
c
c...........convergence has been obtained:
c
            nconv = 1
         else
c
c...........increase the convergence umbral every Nxtrp iterations:
c
            if (nitsc2.ge.nxtrp) then
               nbias = nbias+1
               nitsc2 = 0
            endif
            nitscf = nitscf+1
            nitsc2 = nitsc2+1
c
c...........extrapolate every 3 cycles:
c
            if (nitsc1.ge.3) then
               call extra (ni)
               nitsc1 = 1
            else
               nitsc1 = nitsc1+1
            endif
         endif
         goto 1
c
c-----ENDDO  !End of the SCF loop.-------------------------------------
      endif
c
c.....diagonalize and write out the SCF matrices:
c
      prtmat = nimpre.ge.2
      if (prtmat) then
         call jdaaa (ni, prtmat, luw)
      endif
c
c.....set damping if requested
c
      if (dodamp) then
         nstep2 = 0
         do i = 1, nirrep(ni)
            do j = 1, ncsh(i,ni)+nosh(i,ni)
               j1 = j+nstep2
               do m = 1, npgao(i,ni)
                  gc(m,j1,ni) = damp*c0scf(m,j1)+(one-damp)*gc(m,j1,ni)
               enddo
            enddo
            nstep2 = nstep2+ncsh(i,ni)+nosh(i,ni)
         enddo
         do i = 1, ntgaos(ni)
            georb(i,ni) = damp*e0orb(i) + (one-damp)*georb(i,ni)
         enddo
      endif
c
c.....warn if no SCF cycles, and ignore diagonalization results
c
      if (Mxxtrp.le.0) then
         write (luw,700)
         if (prtmat) then
            nstep2 = 0
            do i = 1, nirrep(ni)
               do j = 1, ncsh(i,ni)+nosh(i,ni)
                  j1 = j+nstep2
                  do m = 1, npgao(i,ni)
                     gc(m,j1,ni) = c0scf(m,j1)
                  enddo
               enddo
               nstep2 = nstep2+ncsh(i,ni)+nosh(i,ni)
            enddo
            do i = 1, ntgaos(ni)
               georb(i,ni) = e0orb(i)
            enddo
         endif
      endif
c
c.....compute the density matrices:
c
      call densi (ni)
c
c.....compute the Fock matrices:
c
      call fockv (ni)
      if (prtmat) then
         write (luw,*) 'SCF cycle: ',nitscf
         write (luw,705)
         write (luw,710) (dt(i),i=1,ndens(ni))
         write (luw,715)
         write (luw,710) (fc(i),i=1,ndens(ni))
         if (openshell(ni)) then
            write (luw,720)
            write (luw,710) (dos(i),i=1,ndens(ni))
            write (luw,725)
            write (luw,710) (fo(i),i=1,ndens(ni))
         endif
      endif
c
c.....compute the total energy:
c
      call energ (ni)
c
c.....print out the results:
c
      if (nimpre.ge.0) call scfout (ni,luw)
      if (nimpre.gt.3) call energsym (ni,luw)
c
c.....stop the program if convergence has not been reached:
c
      if (nconv.eq.0) then
         write (luw,510) nitscf
         if (mxxtrp .gt. 0) call error ('scfat',
     &                     'SCF convergence not reached !', faterr)
      endif
c
c.....return to the main program:
c
      call prtimer (4, ipid, 'scfat', -1)
      return
c
c.....formats:
c
 500  format (/1x, 'SCF diverges: ThrSCF >',e15.8)
 510  format (/1x, 'Convergence not reached in',i6,' iterations')
 600  format (//
     & 1x, '*******************************************************'/
     & 1x, '** Information on the convergence of the SCF process **'/
     & 1x, '*******************************************************'/)
 700  format (/1x, '*** Caution: no SCF iteration, diagonalization',
     &             ' results will be ignored'/)
 705  format (/1x, 'DT() Matrix:')
 710  format (10f12.6)
 715  format (/1x, 'FC() Matrix:')
 720  format (/1x, 'DOS() Matrix:')
 725  format (/1x, 'FO() Matrix:')
      end
