c
c-----------------------------------------------------------------------
c
      function sewald (xni,yni,zni,qq,nsh)
c
c.....ewalder - computes the Ewald potential at (xni, yni, zni).
c
      include           'implicit.inc'
      include           'global.inc'
      include           'celatm.inc'
      include           'fact.inc'
      include           'red.inc'
      include           'converge.inc'
      include           'rsh.inc'
      include           'consts.inc'
      include           'error.inc'
      include           'ewald.inc'
      include           'multipole.inc'
      include           'cont.inc'
      include           'coed.inc'
      include           'neqatm.inc'
c
c.....input parameters:
c
c     (xni,yni,zni) ....... cristallographic coordinates for active ion
c     qq .................. charge at this position
c
c.....output parameters:
c
c     nsh    .......... number of shells summed up
c     sewald .......... Ewald potential value
      real*8            xni, yni, zni, qq
      integer           nsh
c
c.....local variables
c
      real*8            wa(3)
      logical           even
c
c.....online functions: modulus of a vector in cartesian coordinates
c
      dis(xd,yd,zd) = sqrt(xd*xd + yd*yd + zd*zd)
c
c.....(A) Prepare the ewald summations
c
c.....shift the test point to the first cell
c
      if (abs(xni - nint(xni)) .le. epsfrac) then
         wa(1) = zero
      else if (xni.ge.zero) then
         wa(1) = xni - int(xni)
      else
         wa(1) = xni - int(xni) + one
      endif
      if (abs(yni - nint(yni)) .le. epsfrac) then
         wa(2) = zero
      else if (yni.ge.zero) then
         wa(2) = yni - int(yni)
      else
         wa(2) = yni - int(yni) + one
      endif
      if (abs(zni - nint(zni)) .le. epsfrac) then
         wa(3) = zero
      else if (zni.ge.zero) then
         wa(3) = zni - int(zni)
      else
         wa(3) = zni - int(zni) + one
      endif
c
c.....Initialize varibles
c
      sewald = zero
c
c.....(B) Ewald lattice sums of the potential
c
c.....shell number zero: only real part, excluding sublattice self
c     interactions
c
      ewsh = zero
      do i = 1, ncel
         dr1 = xcel(i) - wa(1)
         dr2 = ycel(i) - wa(2)
         dr3 = zcel(i) - wa(3)
         call tocart (dr1,dr2,dr3,xa,xb,xc)
         r = dis(xa,xb,xc)
         t = r / eta
         t2 = t*t
         if (r.gt.epsfrac) then
c...........Obtain the real spherical harmonics 'slm'
            call rsh (xa,xb,xc,r,slm,2*nsym(typcel(i))-2)
            l = 0
            do lm = 0, nlm(typcel(i))
               if (lm.gt.il0(l)+l) l = l + 1
               if (Qm(lm,i).ne.zero) then
                  Qlm = Qm(lm,i) * xocup(typcel(i))
                  isig = 1
                  if (mod(l,2).eq.1) isig = -1
                  xfact1 = isig*two**l*xncte(l)/(sqrpi*facd(l+l-1))
c.................Obtain the incomplete gamma function 'rgamm'
                  call incog (l+half,t2,rgamm)
                  ewsh = ewsh + xfact1 * rgamm * Qlm * slm(lm) 
     &                        / r**(l+1) 
               endif
            enddo 
c
c.....substract the self potential correction if needed
c
         else
            ewsh = ewsh + qq*tosqrpi/eta
         endif
      enddo
      sewald = ewsh
c
c.....shell summation; nsh is the ordinal number of the current shell;
c     shells will be summed until convergence in the point charge
c     potential or to a maximum number of shells, mlayer
c
      nsh = 0
      do while (abs(ewsh).gt.ewconv .and. nsh.lt.mlayer)
         nsh = nsh + 1
         ewsh = zero
c
c.....summing all vectors in a shell, preparing the reciprocal space
c     coefficients for the sums
c
         do ivec = iavec(nsh-1)+1, iavec(nsh)
            k1 = kvec(1,ivec)
            k2 = kvec(2,ivec)
            k3 = kvec(3,ivec)
            call torecsp (dble(k1),dble(k2),dble(k3),xa,xb,xc) 
            vk = dis(xa,xb,xc)
            vk2 = vk*vk
c...........Obtain the real spherical harmonics 'slm'
            call rsh (xa,xb,xc,vk,slm2,maxmp)
            xpot = exp(-(pieta2*vk2)) / (omega*vk2)
            xpot2 = xpot / halfpi 
c
c.....travel all atoms in the unit cell; first, the reciprocal space
c
            do i = 1, ncel
               rr1 = xcel(i) - wa(1)
               rr2 = ycel(i) - wa(2)
               rr3 = zcel(i) - wa(3)
               l = 0
               do lm = 0, nlm(typcel(i))
                  if (lm.gt.il0(l)+l) l = l + 1
                  if (Qm(lm,i).ne.zero) then
                     Qlm = Qm(lm,i) * xocup(typcel(i))
                     isig = 1
                     if (mod(l,2).eq.1) isig = -1
                     xfact2 = isig*xpot2*xncte(l)*(twopi*vk)**l
     &                       *slm2(lm)/facd(l+l-1)
                     rk = twopi * (rr1*k1+rr2*k2+rr3*k3)
                     even = mod(l,2).eq.0
                     if (even) then ! l even
                        inum = l/2
                        isig = 1
                        if (mod(inum,2).eq.1) isig = -1
                        ewsh = ewsh + isig * xfact2 * Qlm * cos(rk)
                     else ! l odd
                        inum = (l+1)/2
                        isig = 1
                        if (mod(inum,2).eq.1) isig = -1
                        ewsh = ewsh + isig * xfact2 * Qlm * sin(rk)
                     endif
                  endif
               enddo  
c
c.....real part must be computed twice, once for each K and -K
c
               dr1 = rr1 + k1
               dr2 = rr2 + k2
               dr3 = rr3 + k3
               call tocart (dr1,dr2,dr3,xa,xb,xc)
               r = dis(xa,xb,xc)
               t = r / eta
               t2 = t*t
c..............Obtain the real spherical harmonics 'slm'
               call rsh (xa,xb,xc,r,slm,2*nsym(typcel(i))-2)
               l = 0
               do lm = 0, nlm(typcel(i))
                  if (lm.gt.il0(l)+l) l = l + 1
                  if (Qm(lm,i).ne.zero) then
                     Qlm = Qm(lm,i) * xocup(typcel(i))
                     isig = 1
                     if (mod(l,2).eq.1) isig = -1
                     xfact1 = isig*two**l*xncte(l)/(sqrpi*facd(l+l-1))
c....................Obtain the incomplete gamma function 'rgamm'
                     call incog (l+half,t2,rgamm)
                     ewsh = ewsh + xfact1 * rgamm * Qlm * slm(lm)
     &                           / r**(l+1)
                  endif
               enddo
               dr1 = rr1 - k1
               dr2 = rr2 - k2
               dr3 = rr3 - k3
               call tocart (dr1,dr2,dr3,xa,xb,xc)
               r = dis(xa,xb,xc)
               t = r / eta
               t2 = t*t
c..............Obtain the real spherical harmonics 'slm'
               call rsh (xa,xb,xc,r,slm,2*nsym(typcel(i))-2)
               l = 0
               do lm = 0, nlm(typcel(i))
                  if (lm.gt.il0(l)+l) l = l + 1
                  if (Qm(lm,i).ne.zero) then
                     Qlm = Qm(lm,i) * xocup(typcel(i))
                     isig = 1
                     if (mod(l,2).eq.1) isig = -1
                     xfact1 = isig*two**l*xncte(l)/(sqrpi*facd(l+l-1))
c....................Obtain the incomplete gamma function 'rgamm'
                     call incog (l+half,t2,rgamm)
                     ewsh = ewsh + xfact1 * rgamm * Qlm * slm(lm)
     &                           / r**(l+1)
                  endif
               enddo
            enddo ! ncel
         enddo ! ivec
         sewald = sewald + ewsh
      enddo ! while
c
c.....convergence control:
c
      if (nsh.eq.mlayer .and. abs(ewsh).gt.ewconv) then
         call error ('sewald',
     &               'convergence not reached with mlayer shells',
     &               faterr)
      endif
c
c.....end of run
c
      return
      end
