c
c----------------------------------------------------------------------
c
      subroutine spgr (spg, lpt, lptx, ier)
c
c.....spgr - interprets the space group symbol.
c
c.....Based upon work by Allen C. Larson (the National Research
c     Council of Canada) in the sixties.
c
c.....Rewritten in Fortran 77, cleaned up and corrected by Angel Martin
c     Pendas (Universidad de Oviedo).
c
c.....Data in the calling sequence are
c
c     spg ...... space group symbol
c     laue ..... output the laue group number.
c                1=1bar, 2=2/m, 3=mmm, 4=4/m, 5=4/mm, 6=r3r, 7=r3mr,
c                8=3, 9=3m1, 10=31m, 11=6/m, 12=6/mmm, 13=m3 AND 14=m3m
c     naxis .... unique axis in monoclinic space groups
c     ncent .... 1bar flag  (0/1) for (acentric/centric)
c     lcent .... lattice centering number.
c                1=P, 2=A, 3=B, 4=C, 5=I, 6=F and 7=R
c     neqv ..... Number of symmetry matrices generated.
c     jrt ...... neqv (3,4,neqv) matrices
c     cen ...... lattice centering vectors
c     ncv ...... Number of lattice centering vectors.
c     lpt ...... stdout
c                if (lpt.lt.0) no listing will be produced
c     lptx ..... stderr
c                if (lptx.lt.0) no error report will be produced
c     ier ...... if (ier.ne.0). An error has been found.
c
      include           'implicit.inc'
      include           'spgcom.inc'
      include           'consts.inc'
c
      character*(30)    spg, spgup
      character*(1)     chr(25)
c
      dimension         rt(5,4,25),d(3,3)
      dimension         l(4,4)
      dimension         lcen(7)
c
c.....data in lcen for  C B A P F I R  respectively
c
      data
     & lcen(1),lcen(2),lcen(3),lcen(4),lcen(5) /4,3,2,1,6/,
     & lcen(6),lcen(7)                         /5,7/
c
c                  1    2    3    4    5    6    7    8    9    10
      data chr  / ' ', 'C', 'B', 'A', 'P', 'F', 'I', 'R', 'M', 'N',
     &            'D', '1', '2', '3', '4', '5', '6', '-', '/', 'H',
     &            '.', ' ', ' ', ' ', ' ' /
c                  11   12   13   14   15   16   17   18   19   20
c
      do i = 1, 4
         do j = 1, 4
            l(i,j) = 0
         enddo
      enddo
      ier    = 0
      ncent  = 0
      laue   = 0
      naxis  = 0
      ierx   = 0
      n      = 0
      nromb  = 0
      ncubic = 0
c
c.....transform symbol to uppercase
c
      length = 0
      do j = len(spg), 1, -1
        if (spg(j:j).ne.' ' .and. spg(j:j).ne.char(0)) then
          length = j
          goto 5
        endif
      enddo
  5   spgup = spg(1:length)
      do j = 1, length
        if (ichar(spgup(j:j)).ge.ichar('a') .and.
     &      ichar(spgup(j:j)).le.ichar('z')) then
          spgup(j:j) = char(ichar(spgup(j:j))-ichar('a')+ichar('A'))
        endif
      enddo
c
c.....break the space group symbol into the 4 fields and code
c     as numerical values for manipulation
      k = 0
      m = 0
      do 110 j=1,length
         ifound=0
         do 100 i=1,21
c...........Exit if found
            if (spgup(j:j).eq.chr(i)) then
               ifound=i
               go to 101
            endif
 100     continue
 101     if (ifound.ne.0) then
            if (k+m+ifound.ne.1) then
                if (ifound.ne.1) then
                   if ( m.eq.0 ) k=k+1
                   m = m+1
                   l(m,k) = ifound
                else
                   m=0
c..................exit loop if more than 3 fields read
                   if (k.gt.3) goto 200
                endif
                if ( ifound.lt.12 .or. m.ge.4) then
                   m=0
c..................exit loop if more than 3 fields read
                   if (k.gt.3) goto 200
                endif
            endif
         endif
 110  continue
c
c
  200 if ( k.le.1 .or. l(1,1).gt.8 ) then
         ier=1
         if (l(1,1).gt.8) ier=2
         if (lptx.ge.0) call sgerrs (spgup(1:30), ier, lptx)
         return
      endif
c
c.....convert the -N notation to the nb(ar) notation
c
      nmenos=0
      if ( l(1,2).eq.18 ) then
         nmenos=nmenos+1
         call sglpak(l(1,2),ier)
         if (ier .gt. 0 ) then
            if (lptx.ge.0) call sgerrs (spgup(1:30), ier, lptx)
            return
         endif
      endif
      if ( l(1,3).eq.18 ) then
         nmenos=nmenos+1
         call sglpak(l(1,3),ier)
         if (ier .gt. 0 ) then
            if (lptx.ge.0) call sgerrs (spgup(1:30), ier, lptx)
            return
         endif
      endif
      if ( l(1,4).eq.18 ) then
         nmenos=nmenos+1
         call sglpak(l(1,4),ier)
         if (ier .gt. 0 ) then
            if (lptx.ge.0) call sgerrs (spgup(1:30), ier, lptx)
            return
         endif
      endif
c
c.....only one bar allowed
c
      if ( nmenos.gt.1 ) then
         ier=24
         if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
         return
      endif
c
c.....set the matrix count n to 2
c
      n = 2
c
c.....set the translation flags
c
      d(1,1) = zero
      d(1,2) = zero
      d(1,3) = zero
      d(2,1) = zero
      d(2,2) = zero
      d(2,3) = zero
      d(3,1) = zero
      d(3,2) = zero
      d(3,3) = zero
c
c.....set the lattice centering flag. 1=P, 2=A, 3=B, 4=C, 5=I, 6=F, 7=R
c
      lcent = l(1,1)-1
      lcent = lcen(lcent)
      if ( lcent.eq.7 ) then
c
c........rhombohedral lattice.
c        make sure that there is a 3-axis.
c
         if ( l(1,2).ne.14 ) then
            ier=3
            if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
            return
         endif
         if ( l(1,k).ne.8 ) then
c
c...........hexagonal axes. retain r centering and set laue to 8 or 9
c
            if ( l(1,k).eq.20 ) k=k-1
            laue = k+6
         else
c
c...........rhombohedral axes. delete R centering.set laue to 6 or 7
c
            lcent = 1
            k = k-1
            laue = k+4
            nromb=1
         endif
      else
c
c........determine laue and some preliminary data
c
         ier = 0
         ncubic = 0
         call sglatc(k, l, d, lcent, laue, naxis, ier, ncubic, id)
         if ( ier.gt.0 ) then
            if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
            return
         endif
      endif
c
      if ( ncubic.eq.1 .or. nromb.eq.1 ) then
c
c.........cubic or rhombohedral cell. insert the 3-fold axis
c
          call sgrmat(rt(1,1,2),0,1,0,0,0,1,1,0,0)
          call sgrmat(rt(1,1,3),0,0,1,1,0,0,0,1,0)
          n = 4
      endif
c
      call sgrmat(rt,1,0,0,0,1,0,0,0,1)
c
c.....decode the last 3 fields of the symbol
c
      do 20 m=2,k
         if ( l(1,m).eq.0 ) then
            ier=6
            if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
            return
         endif
         i = iabs(l(1,m)-5)
  10     if ( i.le.0.or.i.gt.15 ) then
            ier=7
            if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
            return
         endif
         nxi = n
c
c       A   B   C   M   N   D   1   2   3   4   5   6   -   /   H
c  i=   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15
c
         if (i.ge.1 .and. i.le.5) then
c
c...........a mirror is needed
c
            if (m .eq. 2) then
c
c         A   B   C  AXIS
c  m=     1   2   3
c
               if ( laue.gt.3 )  then
                  if (i.eq.3 ) then
                      ier=10
                      if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
                      return
                  endif
c
c.................a C-axis mirror
c
                  call sgrmat(rt(1,1,n),1,0,0,0,1,0,0,0,-1)
                  rt(3,4,n) = d(3,3)
                  if ( i.eq.1.or.i.eq.5 ) rt(1,4,n) = half
                  if ( i.eq.2.or.i.eq.5 ) rt(2,4,n) = half
                  if ( m.eq. 2.and.l(1,2).eq.17) then
c
c....................if this is a 63-axis the mirror is at 1/4
c
                     if ( l(2,2).eq.14 ) rt(3,4,n)=half
                  endif
               else if (k.eq.2) then
                  if (i.eq.2 ) then
                     ier=9
                     if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
                     return
                  endif
c
c.................a B-axis mirror
c
                  call sgrmat(rt(1,1,n),1,0,0,0,-1,0,0,0,1)
                  rt(2,4,n) = d(2,2)
                  if ( i.eq.1.or.i.eq.5 ) rt(1,4,n) = half
                  if ( i.eq.3.or.i.eq.5 ) rt(3,4,n) = half
               else if (i.eq.1) then
                  ier=8
                  if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
                  return
               else
c
c.................a A-axis mirror
c
                  call sgrmat(rt(1,1,n),-1,0,0,0,1,0,0,0,1)
                  rt(1,4,n) = d(1,1)
                  if ( i.eq.2.or.i.eq.5 ) rt(2,4,n)=half
                  if ( i.eq.3.or.i.eq.5 ) rt(3,4,n)=half
               endif
            else if (m.eq.3) then
               if ( l(1,2).eq.14.or.l(1,2).eq.17 ) then
c
                  if ( laue.eq.7 ) then
c
c....................a diagonal mirrror perpendicular to -110
c
                     call sgrmat(rt(1,1,n),0,1,0,1,0,0,0,0,1)
                     rt(1,4,n) = d(2,2)
                     rt(2,4,n) = -d(2,2)
                     if ( i.eq.3.or.i.eq.5 ) rt(3,4,n) = half
                     if (( laue.ne.7 .or. i.ne.3) .and.
     &                   (  i.ne.3 .and. i.ne.4  )) then
                        if ( lcent.eq.6.or.lcent.eq.4 ) then
c
c.......................F or C-centered tetragonal. glides are 1/4,1/4
c
                           rt(1,4,n) = fourth+rt(1,4,n)
                           rt(2,4,n) = fourth+rt(2,4,n)
                        else
                           rt(1,4,n) = half+rt(1,4,n)
                           rt(2,4,n) = half+rt(2,4,n)
                        endif
                     endif
                  else
c
c....................mirror normal to (1000) in hex cell
c
                     call sgrmat(rt(1,1,n),-1,1,0,0,1,0,0,0,1)
                     if ( i.eq.3 ) rt(3,4,n)=half
                  endif
               else if (l(1,2).eq.15) then
                  if (i.eq.1) then
                     ier=8
                     if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
                     return
                  else
c
c...................a A-axis mirror
c
                    call sgrmat(rt(1,1,n),-1,0,0,0,1,0,0,0,1)
                    rt(1,4,n) = d(1,1)
                    if ( i.eq.2.or.i.eq.5 ) rt(2,4,n)=half
                    if ( i.eq.3.or.i.eq.5 ) rt(3,4,n)=half
                  endif
               else
                  if (i.eq.2 ) then
                     ier=9
                     if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
                     return
                  endif
c
c.................a B-axis mirror
c
                  call sgrmat(rt(1,1,n),1,0,0,0,-1,0,0,0,1)
                  rt(2,4,n) = d(2,2)
                  if ( i.eq.1.or.i.eq.5 ) rt(1,4,n) = half
                  if ( i.eq.3.or.i.eq.5 ) rt(3,4,n) = half
               endif
            else if (m.eq.4) then
c
c..............it is not cubic or tetragonal nor trigonal or hexagonal
c
               if (( l(1,3).eq.14.or.l(1,2).eq.15 ) .or.
     &             ( l(1,2).eq.14.or.l(1,2).eq.17 )) then
c
c.................a diagonal mirrror perpendicular to -110
c
                  call sgrmat(rt(1,1,n),0,1,0,1,0,0,0,0,1)
                  rt(1,4,n) = d(2,2)
                  rt(2,4,n) = -d(2,2)
                  if ( i.eq.3.or.i.eq.5 ) rt(3,4,n) = half
                  if (( laue.ne.7 .or. i.ne.3) .and.
     &                (  i.ne.3 .and. i.ne.4  )) then
                     if ( lcent.eq.6.or.lcent.eq.4 ) then
c
c....................F or C-centered tetragonal. glides are 1/4,1/4
c
                        rt(1,4,n) = fourth+rt(1,4,n)
                        rt(2,4,n) = fourth+rt(2,4,n)
                     else
                        rt(1,4,n) = half+rt(1,4,n)
                        rt(2,4,n) = half+rt(2,4,n)
                     endif
                  endif
               else
                  if ( i.eq.3 ) then
                     ier=10
                     if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
                     return
                  endif
c
c.................a c-axis mirror
c
                  call sgrmat(rt(1,1,n),1,0,0,0,1,0,0,0,-1)
                  rt(3,4,n) = d(3,3)
                  if ( i.eq.1.or.i.eq.5 ) rt(1,4,n) = half
                  if ( i.eq.2.or.i.eq.5 ) rt(2,4,n) = half
                  if ( m.eq. 2.and.l(1,2).eq.17) then
c
c....................if this is a 63-axis the mirror is at 1/4
c
                     if ( l(2,2).eq.14 ) rt(3,4,n)=half
                  endif
               endif
            endif
         else if (i.eq.6 ) then
c
c...........d type mirror
c
            if ( lcent.le.1 ) then
               ier=11
               if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
               return
            endif
c
c           go to (500,246,247,248),m
c
            if (m.eq.2) then
               if ( laue.gt.3 ) then
                  call sgrmat(rt(1,1,n),1,0,0,0,1,0,0,0,-1)
                  rt(1,4,n) = fourth
                  rt(2,4,n) = fourth
                  if ( id.eq.2 ) rt(3,4,n)=fourth
               else if (k.eq.2) then
                  call sgrmat(rt(1,1,n),1,0,0,0,-1,0,0,0,1)
                  rt(1,4,n) = fourth
                  if ( id.eq.2 ) rt(2,4,n)=fourth
                  if ( laue.eq.5 ) rt(2,4,n) = d(2,1)
                  rt(3,4,n) = fourth
               else
                  call sgrmat(rt(1,1,n),-1,0,0,0,1,0,0,0,1)
                  if ( id.eq.2 ) rt(1,4,n)=fourth
                  rt(2,4,n) = fourth
                  rt(3,4,n) = fourth
               endif
            else if (m.eq.3) then
               call sgrmat(rt(1,1,n),1,0,0,0,-1,0,0,0,1)
               rt(1,4,n) = fourth
               if ( id.eq.2 ) rt(2,4,n)=fourth
               if ( laue.eq.5 ) rt(2,4,n) = d(2,1)
               rt(3,4,n) = fourth
            else if (m.eq.4) then
               if ( l(1,2).eq.15.or.l(1,3).eq.14 ) then
c
c.................cubic or tetragonal. d-glide along diagonal
c
                  call sgrmat(rt(1,1,n),0,1,0,1,0,0,0,0,1)
                  rt(1,4,n) = fourth
                  rt(2,4,n) = fourth
                  rt(3,4,n) = fourth
                  if (l(1,3).eq.13) then
                     rt(1,4,n) = zero
                     rt(2,4,n) = half
                  endif
               else
c
c.................it is not tetragonal or cubic
c
                  call sgrmat(rt(1,1,n),1,0,0,0,1,0,0,0,-1)
                  rt(1,4,n) = fourth
                  rt(2,4,n) = fourth
                  if ( id.eq.2 ) rt(3,4,n)=fourth
               endif
            endif
         else if (i.eq.7) then
c
c...........1 fold rotation
c
            if ( l(2,m).eq.3 ) then
c
c..............we have a center of symmetry
c
               ncent = 1
            endif
c
c...........Loop exit
c
            goto 20
c
         else if (i.eq.8) then
c
c...........2 fold rotation axis
c...........we will not allow a -2 axis.
c
            if ( l(2,m).eq.3 )then
               ier=19
               if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
               return
            endif
c
c           go to (500,256,257,258),m
c
            if (m.eq.2) then
               if ( k.eq.2 )then
c
c.................rotation about the b-axis
c
                  call sgrmat(rt(1,1,n),-1,0,0,0,1,0,0,0,-1)
                  rt(1,4,n) = d(1,2)
                  rt(3,4,n) = d(3,2)
                  if ( l(2,m).eq.12 ) rt(2,4,n)=half
               else
c
c..................rotation about the a-axis. (orthogonal cell)
c
                   call sgrmat(rt(1,1,n),1,0,0,0,-1,0,0,0,-1)
                   rt(2,4,n) = d(2,1)
                   rt(3,4,n) = d(3,1)
                   if ( iabs(l(2,m)-13).eq.1 ) rt(1,4,n) = half
               endif
            else if (m.eq.3) then
               if ( l(1,2).eq.14 .or. l(1,2).eq.17 ) then
                  if (l(1,2).eq.17 .and. l(1,4).ne.12) then
c
c.....................2-axis normal to (-2110)
c.....................used for the p 6n22 groups
c
                      call sgrmat(rt(1,1,n),1,-1,0,0,-1,0,0,0,-1)
                  else
                     if ( laue.eq.7 ) then
c
c........................2-axis normal to (110)
c
                         call sgrmat(rt(1,1,n),0,-1,0,-1,0,0,0,0,-1)
                     else
c
c........................2-axis along to (11-20) trig  and (110) tetrag
c........................used for the p 3n21 groups
c
                         call sgrmat(rt(1,1,n),0,1,0,1,0,0,0,0,-1)
                         rt(1,4,n) = d(2,1)
                         if ( l(2,m).eq.12 ) rt(1,4,n)=rt(1,4,n)+half
                         rt(2,4,n) = -d(2,1)
                         rt(3,4,n) = d(3,1)
                     endif
                  endif
               else
c
c.................it is not a hexagonal or trigonal group
c.................rotation about the b-axis
c
                  call sgrmat(rt(1,1,n),-1,0,0,0,1,0,0,0,-1)
                  rt(1,4,n) = d(1,2)
                  rt(3,4,n) = d(3,2)
                  if ( l(2,m).eq.12 ) rt(2,4,n)=half
               endif
            else if (m.eq.4) then
c
               if ( l(1,2).ge.14 .or. l(1,3).eq.14) then
                  if ( l(1,2).eq.15 ) then
c
c.....................2-axis along to (11-20) trig  and (110) tetrag
c.....................used for the p 3n21 groups
c
                      call sgrmat(rt(1,1,n),0,1,0,1,0,0,0,0,-1)
                      rt(1,4,n) = d(2,1)
                      if ( l(2,m).eq.12 ) rt(1,4,n)=rt(1,4,n)+half
                      rt(2,4,n) = -d(2,1)
                      rt(3,4,n) = d(3,1)
                  else
ccc                  if ( l(1,2).eq.17.and.l(1,3).ne.12 )
c
c.....................2-axis normal to (10-10)
c
                     call sgrmat(rt(1,1,n),1,0,0,1,-1,0,0,0,-1)
                  endif
               else
                  call sgrmat(rt(1,1,n),-1,0,0,0,-1,0,0,0,1)
                  rt(1,4,n) = d(1,3)
                  rt(2,4,n) = d(2,3)
                  if ( iabs(l(2,m)-13).eq.1 ) rt(3,4,n) = half
                  if (l(2,m).eq.16) rt(3,4,n) = half
               endif
            endif
         else if (i.eq.9) then
            if (m.eq.2 .or. m.eq.3) then
               if (m.eq.2 .and. laue.gt.7) then
                  call sgrmat(rt(1,1,n),0,-1,0,1,-1,0,0,0,1)
                  if ( l(2,m).eq.12 ) rt(3,4,n)=third
                  if ( l(2,m).eq.13 ) rt(3,4,n)=two*third
                  if ( l(2,2).eq.3 ) ncent=1
               else
c
c.................1 fold rotation
c
                  if ( l(2,m).eq.3 ) then
c
c.....................we have a center of symmetry
c
                      ncent = 1
                   endif
c
c..................Loop exit
c
                   goto 20
               endif
            else if (m.eq.4) then
               ier=25
               if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
               return
            endif
         else if (i.eq.10 ) then
c
c...........four fold axis
c
            if ( m.ne.2 ) then
               ier=12
               if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
               return
            endif
            if ( l(2,2).eq. 3 ) then
               call sgrmat(rt(1,1,n),0,1,0,-1,0,0,0,0,-1)
               rt(1,4,n) = d(1,3)
               rt(2,4,n) = d(2,3)
               rt(3,4,n) = d(3,3)
            else
               call sgrmat(rt(1,1,n),0,-1,0,1,0,0,0,0,1)
               rt(1,4,n) = d(1,3)
               rt(2,4,n) = d(2,3)
               if ( l(2,2).eq.12 ) rt(3,4,n) = fourth
               if ( l(2,2).eq.13 ) rt(3,4,n) = half
               if ( l(2,2).eq.14 ) rt(3,4,n) = 75d-2
            endif
         else if (i.eq.11) then
            ier=25
            if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
            return
         else if (i.eq.12) then
c
c...........6-axis
c
            if ( m.ne.2 )then
               ier=13
               if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
               return
            endif
            if ( l(2,2).eq.3 ) then
               call sgrmat(rt(1,1,n),-1,1,0,-1,0,0,0,0,-1)
               if ( l(1,3).eq.2.or.l(1,4).eq.2 ) rt(3,4,n)=half
            else
               call sgrmat(rt(1,1,n),1,-1,0,1,0,0,0,0,1)
               if ( l(2,2).gt.11.and.l(2,2).lt.18 )
     &            rt(3,4,n)=(l(2,2)-11)/6d0
            endif
c
c        else if (i.eq.13 .or. i.eq.14 .or. i.eq.15) then
c
         endif
c
         rt(1,4,n) = mod(rt(1,4,n)+5d0,one)
         rt(2,4,n) = mod(rt(2,4,n)+5d0,one)
         rt(3,4,n) = mod(rt(3,4,n)+5d0,one)
         rt(5,2,n) = 1728*rt(1,4,n)+144*rt(2,4,n)+12*rt(3,4,n)
         njs=n-1
         do 69 m2=1,njs
            if (rt(5,1,m2).eq.rt(5,1,n) .or.
     &          rt(5,1,m2).eq.-rt(5,1,n))  then
               if (rt(5,1,m2).eq.-rt(5,1,n)) ncent=1
               if (rt(5,2,n).ne.rt(5,2,m2)) then
                  call sgtrcf(m,rt,n,m2,lcent,laue,ier,lptx)
                  if (ier.gt.0) ierx=ier
                  ier=0
               endif
c
c..............Loop exit
c
               goto 20
            endif
 69      continue
         n = n+1
         if ( n.gt.25 ) then
            ier=14
            if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
            return
         endif
 59      ident=0
         nxl = n-1
c
c........exit
c
         if ( nxl.lt.nxi )  goto 49
         do 195 nx=nxi,nxl
            do 196 m1=2,nx
               call sgmtml(rt,m1,rt,nx,rt,n)
               njs=n-1
               do 194 m2=1,njs
                  if (rt(5,1,m2).eq.rt(5,1,n) .or.
     &                rt(5,1,m2).eq.-rt(5,1,n))  then
                     if (rt(5,1,m2).eq.-rt(5,1,n)) ncent=1
c
c....................Loop exit
c
                     go to 196
                  endif
194            continue
               n = n+1
               if ( n.gt.25 ) then
                  ier=15
                  if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
                  return
               endif
196         continue
c
c............Loop exit
c
            if ( n-1.eq.nxl ) goto 49
195      continue
         nxi = nxl+1
c
c........Repeat whole cycle
c
         go to 59
c
c........After completing loop cycle
c
 49      continue
c
c........search for a / . a mirror perpendicular to this axis
c........Next m
c
         if ( l(1,m).lt.12 .or. l(2,m).eq.3 )  goto 20
         do 39 i=2,3
c
c...........Next m
c
            if ( l(i,m).eq.0 ) goto 20
c
c...........exit loop 39
c
            if ( l(i,m).eq.19 ) goto 29
            if ( l(i,m).lt.12 ) then
               ier=16
               if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
               return
            endif
 39      continue
c
c........Test for another matrix element
c
 29      if ( l(i+1,m).le.1 ) then
            ier=17
            if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
            return
         endif
         i = iabs(l(i+1,m)-5)
c
c........Next matrix element
c        Non ortodox control flow
c
         goto 10
c
c.....end of loop
c
 20   continue
      neqv = n-1
      do 11 i=1,3
         do 21 k=1,neqv
            do 31 j=1,3
               jrt(i,j,k) = rt(i,j,k)
 31         continue
            jrt(i,4,k) = 12*rt(i,4,k)+144.1d0
            jrt(i,4,k) = jrt(i,4,k)-12*(jrt(i,4,k)/12)
 21      continue
 11   continue
c
c.....print results and construct interface symmetry matrix
c
      call sgprnt(spgup(1:30),lpt)
      if ( ierx.eq.0 ) return
      ier = ierx
c
c.....Error handling
c
      if (lptx.ge.0) call sgerrs(spgup(1:30),ier,lptx)
      return
      end
