c
c-----------------------------------------------------------------------
c
      subroutine prtimer (key,pid,name,luw)
c
c.....prtimer - process timer, accumulates and prints out the elapsed
c             times of a series of up to MPROCESS processes.
c
c     Input parameters are:
c     key ........... 0 = reset all time tables.
c                     1 = reset pid entry and begin counting for it.
c                     2 = continue the cont for pid process. Do not
c                         reset previous entry times.
c                     3 = end of pid process. Free pid entry.
c                     4 = end of pid process. Do not free entry.
c                     5 = end of run. Print out all time tables.
c                     6 = close all processes and print out tables.
c     pid ........... process identification number (1..MPROCESS).
c     name .......... process name (used only in the print out).
c     luw ........... logical unit to write. Negative if print out is
c                     not desired.
c
c.....key controls what data are used in the run, as the following table
c     resumes:
c
c     key value    pid      name       luw
c     ---------  -------   -------   -------
c        0       ignored   ignored   ignored
c       1,2      output    input     ignored
c       3,4      input     ignored   input
c       5,6      ignored   ignored   input
c
      integer          key,pid,luw
      character*(*)    name
c
      include  'error.inc'
c
c     It is deliberately ambiguous in defining "real", since it has to
c     match the system declaration of etime et al (float=real).
c
      integer          mprocess
      parameter        (mprocess=20)
      character*(10)   pname(mprocess)
      real             time(mprocess),cumtime(mprocess),timedum
      logical          popen(mprocess),pused(mprocess)
      logical          firsttime
      integer          pcalls(mprocess)
      save             time,cumtime,pname,popen,pused,pcalls
      save             firsttime
c
      data firsttime /.true./
c
      if (key.eq.0 .or. firsttime) then
c
c........initiallize all entries:
c
         firsttime=.false.
         do 10 i=1,mprocess
            time(i)=0.0
            cumtime(i)=0.0
            popen(i)=.false.
            pused(i)=.false.
            pname(i)=' '
            pcalls(i)=0
 10      continue
      else if (key.eq.1 .or. key.eq.2) then
c
c........begin pid count:
         call atimer (timedum)
         i=1
 15      if (pused(i)) then
            if (pname(i).ne.name) then
               i=i+1
               if (i.gt.mprocess) then
                  call error ('prtimer','pid out of bonds',warning)
                  return
               endif
               goto 15
            endif
         endif
         pid=i
         if (key.eq.1) then
            cumtime(pid)=0.0
            pcalls(pid)=1
         else
            pcalls(pid)=pcalls(pid)+1
         endif
         time(pid)=timedum
         popen(pid)=.true.
         pused(pid)=.true.
         pname(pid)=name
      else if (key.eq.3 .or. key.eq.4) then
c
c........end pid accounting:
c
         if (pid.le.0 .or. pid.gt.mprocess) then
            call error ('prtimer','pid out of bonds',warning)
         else if (.not.popen(pid)) then
            call error ('prtimer','pid unused or closed',warning)
         else
            call atimer (timedum)
            time(pid)=timedum-time(pid)
            cumtime(pid)=cumtime(pid)+time(pid)
            popen(pid)=.false.
            if (luw.gt.0) write (luw,100) pname(pid),time(pid)
            if (key.eq.3) then
               pused(pid)=.false.
               pcalls(pid)=0
               cumtime(pid)=0.0
               time(pid)=0.0
            endif
         endif
      else if (key.eq.5 .or. key.eq.6) then
c
c........print out the time tables:
c
         write (luw,105)
         call atimer (timedum)
         do 25 i=1,mprocess
            if (pused(i)) then
               if (popen(i)) then
                  time(i)=timedum-time(i)
                  cumtime(i)=cumtime(i)+time(i)
                  if (key.eq.6) popen(i)=.false.
               endif
               write (luw,110) i,pname(i),cumtime(i),pcalls(i),popen(i)
            endif
 25      continue
         write (luw,115)
      else
         call error ('prtimer','key value not recognized',warning)
      endif
      return
c
 100  format (/1x,'*** prtimer:'/
     & 1x,'*** process name: ',a10,5x,'elapsed time (sec):',f10.3/)
 105  format (/1x,'***'/1x,'*** prtimer:'/
     & 1x,'*** elapsed time (sec)'/1x,'***'/
     & 1x,'*** -pid------name-----cumtime----pcalls--popen-')
 110  format (1x,'***',i4,2x,a10,f11.3,1x,i7,6x,l2)
 115  format (1x,'***'/)
      end
