c
c-----------------------------------------------------------------------
c
      subroutine symmint (luw, nimpre, ecrys, ceex, cepl)
c
c.....symmint - recomputes the cluster-lattice interaction using a
c     symmetrized expresion for the interaction energy on an crystal
c     or cluster calculation.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'cont.inc'
      include           'consts.inc'
      include           'converge.inc'
      include           'iondens.inc'
      include           'environ.inc'
      include           'error.inc'
      include           'impurity.inc'
      include           'lint.inc'
      include           'neqatm.inc'
      include           'strtools.inc'
      include           'vef.inc'
      include           'tfunc.inc'
      include           'rsh.inc'
      include           'dgaunt.inc'      
c
c.....input parameters
c
      integer           luw, nimpre
      real*8            ecrys, ceex(mnt), cepl(mnt)
c
c.....used indexes: ni, active ion; nf, frozen ion; ish, shell of ions;
c     lpq, active ion density index; lab, frozen ion density index
c
      integer           ni, nf, ish, lpq, lab
c
c.....used as temporal vector in vcl, and as integrals vector in vnc,
c     vex and pla; its index are lpq,lab, where lpq refers to densities
c     of the active center, and lab to densities of the frozen one
c
      real*8            o(mdens,mdens)
c
c.....stores the integrals between densities in one center and all
c     centers of a given type
c
      real*8            term(mdens,mdens)
c
c.....store the symmetrized total exchange and projection contributions
c
      real*8            eexsym, eplsym, eexnol, eplnol
      real*8            totexsym, totplsym, totexnol, totplnol
      real*8            vxtmp, vptmp, vxdum, vpdum
c
      logical           done, converged
      character*(128)   cdum
      real*8            tiny
      parameter         ( tiny = 1d-21 )
c
      call prtimer (2,ipid,'symmint',luw)
c
c.....Loop over nonequivalent ions
c
      totexsym = zero
      totplsym = zero
      totexnol = zero
      totplnol = zero
      do ni = 1, nneq
c
c.....Initialize the matrices: nol matrices will store the asymmetric
c     contributions, regular matrices the symmetrized ones
c
        eexsym = zero
        eexnol = zero
        eplsym = zero
        eplnol = zero
        do lpq = 1, ndens(ni)
          vex(lpq)    = zero
          pla(lpq)    = zero
          vexnol(lpq) = zero
          planol(lpq) = zero
        enddo
c
c.....Create the environment around each ion
c
        if (nimpre .ge. 0) write (luw,600) namneq(ni), ni
        if (infsystem) then
          call envcrys (ni, nimpre-1, luw)
        else
          call envclus (ni, nimpre-1, luw)
        endif
c
c.....Loop over different types of environment ions
c
        isha = 0
        do nf = 1, nneq
          if (nimpre.ge.-1) write (luw,610) namneq(nf), nf
          xs = xocup(nf) 
c
c.....exchange & projection operator contributions
c
          converged = .false.
          done = nenvni(nf).le.0 ! nenvni(nf) = shells of the ions type 'nf'
          ish = isha + 1
          do while (.not.done)
            rs = renv(ish) ! distance of the shell 'ish'
            termax = zero
            do lpq = 1, ndens(ni)
              do lab = 1, ndens(nf)
                term(lab,lpq) = zero
              enddo
            enddo
            do ieq = 1, eqenv(ish) ! eqenv (ish) = number of 'nf' ions in the 'ish' shell
c
c...........Coordinate conversion to cartesian coordinates
c
              call tocart (xenv(ieq,ish), yenv(ieq,ish),
     &                     zenv(ieq,ish), xa, xb, xc)

c
c...........Compute the spherical harmonics
c
              call rsh (xa, xb, xc, rs, slm, maxl)
c
c...........Obtention of the T^l_{lama,lbmb}(R)
c
              j = 0 ! counter for (lm1,lm2,l3)
              l1 = 0 ! l1 = 0,...,maxmp
              do lm1 = 0, lmmax
c             lm1 = il0(l1) + m1; lm1(max) = l1*(l1+2)
                if (lm1.gt.il0(l1)+l1) l1 = l1 + 1
                 m1 = lm1 - il0(l1) ! m1 = -l1,...,l1
                 l2 = 0 ! l2 = 0,...,maxmp
                 do lm2 = 0, lm1
c                lm2 = il0(l2) + m2; lm2(max) = lm1
                   if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                   m2 = lm2 - il0(l2) ! m2 = -l2,...,l2
                   do l3 = l1-l2, l1+l2, 2 ! triangular inequality
                     j = j + 1
                     i = ig0(lm1,lm2,l3)
                     itfunc(lm1,lm2,l3) = j
                     itfunc(lm2,lm1,l3) = j
                     tfunc(j) = zero ! initializations
                     do im3 = 1, m3num(m1,m2,l3)
                       m3 = m3val(m1,m2,im3)
                       lm3 = il0(l3) + m3
                       tfunc(j) = tfunc(j) + dgaunt(i+im3) * slm(lm3)
                     enddo
                   enddo
                 enddo
              enddo
              call overlap (ni, nf, rs, o)
              do lpq = 1, ndens(ni)
                do lab = 1, ndens(nf)
                  term(lab,lpq) = term(lab,lpq) + o(lpq,lab)
                enddo
              enddo
            enddo ! ieq (ions 'nf' that compose the shell 'ish')
            do lpq = 1, ndens(ni)
              vxtmp = zero ! initializations
              vptmp = zero ! initializations
              vxdum = zero ! initializations
              vpdum = zero ! initializations
              if (denlp(lpq,ni).eq.denlq(lpq,ni).and.
     &            denkp(lpq,ni).eq.denkq(lpq,ni).and.
     &            denp(lpq,ni).eq.denq(lpq,ni)) then
                ilpq = 2 
              else
                ilpq = 1  
              endif
              do lab = 1, ndens(nf)
                if (denlp(lab,nf).eq.denlq(lab,nf).and.
     &              denkp(lab,nf).eq.denkq(lab,nf).and.
     &              denp(lab,nf).eq.denq(lab,nf)) then
                  ilab = 2 
                else
                  ilab = 1
                endif                   
                fac = ilab*rhomat(lab,nf)/ilpq
                term(lab,lpq) = term(lab,lpq) * xs
                vxtmp = vxtmp + term(lab,lpq) * aexch(lab,nf)
                vptmp = vptmp + term(lab,lpq) * aproj(lab,nf)
                vxdum = vxdum + term(lab,lpq) * fac 
                vpdum = vpdum + term(lab,lpq) * fac
              enddo ! lab
              vxdum = vxdum * aexch(lpq,ni)
              vpdum = vpdum * aproj(lpq,ni)
              vexnol(lpq) = vexnol(lpq) - vxtmp
              vex(lpq) = vex(lpq) - half * (vxtmp+vxdum/rhomat(lpq,ni))
              planol(lpq) = planol(lpq) + vptmp
              pla(lpq) = pla(lpq) + half * (vptmp+vpdum/rhomat(lpq,ni))
              eexsym = eexsym - vxdum
              eplsym = eplsym + vpdum
              relerr = max(abs(vxtmp/(vexnol(lpq)+tiny)),
     &                     abs(vptmp/(planol(lpq)+tiny))) 
              if (relerr.gt.termax) termax = relerr
            enddo ! lpq
            if (nimpre.ge.4) then
              write (luw,620) ish-isha, 'Vxp'
              do lpq = 1, ndens(ni)
                write (luw,710) (term(lab,lpq), lab = 1, ndens(nf))
              enddo
            endif
            converged = termax.le.convxp
            ish = ish + 1
            done = converged .or. ish.gt.isha+nenvni(nf)
          enddo ! while
          if (.not. converged .and. infsystem) then
            write (luw,810) 'Vxp', ish-isha-1, termax
            cdum = 'Vxp is not fully converged'
            call error ('symmint', cdum, warning) 
          else
            if (nimpre.ge.-1) write (luw,800) 'Vxp', ish-isha-1
          endif
c.....end loop of non-local ions: write accumulated contribution
c
          isha = isha + nenvni(nf)
        enddo ! nf
c
c.....print out the results:
c
        if (nimpre .ge. 1) write (luw,700) 
        if (nimpre .ge. 2) then
           write (luw,705) 'Exchange operators'
           write (luw,710) (vex(k), k = 1, ndens(ni))
           write (luw,705) 'Projection operators'
           write (luw,710) (pla(k), k = 1, ndens(ni))
        endif
c
c.....compute and store the accumulated symmetrized contribution
c
        do lpq = 1, ndens(ni)
          eexnol = eexnol + vexnol(lpq) * rhomat(lpq,ni)
          eplnol = eplnol + planol(lpq) * rhomat(lpq,ni)
        enddo
        excorr = half * (eexsym - eexnol)
        plcorr = half * (eplsym - eplnol)
        eexsym = half * (eexsym + eexnol)
        eplsym = half * (eplsym + eplnol)
        ecrys = ecrys + half * (excorr + plcorr) * mult(ni)
        eadd0 = eadd(ni)
        eadd(ni) = eadd0 + half * (excorr + plcorr)
        eeff(ni) = eeff(ni) + excorr + plcorr
        epot(ni) = epot(ni) + excorr + plcorr
        eint(ni) = eint(ni) + excorr + plcorr
        ceex(ni) = ceex(ni) + excorr
        cepl(ni) = cepl(ni) + plcorr
        totexsym = totexsym + eexsym * mult(ni)
        totplsym = totplsym + eplsym * mult(ni)
        totexnol = totexnol + eexnol * mult(ni)
        totplnol = totplnol + eplnol * mult(ni)
        if (nimpre.ge.0) then
          write (luw,900) ni, namneq(ni), eadd0, eadd(ni)
          write (luw,910) eexnol, eexsym, eplnol, eplsym
        endif
      enddo ! ni
c
c.....print out global corrections
c
      if (nimpre.ge.-1) then
        write (luw,950) totexnol, totexsym, totplnol, totplsym
        write (luw,960) totexnol+totplnol, totexsym+totplsym,
     &                  totexsym+totplsym-totexnol-totplnol
      endif
      call prtimer (4,ipid,'symmint',-1)
      return
c
 600  format (/'*** SYMMETRIZING LATTICE INTERACTION FOR : ',
     &        a, ' (no. ', i2, ' )'/)
 610  format (1x, ' Lattice integrals due to ', a, ' (no. ', i2, ' )')
 620  format (/1x,'+ Shell no. ', i2, 1x, a, ' integrals +')
 700  format (/)
 705  format (/1x, '+ Symmetrized lattice integrals due to: ', a, '+')
 710  format (1x, 1p, 10e13.5)
 800  format (1x, 'Shells used to converge ', a, ' : ', i4)
 810  format (a, ' not converged. Last shell added: ', i4,
     &           ' Integral error (rel.): ', e13.6)
 900  format (/
     & 1x, 'Additive energy with a symmetrized interaction:'/
     & 1x, 'Atom number & name             : ', i4, 2x, a/
     & 1x, 'Orig. asymmetric energy        : ', 1pe21.14/
     & 1x, 'Symmetrized energy             : ', 1pe21.14)
 910  format (
     & 1x, 'Orig. asymmetric exchange en.  : ', 1pe21.14/
     & 1x, 'Symmetrized exchange energy    : ', 1pe21.14/
     & 1x, 'Orig. asymmetric projection en.: ', 1pe21.14/
     & 1x, 'Symmetrized projection energy  : ', 1pe21.14)
 950  format (//
     & 1x, 'Global corrections due to the symmetrized interaction:'//
     & 1x, 'Orig. asymmetric exchange en.  : ', 1pe21.14/
     & 1x, 'Symmetrized exchange energy    : ', 1pe21.14/
     & 1x, 'Orig. asymmetric projection en.: ', 1pe21.14/
     & 1x, 'Symmetrized projection energy  : ', 1pe21.14)
 960  format (/
     & 1x, 'Total asymmetric interaction   : ', 1pe21.14/
     & 1x, 'Total symmetrized interaction  : ', 1pe21.14//
     & 1x, 'Symmetrization correction      : ', 1pe21.14/)
c
      end
