c
c-----------------------------------------------------------------------
c
      subroutine symmloc (xcenter, ycenter, zcenter)
c
c.....symmloc - use the espacial symmetry information to obtain
c               the puntual symmetry for the cluster. Then, obtain
c               the equivalent ions in the cluster. Moreover, obtain
c               the puntual symmetry subgroup for each non-equivalent 
c               atom.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'consts.inc'
      include           'neqatm.inc'
      include           'error.inc'
      include           'celatm.inc'
      include           'spgcom.inc'
      include           'symloc.inc'
      include           'impurity.inc'
c
      integer           xcenter, ycenter, zcenter
c
      integer           indaux(2*neqv), nprim(mimp)
      real*8            xxcenter(3) 
      real*8            vec(2*neqv,3)
      real*8            rotpunt(3,3,48)
      logical           val
c
      xxcenter(1) = xcenter
      xxcenter(2) = ycenter
      xxcenter(3) = zcenter
      ipuntual = 0 
      mrot = 0
      do j = 1, neqv
c.....STEP I: Identification of the local symmetry for the
c             center position
         mrot = mrot + 1
         val = .true.
         do l = 1, 3
            vec(mrot,l) = zero
c...........Ia: Action of Seitz operators on the center position. 
c               Proper rotation            
            do m = 1, 3
               vec(mrot,l) = vec(mrot,l) + rotm(l,m,j) * xxcenter(m)
            enddo
*c...........Ib: Action of Seitz operators on the center position. Traslation, 
*                where 0<=rotm(l,4,j)<=1 (if it exist)
*            vec(mrot,l) = vec(mrot,l) + rotm(l,4,j)
            val = val .and. abs(rotm(l,4,j)).lt.EPSFRAC
c...........Ib: Move created points to the zeroth cell
            if (abs(vec(mrot,l)-nint(vec(mrot,l))).le.EPSFRAC) then
               vec(mrot,l) = zero
            else if (vec(mrot,l).lt.zero) then
               vec(mrot,l) = vec(mrot,l)+1
            else if (vec(mrot,l).ge.one) then
               vec(mrot,l) = vec(mrot,l)-1
            endif
         enddo
c........Ic: Identification of the local symmetry. Proper rotations  
         if (abs(vec(mrot,1)-xxcenter(1)).lt.EPSFRAC .and.
     &       abs(vec(mrot,2)-xxcenter(2)).lt.EPSFRAC .and.
     &       abs(vec(mrot,3)-xxcenter(3)).lt.EPSFRAC .and. val) then  
*         if (abs(vec(mrot,1)-xxcenter(1)).lt.EPSFRAC .and.
*     &       abs(vec(mrot,2)-xxcenter(2)).lt.EPSFRAC .and.
*     &       abs(vec(mrot,3)-xxcenter(3)).lt.EPSFRAC) then
             ipuntual = ipuntual + 1
             indaux(ipuntual) = j
             do l = 1, 3
                do m = 1, 3
                   rotpunt(l,m,ipuntual) = rotm(l,m,j)
                enddo
             enddo
         endif
c........Id: Identification of the local symmetry. Improper rotations
         if (ncent.eq.1) then
            mrot = mrot + 1
            do l = 1, 3
               aux = -vec(mrot-1,l)
               if (abs(aux-nint(aux)).le.EPSFRAC) then
                  vec(mrot,l) = zero
               else if (aux.lt.zero) then
                  vec(mrot,l) = aux + 1
               else
                  vec(mrot,l) = aux
               endif
            enddo
            if (abs(vec(mrot,1)-xxcenter(1)).lt.EPSFRAC .and.
     &          abs(vec(mrot,2)-xxcenter(2)).lt.EPSFRAC .and.
     &          abs(vec(mrot,3)-xxcenter(3)).lt.EPSFRAC .and. val) then
*            if (abs(vec(mrot,1)-xxcenter(1)).lt.EPSFRAC .and.
*     &          abs(vec(mrot,2)-xxcenter(2)).lt.EPSFRAC .and.
*     &          abs(vec(mrot,3)-xxcenter(3)).lt.EPSFRAC) then
                ipuntual = ipuntual + 1
                indaux(ipuntual) = -j
                do l = 1, 3
                   do m = 1, 3
                      rotpunt(l,m,ipuntual) = - rotm(l,m,j)
                   enddo
                enddo
            endif
         endif 
      enddo ! j = 1, ..., neqv
c.....STEP II: Obtain the equivalent atoms in the cluster and
c              the puntual symmetry subgroup for each 
c              non-equivalent atom
      nimtot = 0
      do nf = 1, nneq   
         natact = nimtot  
c........IIa: Action of puntual symmetry operators on the
c             non-equivalent atoms in the cluster
         ilocal = 0 
         do i = 1, ipuntual
            nimtot = nimtot + 1
            if (nimtot .gt. mimp)
     &         call error ('symmloc', 'nimtot > mimp', faterr)
            indcel(nimtot) = indaux(i)
            do l = 1, 3
               xximp(nimtot,l) = zero
               do m = 1, 3
                  xximp(nimtot,l) = xximp(nimtot,l) + 
     &                              rotpunt(l,m,i) * xxneq(nf,m)
               enddo
            enddo 
c........IIb: Obtain the puntual symmetry subgroup for each 
c             non-equivalent atom
            if (abs(xximp(nimtot,1)-xxneq(nf,1)).lt.EPSFRAC .and.
     &          abs(xximp(nimtot,2)-xxneq(nf,2)).lt.EPSFRAC .and.
     &          abs(xximp(nimtot,3)-xxneq(nf,3)).lt.EPSFRAC) then
              ilocal = ilocal + 1
              do l = 1, 3
                do m = 1, 3
                  rotloc(l,m,ilocal,nf) = rotpunt(l,m,i)
                enddo
              enddo
            endif  
         enddo  ! i = 1, ..., ipuntual
         nlocal(nf) = ilocal
c........IIb: Destroy identical atoms
         nprim(1) = natact + 1
         num = 1
         do i =  natact+2, nimtot
            val = .true. 
            j = natact + 1
            do while (val .and. j.le.(i-1))
               if (abs(xximp(j,1)-xximp(i,1)).lt.epsfrac .and.
     &             abs(xximp(j,2)-xximp(i,2)).lt.epsfrac .and.
     &             abs(xximp(j,3)-xximp(i,3)).lt.epsfrac) then
                  val = .false. 
               endif
               j = j + 1
            enddo
            if (val) then
               num = num + 1
               nprim(num) = i
            endif
         enddo  
         do i = 1, num
            iact = natact + i
c           iact = natact + 1, ..., natact + num
            if (iact .gt. mimp)
     &         call error ('symmloc', 'iact > mimp!', faterr)
            indcel(iact) = indcel(nprim(i))
            do j = 1, 3
               xximp(iact,j) = xximp(nprim(i),j)
            enddo
         enddo
         nimtot = iact
c........IIc: Assign type
         do i = natact+1, nimtot
            typimp(i) = nf
         enddo
c........IId: Multiplicity of the position
         mult(nf) = nimtot - natact            
      enddo ! nf = 1, ..., nneq
      return
      end         
         
