c
c-----------------------------------------------------------------------
c
      function tewald (xni,yni,zni,qq,l1,m1,nsh)
c
c.....ewalder - computes the Ewald potential at (xni, yni, zni).
c
      include           'implicit.inc'
      include           'global.inc'
      include           'celatm.inc'
      include           'cont.inc'
      include           'fact.inc'
      include           'red.inc'
      include           'converge.inc'
      include           'rsh.inc'
      include           'dgaunt.inc'
      include           'consts.inc'
      include           'error.inc'
      include           'ewald.inc'
      include           'multipole.inc'
      include           'coed.inc'
      include           'neqatm.inc'
c
c.....input parameters:
c
c     (xni,yni,zni) ....... cristallographic coordinates for active ion
c     qq .................. charge at this position
c     (l1,m1) ....... angular and azimutal quantum numbers 
c
c.....output parameters:
c
c     nsh    .......... number of shells summed up
c     tewald .......... Ewald potential value
      real*8            xni, yni, zni, qq
      integer           l1, m1, nsh
c
c.....local variables
c
      real*8            wa(3)
      logical           even
c
c.....online functions: modulus of a vector in cartesian coordinates
c
      dis(xd,yd,zd) = sqrt(xd*xd + yd*yd + zd*zd)
c
c.....(A) Prepare the ewald summations
c
c.....shift the test point to the first cell
c
      if (abs(xni - nint(xni)) .le. epsfrac) then
         wa(1) = zero
      else if (xni.ge.zero) then
         wa(1) = xni - int(xni)
      else
         wa(1) = xni - int(xni) + one
      endif
      if (abs(yni - nint(yni)) .le. epsfrac) then
         wa(2) = zero
      else if (yni.ge.zero) then
         wa(2) = yni - int(yni)
      else
         wa(2) = yni - int(yni) + one
      endif
      if (abs(zni - nint(zni)) .le. epsfrac) then
         wa(3) = zero
      else if (zni.ge.zero) then
         wa(3) = zni - int(zni)
      else
         wa(3) = zni - int(zni) + one
      endif
c
c.....Some constant factors
c
      lm1 = il0(l1) + m1
c
c.....Initialize variables
c
      tewald = zero
c
c.....(B) Ewald lattice sums of the potential
c
c.....shell number zero: only real part, excluding sublattice self
c     interactions
c
      ewsh = zero
      do i = 1, ncel
         dr1 = xcel(i) - wa(1)
         dr2 = ycel(i) - wa(2)
         dr3 = zcel(i) - wa(3)
         call tocart (dr1,dr2,dr3,xa,xb,xc)
         r = dis(xa,xb,xc) 
         t = r / eta
         t2 = t*t
         if (r.gt.epsfrac) then
c...........Obtain the real spherical harmonics 'slm'
            call rsh (xa,xb,xc,r,slm,l1+2*nsym(typcel(i))-2)
            l2 = 0
            do lm2 = 0, nlm(typcel(i))
               if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
               ltot = l1 + l2
               if (Qm(lm2,i).ne.zero) then
                  Qlm = Qm(lm2,i) * xocup(typcel(i))
                  isig = 1
                  if (mod(l2,2).eq.1) isig = -1
                  xfact1 = isig*two**ltot*coefq(l1,l2) /
     &                     (sqrpi*facd(ltot+ltot-1))
c.................Obtain the 't^ltot_{lm1,lm2}' function
                  m2 = lm2 - il0(l2)
                  id = ig0(lm1,lm2,ltot)
                  mmax = m3num(m1,m2,ltot)
                  tslm = zero
                  do im3 = 1, mmax
                     mtot = m3val(m1,m2,im3)
                     lmtot = il0(ltot) + mtot
                     tslm = tslm + dgaunt(id+im3)*slm(lmtot)
                  enddo
c.................Obtain the incomplete gamma function 'rgamm'
                  call incog (ltot+half,t2,rgamm)
                  ewsh = ewsh + xfact1 * rgamm * Qlm * tslm
     &                        / r**(ltot+1)  
               endif
            enddo
c
c.....substract the self potential correction if needed
c
         else
           if (l1.eq.0) then
               ewsh = ewsh + qq*tosqrpi/eta
           endif
         endif
      enddo
      tewald = ewsh
c
c.....shell summation; nsh is the ordinal number of the current shell;
c     shells will be summed until convergence in the point charge
c     potential or to a maximum number of shells, mlayer
c
      nsh = 0
      do while (abs(ewsh).gt.ewconv .and. nsh.lt.mlayer)
         nsh = nsh + 1
         ewsh = zero
c
c.....summing all vectors in a shell, preparing the reciprocal space
c     coefficients for the sums
c
         do ivec = iavec(nsh-1)+1, iavec(nsh)
            k1 = kvec(1,ivec)
            k2 = kvec(2,ivec)
            k3 = kvec(3,ivec)
            call torecsp (dble(k1),dble(k2),dble(k3),xa,xb,xc) 
            vk = dis(xa,xb,xc)
            vk2 = vk*vk
c...........Obtain the real spherical harmonic 'slm'
            call rsh (xa,xb,xc,vk,slm2,l1+maxmp)
            xpot = exp(-(pieta2*vk2)) / (omega*vk2) 
            xpot2 = xpot / halfpi
c
c.....travel all atoms in the unit cell; first, the reciprocal space
c
            do i = 1, ncel
               rr1 = xcel(i) - wa(1)
               rr2 = ycel(i) - wa(2)
               rr3 = zcel(i) - wa(3)
               l2 = 0
               do lm2 = 0, nlm(typcel(i))
                  if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                  ltot = l1 + l2
                  if (Qm(lm2,i).ne.zero) then
                     Qlm = Qm(lm2,i) * xocup(typcel(i))
                     isig = 1
                     if (mod(l2,2).eq.1) isig = -1
c....................Obtain the 't^ltot_{lm1,lm2}' function
                     m2 = lm2 - il0(l2)
                     id = ig0(lm1,lm2,ltot)
                     mmax = m3num(m1,m2,ltot)
                     tslm = zero
                     do im3 = 1, mmax
                        mtot = m3val(m1,m2,im3)
                        lmtot = il0(ltot) + mtot
                        tslm = tslm + dgaunt(id+im3)*slm2(lmtot)
                     enddo
c
                     xfact2 = isig*xpot2*coefq(l1,l2)*(twopi*vk)**ltot
     &                       *tslm/facd(ltot+ltot-1)
                     rk = twopi * (rr1*k1+rr2*k2+rr3*k3)
                     even = mod(ltot,2).eq.0
                     if (even) then ! ltot even
                        inum = ltot/2
                        isig = 1
                        if (mod(inum,2).eq.1) isig = -1
                        ewsh = ewsh + isig * xfact2 * Qlm * cos(rk)
                     else ! ltot odd
                        inum=(ltot+1)/2
                        isig=1
                        if (mod(inum,2).eq.1) isig = -1
                        ewsh = ewsh + isig * xfact2 * Qlm * sin(rk)
                     endif
                  endif
               enddo          
c
c.....real part must be computed twice, once for each K and -K
c
               dr1 = rr1 + k1
               dr2 = rr2 + k2
               dr3 = rr3 + k3
               call tocart (dr1,dr2,dr3,xa,xb,xc)
               r = dis(xa,xb,xc)
               t = r / eta
               t2 = t*t
c..............Obtain the real spherical harmonic 'slm'
               call rsh (xa,xb,xc,r,slm,l1+2*nsym(typcel(i))-2)
               l2 = 0
               do lm2 = 0, nlm(typcel(i))
                  if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                  ltot = l1 + l2
                  if (Qm(lm2,i).ne.zero) then
                     Qlm = Qm(lm2,i) * xocup(typcel(i))
                     isig = 1
                     if (mod(l2,2).eq.1) isig = -1
                     xfact1 = isig*two**ltot*coefq(l1,l2) /
     &                       (sqrpi*facd(ltot+ltot-1))
c....................Obtain the 't^ltot_{lm1,lm2}' function
                     m2 = lm2 - il0(l2)
                     id = ig0(lm1,lm2,ltot)
                     mmax = m3num(m1,m2,ltot)
                     tslm = zero
                     do im3 = 1, mmax
                        mtot = m3val(m1,m2,im3)
                        lmtot = il0(ltot) + mtot
                        tslm = tslm + dgaunt(id+im3)*slm(lmtot)
                     enddo
c....................Obtain the incomplete gamma function 'rgamm'
                     call incog (ltot+half,t2,rgamm)
                     ewsh = ewsh + xfact1 * rgamm * Qlm * tslm 
     &                           / r**(ltot+1)
                  endif
               enddo
               dr1 = rr1 - k1
               dr2 = rr2 - k2
               dr3 = rr3 - k3
               call tocart (dr1,dr2,dr3,xa,xb,xc)
               r = dis(xa,xb,xc)
               t = r / eta
               t2 = t*t
c..............Obtain the real spherical harmonic 'slm'
               call rsh (xa,xb,xc,r,slm,l1+2*nsym(typcel(i))-2)
               l2 = 0
               do lm2 = 0, nlm(typcel(i))
                  if (lm2.gt.il0(l2)+l2) l2 = l2 + 1
                  ltot = l1 + l2
                  if (Qm(lm2,i).ne.zero) then
                     Qlm = Qm(lm2,i) * xocup(typcel(i))
                     isig = 1
                     if (mod(l2,2).eq.1) isig = -1
                     xfact1 = isig*two**ltot*coefq(l1,l2) /
     &                       (sqrpi*facd(ltot+ltot-1))
c....................Obtain the 't^ltot_{lm1,lm2}' function
                     m2 = lm2 - il0(l2)
                     id = ig0(lm1,lm2,ltot)
                     mmax = m3num(m1,m2,ltot)
                     tslm = zero
                     do im3 = 1, mmax
                        mtot = m3val(m1,m2,im3)
                        lmtot = il0(ltot) + mtot
                        tslm = tslm + dgaunt(id+im3)*slm(lmtot)
                     enddo
c....................Obtain the incomplete gamma function 'rgamm'
                     call incog (ltot+half,t2,rgamm)
                     ewsh = ewsh + xfact1 * rgamm * Qlm * tslm
     &                           / r**(ltot+1)
                  endif
               enddo
            enddo ! ncel
         enddo ! ivec
         tewald = tewald + ewsh
      enddo ! while
c
c.....convergence control:
c
      if (nsh.eq.mlayer .and. abs(ewsh).gt.ewconv) then
         call error ('tewald',
     &               'convergence not reached with mlayer shells',
     &               faterr)
      endif
c
c.....end of run
c
      return
      end
