c
c-----------------------------------------------------------------------
c
      subroutine uchf (ni, typint, luw, nimpre)
c
c.....uchf - computes the unrelaxed Coulomb-Hartree-Fock correlation
c     correction to the energy.
c
      include           'implicit.inc'
      include           'global.inc' 
      include           'cont.inc'
      include           'consts.inc'
      include           'converge.inc'
      include           'corr.inc'
      include           'error.inc'
      include           'files.inc'
      include           'intg.inc'
      include           'lint.inc'
      include           'scf.inc'
      include           'strtools.inc'
      include           'types.inc'
c
      integer           ni, typint, luw, nimpre
c
c.....read (or write) uncorrelated integrals:
c
      call istupq (fcint(ni), ni, typuCHF, typint, luw, nimpre)
c
c.....begin timing:
c
      call prtimer (2,ipid,'uchf',-1)
c
c.....compute the fock matrices:
c
      call fockv (ni)
c
c.....determine correlation energy: first, store uncorrelated energies
c     for future refereesre
c
      eeffHF = eeff(ni)
      ekinHF = ekin(ni)
      eintHF = eint(ni)
c
c.....bielectronic energy:
c
      e2e = zero
      k = 0
      do i = 1, ndens(ni)
         do j = 1, i-1
            k = k + 1
            e2e = e2e + (dt(i)*p(k)*dt(j) - dos(i)*q(k)*dos(j))
         enddo
         k = k + 1
         e2e = e2e + half * (dt(i)*p(k)*dt(i) - dos(i)*q(k)*dos(i))
      enddo
c
c.....monoelectronic energy (includes interaction with the lattice):
c
      enuc     = zero
      emp      = zero
      ensr     = zero
      eesr     = zero
      eex      = zero
      epl      = zero
      ekin(ni) = zero
      eint(ni) = zero
      do i = 1, ndens(ni)
         ekin(ni) = ekin(ni) + dt(i) * t(i)
         enuc     = enuc     + dt(i) * u(i)
         eint(ni) = eint(ni) + dt(i) * v(i)
         emp      = emp      + dt(i) * vmp(i)
         ensr     = ensr     + dt(i) * vnsr(i)
         eesr     = eesr     + dt(i) * vesr(i)
         eex      = eex      + dt(i) * vex(i)
         epl      = epl      + dt(i) * pla(i)
      enddo
      enet = ekin(ni) + enuc + e2e
      eint(ni) = eint(ni) + enllr + enlsr
      epot(ni) = enuc + e2e + eint(ni)
      eeff(ni) = enet + eint(ni)
      eadd(ni) = enet + half * eint(ni)
c
c.....safety test:
c
      ecorr(ni) = eeff(ni) - eeffHF
      if (abs(eintHF-eint(ni)) .gt. piconv) call error ('uchf',
     &  'Corr. & uncorrelated interaction energies don''t coincide'//
     &  newline//' the calculation is possibly inconsistent', warning)
      if (abs(ekinHF-ekin(ni)) .gt. piconv) call error ('uchf',
     &  'Correlated & uncorrelated kinetic energies don''t coincide'//
     &  newline//' the calculation is possibly inconsistent', warning)
c
c.....print results:
c
      if (nimpre.ge.0) then
         write (luw,500)
         write (luw,510) con1(int(zn(ni))), con2(int(zn(ni)))
         write (luw,520) eeff(ni), eint(ni), eadd(ni), enet, ecorr(ni)
      endif
      call prtimer (4,ipid,'uchf',-1)
      return
c
c.....formats:
c
 500  format (//
     & 1x, '*** COULOMB-HARTREE-FOCK CORRELATION ENERGY CORRECTION :'//
     & 1x, 'uCHF mode: HF wavefunctions are not modified.')
 510  format (
     & 1x, 'Coulomb hole treatment of the J integrals'/
     & 1x, 'Cut off (CON1,CON2) parameters:', 2f15.6/)
 520  format (/
     & 1x, 'Correlated-Eeff...= ', 1pe21.14 /
     & 1x, 'Correlated-Eint...= ', 1pe21.14 /
     & 1x, 'Correlated-Eadd...= ', 1pe21.14 /
     & 1x, 'Correlated-Enet...= ', 1pe21.14 /
     & 1x, 'Correlation-Energy= ', 1pe21.14  )
      end
