c
c-----------------------------------------------------------------------
c
      subroutine        uniq (v, ind, nini, nfin)
c
c.....uniq - eliminate repeated vectors from a given set.
c
c     Returns all diferent vectors v(i,3) (i=1,n) existing in the
c     initial set v(k,3) (k=1,n). Only v(nini,3) through v(nfin,3)
c     are sorted.
c
c v(mcel,3) ... Different positions (without the action of centering
c               vectors) in the unit cell for each inequivalent atom
c               (INPUT/OUTPUT)
c ind(mcel) ... Pointer that indicates the rotation (proper or
c                  improper) that generates the current position from 
c                  the Wyckoff position of the representative atom
c                  ind < 0 -> improper rotations
c                  ind > 0 -> proper rotations
c               (INPUT/OUTPUT)
c nini      ... Initial vector v(ini,3) (INPUT/OUTPUT)
c               (Note: The ini value does not change in this routine)
c nfin      ... Final vector v(nfin,3) (INPUT/OUTPUT)
c               (Note: The nfin value can change in this routine)
c
      include           'implicit.inc'
      include           'global.inc'
      include           'error.inc'
      include           'celatm.inc'
c
      dimension         v(mcel,3), ind(mcel), nprim(mcel)
      logical           val
c
      if (nfin.gt.mcel) call error ('uniq',
     &                              'nfin > mcel!', faterr)
c.....STEP I: Obtaining the different vectors
      nprim(1)=nini
      num=1
      do i=nini+1,nfin
         val= .TRUE.
         j = nini 
         do while (val .and. j.le.(i-1))
           if (abs(v(j,1)-v(i,1)).lt.epsfrac .and.
     &         abs(v(j,2)-v(i,2)).lt.epsfrac .and.
     &         abs(v(j,3)-v(i,3)).lt.epsfrac) then
               val=.FALSE.
           endif
           j = j + 1
         enddo
         if (val) then
            num=num+1
            nprim(num)=i
         endif
      enddo
c.....STEP II: Put in order the different vectors
      do i=1,num
         iact=nini+i-1
c        iact=nini,...,nini+num-1 
         if (iact.gt.mcel) call error ('uniq',
     &                                 'iact > mcel!', faterr)
         ind(iact) = ind(nprim(i)) ! new
         do j=1,3
            v(iact,j)=v(nprim(i),j)
         enddo    
      enddo
      nfin=iact
      return
      end
