c
c-----------------------------------------------------------------------
c
      subroutine vacuo (line, lp, luw, nimpre)
c
c.....vacuo - pseudo-shell that makes atomic calculations.
c
c.....INPUT Parameter:
c     line ..... 'VACUO' order line.
c     lp ....... parsing position on this line.
c     luw ...... output logical unit.
c     nimpre ... print level of this run.
c
      include           'implicit.inc'
      include           'global.inc'
      include           'cont.inc'
      include           'corr.inc'
      include           'error.inc'
      include           'files.inc'
      include           'scf.inc'
      include           'types.inc'
      include           'vef.inc'
c
      logical           setword
      character*(ilen)  line
      integer           lp, luw, nimpre
c
c.....begin timing
c
      call prtimer (2, ipid, 'vacuo', luw)
      if (nimpre.ge.-1) write (luw,620)
c
c.....read the ion description
c
      ni = 1
      if (.not. setword (fion(ni), line, lp))
     &   call error ('vacuo', 'missing ionic filename', faterr)
      call readion (fion(ni), ni, nimpre, luw)
      if (.not. setword (fulm(ni), line, lp))
     &   call error ('vacuo', 'missing ulm filename', faterr) 
      call readulm (fulm(ni), ni, nimpre, luw)
      if (.not. setword (fcoef(ni), line, lp))
     &   call error ('vacuo', 'missing coef filename', faterr)
      call readcoef (fcoef(ni), ni, nimpre, luw) 
c
c.....compute monocentric integrals
c
      if (typcorr .eq. typrCHF) then
         if (.not. setword (fcint(ni), line, lp))
     &      call error ('vacuo', 'missing integrals filename', faterr)
         call istupq (fcint(ni), ni, typuCHF, wrint, luw, nimpre)
      else if (typcorr .eq. typuCHF) then
         if (.not. setword (fint(ni), line, lp))
     &      call error ('vacuo', 'missing integrals filename', faterr)
         if (.not. setword (fcint(ni), line, lp))
     &      call error ('vacuo', 'missing integrals filename', faterr)
         call istupq (fint(ni), ni, typHF, wrint, luw, nimpre)
      else
         if (.not. setword (fint(ni), line, lp))
     &      call error ('vacuo', 'missing integrals filename', faterr)
         call istupq (fint(ni), ni, typHF, wrint, luw, nimpre)
      endif
c
c.....perform the scf process
c
      call scfat (ni, luw, nimpre)
      if (typcorr .eq. typuCHF) then
         call uchf (ni, wrint, luw, nimpre)
c      else if (typcorr.eq.typWig .or. typcorr.eq.typGom .or.
c     &         typcorr.eq.typLYP .or. typcorr.eq.typBR) then
c         call densi (ni)
c         do lpq = 1, ndens(ni)
c            rhomat(lpq,ni) = dt(lpq)
c         enddo
c         call correl (ni, luw, nimpre)
      endif
      if (nimpre.ge.-2) call writecoef (fcoef(ni), ni)
c
c.....write out the results
c
      if (nimpre.lt.-1) then
         write (luw,790) eeff(ni)
         call prtimer (4, ipid, 'vacuo', -1)
      else
         if (typcorr .eq. typHF) then
            write (luw,625) 'HF (none)'
            write (luw,630) eeff(ni)
         else if (typcorr .eq. typuCHF) then
            write (luw,625) 'uCHF'
            write (luw,630) eeff(ni)
            write (luw,625) 'HF (none)'
            write (luw,630) eeff(ni) - ecorr(ni)
         else if (typcorr .eq. typrCHF) then
            write (luw,625) 'rCHF'
            write (luw,630) eeff(ni)
c         else if (typcorr .eq. typWig) then
c            write (luw,625) 'Wigner'
c            write (luw,630) eeff(ni)
c            write (luw,625) 'HF (none)'
c            write (luw,630) eeff(ni) - ecorr(ni)
c         else if (typcorr .eq. typGom) then
c            write (luw,625) 'Gombas'
c            write (luw,630) eeff(ni)
c            write (luw,625) 'HF (none)'
c            write (luw,630) eeff(ni) - ecorr(ni)
c         else if (typcorr .eq. typLYP) then
c            write (luw,625) 'LYP (Lee, Yang and Parr)'
c            write (luw,630) eeff(ni)
c            write (luw,625) 'HF (none)'
c            write (luw,630) eeff(ni) - ecorr(ni)
c         else if (typcorr .eq. typBR) then
c            write (luw,625) 'BR (Brual-Rothstein)'
c            write (luw,630) eeff(ni)
c            write (luw,625) 'HF (none)'
c            write (luw,630) eeff(ni) - ecorr(ni)
         endif
         call prtimer (4, ipid, 'vacuo', luw)
         write (luw,640)
      endif
c
c.....end
c
      return
c
 620  format (/
     & 1x, '***************************************************'/
     & 1x, '**                 - GPI: VACUO -                **'/
     & 1x, '** Shell designed for easy runs on isolated ions **'/
     & 1x, '**     (c) C.J.F. Solano.  October 29, 1991      **'/
     & 1x, '***************************************************'// )
 625  format (/1x, '*** CORRELATION  : ', a)
 630  format (/1x, '*** In vacuo energy of the ion   : ', 1pe22.14)
 640  format (/1x, '*** END OF VACUO RUN ****************************')
 790  format (1x,e24.16)
      end
