c
c-----------------------------------------------------------------------
c
      subroutine vefion (fion, ni, typrun, luw, nimpre)
c
c.....vefion - computes the effective potential of an ion.
c
c.....Parameters:
c     fion ...... ionic filename, used to guess a .vef file
c     ni ........ non-equivalent ion number
c     typrun .... nwint: compute effective potentials (normal call)
c                 wrint: write potentials to .vef file (no calculation)
c                 rdint: read potentials from .vef file; if .vef file is
c                        missing, compute them estimating F by F_vac
c     luw ....... print output to this LU
c     nimpre .... <0 : no print out
c                 =0 : print headings
c                 >0 : verbose (print every matrix element)
c
      include           'implicit.inc'
      include           'global.inc'
      include           'aos.inc'
      include           'cont.inc'
      include           'consts.inc'
      include           'corr.inc'
      include           'error.inc'
      include           'impurity.inc'
      include           'intg.inc'
      include           'iondens.inc'
      include           'lint.inc'
      include           'scf.inc'
      include           'stos.inc'
      include           'strtools.inc'
      include           'types.inc'
      include           'vef.inc'
      include           'symmetry.inc'
c
      character*(*)     fion
      integer           ni, typrun, luw, nimpre
c
      integer           lpq, l, ip, iq, i, i1, idm
      integer           lualloc
      real*8            tmp, smat(mdens), cmat(mdens), rmat(mdens)
      integer           lp, leng
      character*(128)   cdum, fvef
c
c.....The run begins:
c
      call prtimer (2, ipid, 'vefion', -1)
c
c.....(I) rdint/wrint: no calculation needed
c
      if (typrun.eq.rdint .or. typrun.eq.wrint) then
c
c.....(a) set the filename of the .vef file.
c
        do lp = leng(fion), 1, -1
          if (fion(lp:lp).eq.'.') then
            fvef = fion(1:lp)//'vef'
            goto 1
          endif
        enddo
        fvef = fion(1:leng(fion))//'.vef'
c
c.....(b) open the file
c
  1     lux = lualloc (lux)
        open (lux, file=fvef(1:leng(fvef)), form='unformatted',
     &        status='unknown', iostat=nios)
c
c.....(c) wrint mode
c
        if (typrun.eq.wrint) then
          if (nios .ne. 0) then
            cdum = 'Cannot write vef file:'//fvef(1:leng(fvef))
            call error ('vefion', cdum, warning)
          else
            write (lux) (rhomat(i,ni), i=1,ndens(ni))
            write (lux) (aexch(i,ni), i=1,ndens(ni))
            write (lux) (aproj(i,ni), i=1,ndens(ni))
            if (nimpre .gt. 1) then
              write (luw,520) 'Density matrix for coulomb potential:'
              write (luw,510) (rhomat(lpq,ni),lpq=1,ndens(ni))
              write (luw,520) 'Non-local exchange matrix A:'
              write (luw,510) (aexch(lpq,ni),lpq=1,ndens(ni))
              write (luw,520) 'Projection operator matrix:'
              write (luw,510) (aproj(lpq,ni),lpq=1,ndens(ni))
            endif
          endif
c
c.....(d) rdint mode
c
        else
          if (nios .ne. 0) then
            goto 3
          else
            read (lux,err=2,end=2,iostat=nios)
     &           (rhomat(i,ni), i=1,ndens(ni))
            read (lux,err=2,end=2,iostat=nios)
     &           (aexch(i,ni), i=1,ndens(ni))
            read (lux,err=2,end=2,iostat=nios)
     &           (aproj(i,ni), i=1,ndens(ni))
            if (nimpre .gt. 1) then
              write (luw,520) 'Density matrix for coulomb potential:'
              write (luw,510) (rhomat(lpq,ni),lpq=1,ndens(ni))
              write (luw,520) 'Non-local exchange matrix A:'
              write (luw,510) (aexch(lpq,ni),lpq=1,ndens(ni))
              write (luw,520) 'Projection operator matrix:'
              write (luw,510) (aproj(lpq,ni),lpq=1,ndens(ni))
            endif
          endif
        endif
        call lunalloc (lux)
        call prtimer (4, ipid, 'vefion', -1)
        return
c
c.....(e) rdint failed: compute matrices; if using Huzinaga TMSR
c     projector, estimate F as the in-vacuo operator (we assume
c     that v()=0 and pla()=0); if using the Hojer-Chung projector with
c     total F expected values, use the F^R eigenvalue as an estimate
c     (since pla()=0, we cannot get the F expected value)
c
 2      write (luw,500) nios
        close (lux)
 3      call lunalloc (lux)
        if (typpla.eq.tmsrH) then
          cdum = 'Cannot read vef file:'//fvef(1:leng(fvef))//
     &           newline//'Estimating F to initialize Vef'
          call error ('vefion', cdum, warning)
        else if (typpla.eq.totHC) then
          cdum = 'Cannot read vef file:'//fvef(1:leng(fvef))//
     &           newline//'Using F^R eigenvalues to initialize Vef'
          call error ('vefion', cdum, warning)
        else if (typpla.ne.noproj) then
          cdum = 'Cannot read vef file:'//fvef(1:leng(fvef))//
     &           newline//'Computing Vef'
          call error ('vefion', cdum, noerr)
        endif
        call istupq (fvef, ni, typcorr, nwint, luw, nimpre-2)
        call densi (ni)
        if (typpla.eq.tmsrH) call fockv (ni)
      endif
c
c.....(II) nwint: ordinary calculation of the Vef matrices
c
c.....(A) Save the rho matrix:
c
      do lpq = 1, ndens(ni)
        rhomat(lpq,ni) = dt(lpq)
      enddo
c
c.....(B) obtain the non-local exchange matrix Aexch():
c
c.....Assign the overlap matrix and initialize the exchange one
c
      do lpq = 1, ndens(ni)
        smat(lpq) = ss(lpq)
        cmat(lpq) = zero
      enddo
c
c.....Compute the exchange matrix: xk()*rho()/4, because xk was
c     symmetrized without halving it, and rho includes electrons of
c     both spins (rho is suited for the P()=J()-xk()/4 matrix)
c
      idm = 0
      do lpq = 1, ndens(ni)
        do lab = 1, lpq-1
          idm = idm + 1
          cmat(lab) = cmat(lab) + dt(lpq) * xk(idm) * fourth
          cmat(lpq) = cmat(lpq) + dt(lab) * xk(idm) * fourth
        enddo
        idm = idm + 1
        cmat(lpq) = cmat(lpq) + dt(lpq) * xk(idm) * fourth
      enddo
c
c.....Non-local exchange matrix; loop over symmetries
c
      do l = 1, nirrep(ni)
        ndim = npgao(l,ni)
        i1 = nadens(l,ni) + 1
c
c.....Obtain the inverse of the overlap matrix (S**(-1))
c
        call ynver (ndim,smat(i1))
c
c.....Evaluate the triple product A = S**(-1) * K * S**(-1)
c
        call triprod (ndim,smat(i1),cmat(i1),smat(i1))
c
c.....Store in Aexch() the A() matrix
c
        do lpq = i1, nadens(l+1,ni)
         aexch(lpq,ni) = nsub(l,ni)*cmat(lpq)
        enddo
c
c.....(C) compute the Projection operator matrix: Huzinaga projector
c     using TMSR
c
        if (typpla.eq.tmsrH) then
          do lpq = i1, nadens(l+1,ni)
            cmat(lpq) = fc(lpq) - pla(lpq)
            aproj(lpq,ni) = cmat(lpq)
            if (denlp(lpq,ni).eq.denlq(lpq,ni).and.
     &          denkp(lpq,ni).eq.denkq(lpq,ni).and.
     &          denp(lpq,ni).eq.denq(lpq,ni)) then
              rmat(lpq) = dt(lpq) * half
            else
              rmat(lpq) = dt(lpq) * fourth
            endif
          enddo
          call triprod (ndim,smat(i1),cmat(i1),rmat(i1))
          do lpq = i1, nadens(l+1,ni)
            tmp = aproj(lpq,ni)
            aproj(lpq,ni) = - cmat(lpq)
            cmat(lpq) = tmp
          enddo
          call triprod (ndim,rmat(i1),cmat(i1),smat(i1))
          do lpq = i1, nadens(l+1,ni)
            aproj(lpq,ni) =  nsub(l,ni)*(aproj(lpq,ni) - cmat(lpq))
          enddo
        endif
      enddo
c
c.....Projection operator matrix: Hojer-Chung projectors
c
      if (typpla.eq.totHC.or.typpla.eq.grpHC.or.typpla.eq.absHC) then
        do lpq = 1, ndens(ni)
          aproj(lpq,ni) = zero
        enddo
        lpq = 0
        do l = 1, nirrep(ni)
          do i = nagaos(l,ni)+1, nagaos(l+1,ni)
            if (nosh(l,ni).eq.1) then
              frocup = ngelec(i,ni) / (2*nsub(l,ni))
            else
              frocup = one
            endif
            if (ngelec(i,ni) .gt. zero) then
              tmp = zero
              if (typpla.eq.totHC) then
                lpq = nadens(l,ni)
                do ip = 1, npgao(l,ni)
                  do iq = 1, ip
                    lpq = lpq + 1
                    if (ip.eq.iq) then
                      tmp = tmp + pla(lpq) * gc(ip,i,ni) * gc(iq,i,ni)
                    else
                      tmp = tmp + pla(lpq) * 2 * gc(ip,i,ni) 
     &                                         * gc(iq,i,ni)
                    endif
                  enddo
                enddo
              endif
              lpq = nadens(l,ni)
              do ip = 1, npgao(l,ni)
                do iq = 1, ip
                  lpq = lpq + 1
                  if (typpla.eq.absHC) then
                    aproj(lpq,ni) = aproj(lpq,ni) + nsub(l,ni) * two *
     &                              gc(ip,i,ni) * abs(georb(i,ni)) *
     &                              gc(iq,i,ni) * frocup
                  else
                    aproj(lpq,ni) = aproj(lpq,ni) - nsub(l,ni) * two *
     &                              gc(ip,i,ni) * (georb(i,ni)-tmp) * 
     &                              gc(iq,i,ni) * frocup
                  endif
                enddo
              enddo
            endif
          enddo
        enddo
      endif
c
c.....noproj: ignore projection operator
c
      if (typpla.eq.noproj) then
        do lpq = 1, ndens(ni)
          aproj(lpq,ni) = zero
        enddo
      endif
c
c.....(D) Printout of the effective potential matrices
c
      if (nimpre .gt. 0) then
         write (luw,520) 'Density matrix for coulomb potential:'
         write (luw,510) (rhomat(lpq,ni),lpq=1,ndens(ni))
         write (luw,520) 'Non-local exchange matrix A:'
         write (luw,510) (aexch(lpq,ni),lpq=1,ndens(ni))
         write (luw,520) 'Projection operator matrix:'
         write (luw,510) (aproj(lpq,ni),lpq=1,ndens(ni))
      endif
c
c.....end of the routine:
c
      call prtimer (4, ipid, 'vefion', -1)
      return
c
c.....formats:
c
 500  format (//1x, 'Reading error   :',i5)
 510  format (1x, 10f12.6)
 520  format (/1x, a)
      end
